package com.japisoft.editix.action.dtdschema;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.xmlpad.XMLContainer;

import com.thaiopensource.relaxng.edit.SchemaCollection;
import com.thaiopensource.relaxng.input.InputFormat;
import com.thaiopensource.relaxng.input.dtd.DtdInputFormat;
import com.thaiopensource.relaxng.input.parse.sax.SAXParseInputFormat;
import com.thaiopensource.relaxng.output.LocalOutputDirectory;
import com.thaiopensource.relaxng.output.OutputDirectory;
import com.thaiopensource.relaxng.output.OutputFormat;
import com.thaiopensource.relaxng.output.dtd.DtdOutputFormat;
import com.thaiopensource.relaxng.output.rnc.RncOutputFormat;
import com.thaiopensource.relaxng.output.rng.RngOutputFormat;
import com.thaiopensource.relaxng.output.xsd.XsdOutputFormat;
import com.thaiopensource.util.UriOrFile;
import com.thaiopensource.xml.sax.ErrorHandlerImpl;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ConvertAction extends AbstractAction {

	private static final String DEFAULT_OUTPUT_ENCODING = "ISO-8859-1";
	private static final int DEFAULT_LINE_LENGTH = 72;
	private static final int DEFAULT_INDENT = 2;

	private final ErrorHandlerImpl eh = new ErrorHandlerImpl();

	public void actionPerformed(ActionEvent e) {
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if (container == null)
			return;

		if ( EditixFactory.mustSaveDialog( container ) ) {
			return;
		}

		ActionModel.activeActionById(ActionModel.SAVE, e);
		if (!ActionModel.LAST_ACTION_STATE)
			return;

		String type = container.getDocumentInfo().getType();
		boolean rng = "RNG".equals(type);

		InputFormat in = null;
		if (rng)
			in = new SAXParseInputFormat();
		else
			in = new DtdInputFormat();

		String param = (String) getValue("param");
		if (param == null)
			return;

		OutputFormat of = null;

		if (param.equalsIgnoreCase("dtd"))
			of = new DtdOutputFormat();
		else if (param.equalsIgnoreCase("rng"))
			of = new RngOutputFormat();
		else if (param.equalsIgnoreCase("xsd"))
			of = new XsdOutputFormat();
		else if (param.equalsIgnoreCase("rnc"))
			of = new RncOutputFormat();
		else {
			System.err.println("UNKNOWN OUTPUT = " + param);
			return;
		}

		JFileChooser chooser = new JFileChooser();
		chooser.setDialogTitle("Choose your result file");

		final String EXT = param;
		final String DES = "(*." + EXT + ")";

		
		chooser.setFileFilter(
			new FileFilter() {

				public boolean accept(File f) {
					return f.isDirectory() || f.toString().endsWith( EXT );
				}

				public String getDescription() {
					return DES;
				}
			} );
		
		File outputFile = null;
		if (chooser.showSaveDialog(container.getView()) == JFileChooser.APPROVE_OPTION) {
			outputFile = chooser.getSelectedFile();
		} else
			return;

		try {
			if ( container.hasErrorMessage() ) {
				container.getErrorManager().notifyError( "Cannot convert if the current document contains errors" );
				return;
			}

			SchemaCollection sc;
			sc =
				in
					.load(
						UriOrFile.toUri(container.getCurrentDocumentLocation()),
						new String[] {
			}, param, eh);

			OutputDirectory od =
				new LocalOutputDirectory(
					sc.getMainUri(),
					outputFile,
					param,
					DEFAULT_OUTPUT_ENCODING,
					DEFAULT_LINE_LENGTH,
					DEFAULT_INDENT);
			
			of.output(sc, od, new String[] {
			}, container.getDocumentInfo().getType().toLowerCase(), eh);

			ActionModel.activeActionById(
				ActionModel.OPEN,
				e,
				outputFile.toString() );

		} catch (Exception exc) {
			container.getErrorManager().initErrorProcessing();
			container.getErrorManager().notifyError( "Can't convert to " + param + " : Check your schema" );
			container.getErrorManager().stopErrorProcessing();
		}
	}

}
