package com.japisoft.editix.action.dtdschema;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JOptionPane;

import com.japisoft.dtdparser.DTDParser;
import com.japisoft.dtdparser.node.DTDNode;
import com.japisoft.dtdparser.node.ElementDTDNode;
import com.japisoft.dtdparser.node.RootDTDNode;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class GenerateTemplateAction extends AbstractAction {

	public void actionPerformed(ActionEvent arg0) {

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container.getCurrentDocumentLocation() == null ) {
			EditixFactory.buildAndShowWarningDialog( "Please save your DTD/Schema before generating a new XML document" );
			return;
		}

		String type = container.getDocumentInfo().getType();

		List<LabelWrapper> elements = new ArrayList<LabelWrapper>();

		if ( "DTD".equals( type ) ) {
			
			try {
								
				DTDParser parser = new DTDParser();
				parser.parse( new StringReader( container.getText() ) );
				RootDTDNode root = parser.getDTDElement();				
				for  (int i = 0; i < root.getDTDNodeCount(); i++ ) {
					if ( root.getDTDNodeAt( i ).isElement() ) {
						elements.add( new LabelWrapper( ( ElementDTDNode )root.getDTDNodeAt( i ) ) );
					}
				}

				if ( elements.size() > 0 ) {
				
					LabelWrapper rootElement = (LabelWrapper)JOptionPane.showInputDialog(
							EditixFrame.THIS, "Choose your root element",
							"XML Document root",
							JOptionPane.QUESTION_MESSAGE, 
							null, 
							elements.toArray(), 
							elements.get( 0 )
					);				
					
					if ( rootElement != null ) {
						
						try {
						
							GenerateTemplateFromElementAction.generateXMLDocumentFromDTD( 
									( ( ElementDTDNode )rootElement.source ).getName(), container.getCurrentDocumentLocation() );
							
						} catch( Throwable th ) {
							
							EditixFactory.buildAndShowErrorDialog( "Can't generate your XML document : " + th.getMessage() );
							
						}
	
					}
					
				} else {
					
					EditixFactory.buildAndShowWarningDialog( "Can't find an element definition inside your DTD ?" );
					
				}

			} catch( IOException exc ) {
				EditixFactory.buildAndShowErrorDialog( "Can't parse your DTD, please fix it" );
			}

		} else
		if ( "XSD".equals( type ) ) {

			FPNode root = container.getRootNode();
			if ( root == null ) {
				EditixFactory.buildAndShowErrorDialog( "Can't parse your Schema, please fix it" );
			} else {

				for ( int i = 0; i < root.getChildCount(); i++ ) {
					FPNode node = ( FPNode )root.getChildAt( i );
					if ( node.matchContent( "element" ) ) {
						elements.add( new LabelWrapper( node ) );
					}
				}

				if ( elements.size() > 0 ) {

					LabelWrapper rootElement = (LabelWrapper)JOptionPane.showInputDialog(
							EditixFrame.THIS, "Choose your root element",
							"XML Document root",
							JOptionPane.QUESTION_MESSAGE, 
							null, 
							elements.toArray(), 
							elements.get( 0 )
					);				
					
					if ( rootElement != null ) {
						
						try {

							GenerateTemplateFromElementAction.generateXMLDocumentFromXSD( ( FPNode )rootElement.source, container.getCurrentDocumentLocation() );

						} catch( Throwable th ) {
						
							EditixFactory.buildAndShowErrorDialog( "Can't generate your XML document : " + th.getMessage() );
							
						}

					}					

				} else {
					
					EditixFactory.buildAndShowWarningDialog( "Can't find a global element definition inside your Schema ?" );
					
				}
				
			}

		}

	}

	/* ----------------------------------------------------------------------------- */
	
	class LabelWrapper {
		Object source;	
		
		LabelWrapper( Object source ) {
			this.source = source;
		}
		
		@Override
		public String toString() {
			if ( source instanceof FPNode ) {
				return ( ( FPNode )source ).getAttribute( "name" );
			} else
			if ( source instanceof ElementDTDNode ) {
				return ( ( ElementDTDNode )source ).getName();
			} else
				return super.toString();
		}
	}

}
