package com.japisoft.editix.action.dtdschema.generator.transformer;

import java.util.ArrayList;
import java.util.Vector;

import com.japisoft.editix.action.dtdschema.generator.MetaAttribute;
import com.japisoft.editix.action.dtdschema.generator.MetaNode;
import com.japisoft.editix.action.dtdschema.generator.MetaObject;
import com.japisoft.editix.action.dtdschema.generator.Transformer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RelaxNGTransformer extends AbstractTransformer {
	
	@Override
	protected void initTransform(MetaNode root, StringBuffer sb) {
		sb.append( "<grammar xmlns=\"http://relaxng.org/ns/structure/1.0\" datatypeLibrary=\"http://www.w3.org/2001/XMLSchema-datatypes\">\n" );
		sb.append( "\t<start>\n" );
		sb.append( "\t\t<ref name=\"" ).append( root.getName() ).append( ".element\"/>\n" );		
		sb.append( "\t</start>\n" );
	}
	
	@Override
	protected void closeTransform(MetaNode root, StringBuffer sb) {
		sb.append( "</grammar>\n" );
	}
	
	@Override
	protected void generateMetaNode( MetaNode element, StringBuffer sb ) {
		sb.append( "\n\t<define name=\"" ).append( element.getName() ).append( ".element\">\n" );
		sb.append( "\t\t<element name=\"" ).append( element.getName() ).append( "\">\n" );
		
		if ( element.acceptText() ) {
			sb.append( "\t\t\t<text/>\n" );
		} else
			if ( element.getChildren().size() == 0 ) {
				sb.append( "\t\t\t<empty/>\n" );
			}

		// Add attributes

		if ( element.hasAttributes() ) {
			Vector atts = element.getAttributes();
			for ( int i = 0; i < atts.size(); i++ ) {
				MetaAttribute att = ( MetaAttribute )atts.get( i );
				if ( !att.isAlways() )
					sb.append( "\t\t\t<optional>\n" );

					sb.append( "\t\t\t\t<attribute name=\"" ).append( att.getName() ).append( "\">\n" );
					if ( !att.getType().equals( MetaObject.TEXT_TYPE ) )
						sb.append( "\t\t\t\t\t<data type=\"" ).append( SchemaTransformer.translateType( att.getType() ) ).append( "\"/>\n" );
					else
						sb.append( "\t\t\t\t<text/>\n" );
					sb.append( "\t\t\t\t</attribute>\n" );
				
				if ( !att.isAlways() )
					sb.append( "\t\t\t</optional>\n" );
				
			}
		}
		
		if ( element.getChildren().size() > 0 ) {
			sb.append( "\t\t\t<zeroOrMore>\n" );
			
			if ( sequenceMode ) {
				sb.append( "\t\t\t\t<group>\n" );
			} else {
				sb.append( "\t\t\t\t<choice>\n" );
			}

			for ( int i = 0; i < element.getChildren().size(); i++ ) {
				MetaNode node = ( MetaNode )element.getChildren().get( i );
				sb.append( "\t\t\t\t\t<ref name=\"" ).append( node.getName() ).append( ".element\"/>\n" );
			}

			if ( sequenceMode ) {
				sb.append( "\t\t\t\t</group>\n" );
			} else {
				sb.append( "\t\t\t\t</choice>\n" );
			}
			sb.append( "\t\t\t</zeroOrMore>\n" );
		}

		sb.append( "\t\t</element>\n" );
		sb.append( "\t</define>\n" );
	}

	public String getType() {
		return "RNG";
	}

	public boolean hasVersion() {
		return true;
	}

}
