package com.japisoft.editix.action.file;

import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import org.apache.commons.vfs.FileSystemException;
import org.apache.commons.vfs.FileSystemManager;
import org.apache.commons.vfs.VFS;

import com.japisoft.editix.action.panels.PanelAction;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.panels.universalbrowser.ZIPBrowserPanel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.p3.Manager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BrowseZIPAction extends PanelAction {

	static class CustomFileFilter extends FileFilter {
		
		private String description;
		private ArrayList exts = null;
		
		CustomFileFilter( String extDesc ) throws Throwable {
			StringTokenizer st = new StringTokenizer( extDesc, "-" );
			String title = st.nextToken();
			description = title.substring( 1, title.length() - 1 );
			while ( st.hasMoreTokens() ) {
				if ( exts == null )
					exts = new ArrayList();
				String star = "." + st.nextToken().toLowerCase(); 
				exts.add( star );
				description += " (*" + star + ")";
			}
		}

		public boolean accept(File f) {
			if ( f.isDirectory() )
				return true;
			String file = f.toString().toLowerCase();
			if ( exts != null )
				for ( int i = 0; i < exts.size(); i++ )
					if ( file.endsWith( ( String )exts.get( i ) ) )
						return true;
			return false;
		}
		public String getDescription() {
			return description;
		}
	}

	public static void addFileFilterForZipArchives( JFileChooser chooser, boolean selectFirstOne ) {
		String zipFiles = 
			Preferences.getPreference( "zipbrowser", "zip extensions", "[Default archive]-zip;[Java Files]-jar-sar-ear-par-ejb3-war;[Microsoft Office documents]-docx-xlsx-pptx;[Open office documents]-odt-ods-odp-odg-odf" );

		StringTokenizer st = new StringTokenizer( zipFiles, ";" );
		FileFilter firstOne = null;
		String val = null;
		while ( st.hasMoreTokens() ) {
			try {
				val = st.nextToken();
				CustomFileFilter cff = new CustomFileFilter( val );
				chooser.addChoosableFileFilter( cff );
				if ( firstOne == null )
					firstOne = cff;
			} catch (Throwable e1) {
				EditixFactory.buildAndShowWarningDialog( "Wrong zipbrowser preference value :" + val + " ??" );
			}
		}
		
		if ( firstOne != null && selectFirstOne )
			chooser.setFileFilter( firstOne );
	}

	public static boolean isFileArchive( String path ) {
		int i = path.lastIndexOf( "." );
		if ( i == -1 )
			return false;
		String ext = path.substring( i + 1 ).toLowerCase();
		String zipFiles = 
			Preferences.getPreference( "zipbrowser", "zip extensions", "[Default archive]-zip;[Java Files]-jar-sar-ear-par-ejb3-war;[Microsoft Office documents]-docx-xslx-pptx;[Open office documents]-odt-ods-odp-odg-odf" );
		StringTokenizer st = new StringTokenizer( zipFiles.toLowerCase(), ";-" );
		while ( st.hasMoreTokens() ) {
			if ( ext.equals( st.nextToken() ) )
				return true;
		}
		return false;
	}

	public void actionPerformed(ActionEvent e) {

		if ( Manager.isFree() ) {

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			return;
			
		}			

		//		
		
		JFileChooser chooser = new JFileChooser();
		addFileFilterForZipArchives( chooser, true );
		
		if ( chooser.showOpenDialog( EditixFrame.THIS ) != 
			JFileChooser.APPROVE_OPTION ) {			
			return;
		}

		File selection = chooser.getSelectedFile();
		browse( selection );
		
		//
	}
	
	public void browse( File selection ) {

		try {
			FileSystemManager fsManager = VFS.getManager();

			// VFS URI format
			String uri = 
				"zip" + ":file://" + selection.toString();
			
			( ( ZIPBrowserPanel )preparePanel() ).browse(
					uri,
					null,
					null );

		} catch (FileSystemException e1) {
			EditixFactory.buildAndShowErrorDialog( "Inner Error : " + e1.getMessage() );
			return;
		}

		alwaysShown = true;
		super.actionPerformed( null );
	}

}
