package com.japisoft.editix.action.file;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;

import com.japisoft.editix.project.ProjectManager;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.panels.PanelManager;
import com.japisoft.framework.application.descriptor.InterfaceBuilder;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class OpenProjectAction extends AbstractAction {

	private boolean canCloseProject( ActionEvent e ) {
		CloseAllAction.RETURN_STATUS = true;
		return CloseAllAction.RETURN_STATUS;
	}

	public void actionPerformed( ActionEvent e ) {
		// Check to save
		
		if ( getValue( "param" ) != null ) {
			if ( !canCloseProject( e ) )
				return;
			openProject( true, "" + getValue( "param" ) );
			return;
		}
		
		JFileChooser fc = EditixFactory.buildProjectFileChooser();
		if ( fc.showOpenDialog( EditixFrame.THIS.getComponent( 0 ) ) == JFileChooser.APPROVE_OPTION ) {
			File f = fc.getSelectedFile();
			if ( !f.exists() || !canCloseProject( e ) ) {
				if ( !f.exists() ) {
					EditixFactory.buildAndShowErrorDialog("Can't load " + f);
				}
				return;			
			}
			openProject( false, f.toString());
		} else {
			EditixFrame.THIS.checkOneContainer();
		}
	}
	
	public static void openProject( boolean param, String file) {
		// Ask for the ProjectPanel to be visible
		PanelManager.initByAction( "projectManager" );
		boolean ok = ProjectManager.openProject( file );

		if ( !ok ) {
			EditixFactory.buildAndShowErrorDialog( "Can't load " + file );
			EditixFrame.THIS.getBuilder().removeMenuItemForParam(
					InterfaceBuilder.MENU_RECENT_PROJECT, file );			
			EditixFrame.THIS.checkOneContainer();
			return;
		} else {
			// Open the project panel
			
		}

		if ( !param ) {
			updateOpenRecent( file );
		}
	}

	static void updateOpenRecent( String file ) {
		if ( file != null ) {
			Action a = new OpenProjectAction();
			a.putValue( Action.NAME, file );
			a.putValue( "param", file );
			a.putValue( "iconPath", "images/package.png" );
	
			EditixFrame.THIS.getBuilder().insertMenuItemAtFirst(
				InterfaceBuilder.MENU_RECENT_PROJECT,
				a,
				20 );
		}
	}

}
