package com.japisoft.editix.action.file.export;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Stack;
import java.util.Map.Entry;

import javax.xml.parsers.SAXParser;

import org.apache.xerces.jaxp.SAXParserFactoryImpl;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ClassGenerator implements ContentHandler {

	private HashMap mapMetaClass = new HashMap();

	public ClassGenerator(String xmlFile, String outputDir, String pack)
			throws Throwable {
		SAXParserFactoryImpl impl = new SAXParserFactoryImpl();
		SAXParser p = impl.newSAXParser();
		p.getXMLReader().setContentHandler(this);
		p.getXMLReader().parse(xmlFile);

		File f = new File(outputDir);
		Iterator itkeys = mapMetaClass.entrySet().iterator();
		while (itkeys.hasNext()) {
			String key = (String) ((Entry) itkeys.next()).getKey();
			JavaMetaClass mc = (JavaMetaClass) mapMetaClass.get(key);
			File _f = new File(outputDir, Tools.toClassName(key) + ".java");
			if (!_f.exists()) {
				PrintWriter pw = new PrintWriter(new FileWriter(_f));
				try {
/*					System.out.println("Process " + Tools.toClassName(key)
							+ ".java"); */
					mc.write(pw, pack);
				} finally {
					pw.close();
				}
			} /* else
				System.out.println("Ignoring " + _f); */
		}
	}

	private JavaMetaClass currentMetaClass = null;

	private Stack stackOfMetaClass = new Stack();

	public void characters(char[] arg0, int arg1, int arg2) throws SAXException {
		// Check for non empty content
		String s = new String(arg0, arg1, arg2);
		for (int i = 0; i < s.length(); i++) {
			if (!Character.isWhitespace(s.charAt(i)))
				currentMetaClass.setText();
		}
	}

	public void startElement(String arg0, String arg1, String arg2,
			Attributes arg3) throws SAXException {

		String localName = Tools.getLocalNameForQName(arg2);

		if (currentMetaClass != null) {
			currentMetaClass.addContentElement(localName);
			stackOfMetaClass.push(currentMetaClass);
		}
		currentMetaClass = (JavaMetaClass) mapMetaClass.get(localName);
		if (currentMetaClass == null) {
			currentMetaClass = new JavaMetaClass(localName);
			mapMetaClass.put(localName, currentMetaClass);
		}
		for (int i = 0; i < arg3.getLength(); i++) {
			currentMetaClass.setAttribute(Tools.getLocalNameForQName(arg3
					.getQName(i)), arg3.getValue(i));
		}
	}

	public void endElement(String arg0, String arg1, String arg2)
			throws SAXException {
		try {
			currentMetaClass = (JavaMetaClass) stackOfMetaClass.pop();
		} catch (RuntimeException e) {
		}
	}

	public void endDocument() throws SAXException {
	}

	public void endPrefixMapping(String arg0) throws SAXException {
	}

	public void ignorableWhitespace(char[] arg0, int arg1, int arg2)
			throws SAXException {
	}

	public void processingInstruction(String arg0, String arg1)
			throws SAXException {
	}

	public void setDocumentLocator(Locator arg0) {
	}

	public void skippedEntity(String arg0) throws SAXException {
	}

	public void startDocument() throws SAXException {
	}

	public void startPrefixMapping(String arg0, String arg1)
			throws SAXException {
	}

	public static void main(String[] args) throws Throwable {
		//ClassGenerator cg = new ClassGenerator("src/multiple_textfield.xdp",
		//		"src/com/crionics/xfa/elements", "com.crionics.xfa.elements");
	}
}
