package com.japisoft.editix.action.file.export;

import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ExportJavaAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {

		if ( Manager.isFree() ) {

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			return;
			
		}		
		
		//		
		XMLContainer container = 
			EditixFrame.THIS.getSelectedContainer();

		if ( container == null ) {
			EditixFactory.buildAndShowErrorDialog( "No document" );
			return;
		}

		if ( container.getCurrentDocumentLocation() == null ) {
			EditixFactory.buildAndShowErrorDialog( "Your must save your document" );
			return;			
		}

		JFileChooser f = new JFileChooser();
		f.setMultiSelectionEnabled( false );
		
		f.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
		if ( f.showSaveDialog( EditixFrame.THIS ) == 
			JFileChooser.APPROVE_OPTION ) {

			String pack = EditixFactory.buildAndShowInputDialog( "Java package ?" );
			if ( pack == null )
				pack = "editix";
			
			try {
				ClassGenerator
					cg = new ClassGenerator(
							container.getCurrentDocumentLocation(),
							f.getSelectedFile().toString(),
							pack
					);

				// Copy the parser too
				copyJavaFile( "javatemplate/AbstractElement.javat",f.getSelectedFile(),pack );
				copyJavaFile( "javatemplate/ObjectBuilder.javat",f.getSelectedFile(),pack );
				copyJavaFile( "javatemplate/SaxHandler.javat",f.getSelectedFile(),pack );
				copyJavaFile( "javatemplate/Tools.javat",f.getSelectedFile(),pack );

			} catch (Throwable e1) {

				EditixFactory.buildAndShowErrorDialog( 
						"Can't create classes : " + e1.getMessage() );

			}

		}
		//
	}

	private void copyJavaFile( String name, File dirDest, String newpackage ) {
		URL u = ClassLoader.getSystemClassLoader().getResource( name );
		if ( u == null ) {
			ApplicationModel.debug("Can't write java class " + name );
		} else {
			try {
				InputStream input = u.openStream();
				InputStreamReader reader = new InputStreamReader( input );
				BufferedReader br = new BufferedReader( reader );
				try {
					// Skip the package
					br.readLine();
					
					int i = name.indexOf( "/" );
					name = name.substring( i + 1 );
					
					BufferedWriter bw 
						= new BufferedWriter( 
								new FileWriter( 
										new File( 
												dirDest, 
												name.substring( 0, name.length() - 1 ) ) ) );
					try {
						if ( newpackage != null ) {
							bw.write( "package " + newpackage + ";" );
							bw.newLine();
						}
						String line = null;
						while ( ( line = br.readLine() ) != null ) {
							bw.write( line );
							bw.newLine();
						}
					} finally {
						bw.close();
					}
				} finally {
					br.close();
				}
			} catch (IOException e) {
			}
			
		}
	}

}
