package com.japisoft.editix.action.file.imp;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JDBCDriverModel {

	private List<JDBCDriver> data = null;
	
	public void addDriver( JDBCDriver driver ) {
		if ( data == null )
			data = new ArrayList<JDBCDriver>();
		data.add( driver );
		fireModelUpdated();
	}
	
	public void removeDriver( JDBCDriver driver ) {
		if ( data != null )
			data.remove( driver );
		fireModelUpdated();
	}

	private JDBCDriverModelListener listener = null;
	
	public void setListener( JDBCDriverModelListener listener ) {
		this.listener = listener;
	}

	private void fireModelUpdated() {
		if ( listener != null ) {
			listener.modelUpdated();
		}
	}

	public int size() {
		if ( data == null ) {
			return 0;
		}
		return data.size();
	}

	public JDBCDriver getDriver( int index ) {
		if ( data == null )
			return null;
		return data.get( index );
	}

	public void write( File output ) throws Exception {
		if ( data != null ) {
			if ( data.size() > 0 ) {
				Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
				Element root = doc.createElement( "jdbcdrivers" );
				doc.appendChild( root );
				for ( int i = 0; i < size(); i++ ) {
					root.appendChild( getDriver( i ).toXML( doc ) );
				}
				Transformer t = TransformerFactory.newInstance().newTransformer();
				t.transform( 
					new DOMSource( doc ), 
					new StreamResult( output ) 
				);
			}
		}
	}

	public void read( File input ) throws Exception {
		DocumentBuilder db = 
			DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document doc = db.parse( input );
		Element root = doc.getDocumentElement();
		data = null;
		NodeList nl = root.getChildNodes();
		for ( int i = 0; i < nl.getLength(); i++ ) {
			Node n = nl.item( i );
			if ( n instanceof Element ) {
				addDriver( 
					new JDBCDriver( 
						( Element )n ) 
				);
			}
		}
	}

}
