package com.japisoft.editix.action.file.project;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import com.japisoft.editix.action.file.BrowseZIPAction;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.toolkit.FileToolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ImportProjectAction extends AbstractAction {

	@Override
	public void actionPerformed(ActionEvent e) {
		JFileChooser fileChooser = new JFileChooser();
		BrowseZIPAction.addFileFilterForZipArchives( fileChooser, false );
		FileFilter[] ff = fileChooser.getChoosableFileFilters();

		if ( ff.length > 1 ) {
			fileChooser.setFileFilter( ff[ 1 ] );
		}
		if ( fileChooser.showOpenDialog( EditixFrame.THIS ) == 
				JFileChooser.APPROVE_OPTION ) {

			boolean overwrite = true;
			
			
			// Store by default the new project inside the home/.editix directory
			File f = fileChooser.getSelectedFile();
			if ( f == null )
				return;
			String fileName = f.getName();
			File projectPath = ApplicationModel.getAppFile(fileName);
			if ( !projectPath.mkdirs() ) {
				if ( projectPath.exists() ) {
					if ( !EditixFactory.buildAndShowConfirmDialog( "This project exists, do you want to overwrite it ?" ) ) {
						overwrite = false;
					}
				} else {
					EditixFactory.buildAndShowErrorDialog( "Can't store the project here ? " + projectPath );
					return;
				}
			}
			
			try {
				
				if ( overwrite ) {
				
					FileToolkit.unzip( f, projectPath );
					
					/*
					
					// Unzip the content
					ZipFile z = new ZipFile( f );
					try {
						Enumeration<? extends ZipEntry> entries = z.entries();
						while ( entries.hasMoreElements() ) {
							ZipEntry ze = entries.nextElement();
							File zef = new File( projectPath, ze.getName() );
							
							if ( ze.isDirectory() ) {
								zef.mkdirs();
							} else {
								// Write the file
								FileOutputStream out = new FileOutputStream( zef );
								try {
									InputStream i = z.getInputStream( ze );
									int c;
									while ( ( c = i.read() ) != -1 )
										out.write( c );								
								} finally {
									out.close();
								}
							}
						}
						
					} finally {
						z.close();
					} */
					
					
					
				}
				
				( ( NewProjectAction )ActionModel.restoreAction( "newp" ) ).loadProject( projectPath );
				
			} catch( Exception exc ) {
				EditixFactory.buildAndShowErrorDialog( "Can't use this file : " + exc.getMessage() );
				return;
			}
			
		}
	}

	public static void archiveProject( File projectPath, File archive ) throws Exception {
		Path pp = Paths.get( projectPath.toString() );
		ZipOutputStream z = new ZipOutputStream( new FileOutputStream( archive ) );
		Files.walk( pp ).filter( f -> !Files.isDirectory( f ) ).forEach(
			f -> {
				ZipEntry ze = new ZipEntry( pp.relativize( f ).toString() );
				try {
					z.putNextEntry( ze );
					Files.copy( f, z );
					z.closeEntry();
				} catch( IOException exc ) {
					
				}
		} );
		z.close();
	}
	
}
