package com.japisoft.editix.action.options;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;

import javax.swing.AbstractAction;

import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.main.Main;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.dialog.command.ExternalToolPane;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ExternalToolAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {

		ExternalToolPane frame = new ExternalToolPane();

		frame.setMacro(
				new String[] {
						"Current Document path - ${cf}",
						"Current Directory Path - ${cd}",
						"Current Document file name - ${cn}",
						"Home Directory - ${home}"
				} );

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if (container != null) {
			String documentLocation = container.getCurrentDocumentLocation();
			if (documentLocation != null) {
				frame.setMacro(ExternalToolPane.CURRENT_PATH_MACRO,
						documentLocation);
				File f = new File(documentLocation);
				frame.setMacro(ExternalToolPane.CURRENT_DIRECTORY_MACRO, f
						.getParent());
				int i = documentLocation.lastIndexOf("/");
				if (i == -1)
					i = documentLocation.lastIndexOf("\\");
				if (i != -1)
					frame.setMacro(ExternalToolPane.CURRENT_FILENAME_MACRO,
							documentLocation.substring(i + 1));
				else
					frame.setMacro(ExternalToolPane.CURRENT_FILENAME_MACRO,
							documentLocation);
			}
		}

		// Reload previous choice
		File f = EditixApplicationModel.getAppUserPath();
		if (f != null) {
			File f2 = new File(f, "tools.dat");

			if (f2.exists()) {
				try {
					ObjectInputStream input = new ObjectInputStream(
							new FileInputStream(f2));
					try {
						ArrayList list = (ArrayList) input.readObject();
						frame.setActionItems(list);
					} finally {
						input.close();
					}
				} catch (IOException exc) {
					exc.printStackTrace();
				} catch (ClassNotFoundException exc2) {
					exc2.printStackTrace();
				}
			}
		}

		DialogManager.showDialog( 
				EditixFrame.THIS,
				"External Tools", 
				"External Tools", 
				"Call an external tool",
				null,
				frame );				
				
		if (f != null) {
			File f2 = new File(f, "tools.dat");
			try {

				ObjectOutputStream output = new ObjectOutputStream(
						new FileOutputStream(f2));
				try {
					output.writeObject(frame.getActionsItems());
				} finally {
					output.close();
				}

			} catch (IOException exc) {
				exc.printStackTrace();
			}
		}
	}

}
