package com.japisoft.editix.action.script;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import com.japisoft.editix.script.BasicScript;
import com.japisoft.editix.script.Script;
import com.japisoft.editix.script.ScriptModel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ui.table.ExportableTable;
import com.japisoft.framework.ui.toolkit.FileManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ScriptManagerPanel extends JPanel implements TableModel {

	private JTable t = null;
	
	public ScriptManagerPanel() {
		setLayout( new BorderLayout() );
		JToolBar tb = new JToolBar();
		tb.setFloatable( false );
		tb.add( new AddAction() );
		tb.add( new RemoveAction() );
		add( tb, BorderLayout.NORTH );
		add( new JScrollPane( t = new ExportableTable( this ) ) );
	}
	
	// ---------------------------------------------------------------------------------------
	
	private TableModelListener listener = null;
	
	public void addTableModelListener(TableModelListener l) {
		this.listener = l;
	}

	public Class<?> getColumnClass(int columnIndex) {
		return String.class;
	}

	public int getColumnCount() {
		return 3;
	}

	public String getColumnName(int columnIndex) {
		if ( columnIndex == 0 )
			return "Name";
		if ( columnIndex == 1 )
			return "Shortcut";
		return "Path";
	}

	public int getRowCount() {
		return ScriptModel.getInstance().getScripts().size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		Script s = ScriptModel.getInstance().getScripts().get( rowIndex );
		if ( columnIndex == 0 )
			return s.getName();
		if ( columnIndex == 1 )
			return s.getShortkey();
		return s.getPath();
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return columnIndex <= 1;
	}

	public void removeTableModelListener(TableModelListener l) {
		this.listener = null;
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		BasicScript s = ( BasicScript )ScriptModel.getInstance().getScripts().get( rowIndex );
		if ( columnIndex == 0 )
			s.setName( ( String )aValue );
		if ( columnIndex == 1 )
			s.setShortkey( ( String )aValue );
		save();
	}

	private void save() {
		try {
			ScriptModel.getInstance().save();
		} catch( Exception exc ) {
			ApplicationModel.fireApplicationValue( "error", "Can't save the script data" );
			ApplicationModel.debug( exc );
		}		
	}
	
	class AddAction extends AbstractAction {
		public AddAction() {
			putValue( Action.NAME, "New..." );
		}
		public void actionPerformed(ActionEvent e) {
			File path = FileManager.getSelectedFile(true, "js", "JavaScript file (*.js)" );
			if ( path != null ) {
				ScriptModel.getInstance().getScripts().add( new BasicScript( "MyScript", path, "" ) );
				listener.tableChanged( new TableModelEvent(ScriptManagerPanel.this ) );
				save();
			}
		}
	}
	
	class RemoveAction extends AbstractAction {
		public RemoveAction() {
			putValue(Action.NAME,"Remove" );
		}
		public void actionPerformed(ActionEvent e) {
			int row = t.getSelectedRow();
			if ( row >= 0 ) {
				ScriptModel.getInstance().getScripts().remove( row );
				listener.tableChanged( new TableModelEvent(ScriptManagerPanel.this ) );
				save();
			}
		}
	}
	
}
