package com.japisoft.editix.action.xml;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.XMLDocumentInfo;
import com.japisoft.xmlpad.action.xml.CheckableAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CheckAllAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {

		XMLContainer currentContainer = EditixFrame.THIS.getSelectedContainer();
		
		int nbDocumentChecked = 0;
		int nbErrorFound = 0;

		int indiceFirstError = -1;

		for ( int i = 0; i < 
			EditixFrame.THIS.getXMLContainerCount(); i++ ) {
			XMLContainer editor = EditixFrame.THIS.getXMLContainer( i );
			if ( editor == null )
				continue;
			XMLDocumentInfo info = editor.getDocumentInfo();
			String checkerId = info.getParamValue( "checkerid" );
			if ( checkerId != null ) {
				Action a =
					ActionModel.restoreAction( checkerId );
				if ( a != null ) {
					if ( a instanceof CheckableAction ) {
						CheckableAction ca = ( CheckableAction )a;
						boolean error = 
							!ca.checkDocument( editor, true );
						if ( error ) {
							nbErrorFound++;
							if ( indiceFirstError == -1 ) {
								if ( editor != currentContainer )
									indiceFirstError = i;
							}
						}
						nbDocumentChecked++;
					} else
						System.out.println( 
								"Action " + checkerId + " should be a CheckableAction ?" );
				} else
					System.out.println( 
							"Can't find action " + checkerId );
			}
		}

		if ( indiceFirstError == -1 ) {
		
			EditixFactory.buildAndShowInformationDialog( 
					nbDocumentChecked + 
					" document(s) checked\n" + 
						nbErrorFound + 
							" error(s) found" );
		
		} else {
			
			if ( EditixFactory.buildAndShowConfirmDialog( nbErrorFound + " error(s) found\nGo to the first error ?") ) {
				EditixFrame.THIS.activeXMLContainer( indiceFirstError );
			}
			
		}
	}

}
