package com.japisoft.editix.action.xml;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.SwingUtilities;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixStatusBar;
import com.japisoft.framework.actions.SynchronizableAction;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class LockedTagAction extends AbstractAction implements SynchronizableAction {

	public void actionPerformed(ActionEvent e) {
		
		if ( Manager.isFree() ) {
			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			return;
		}			

		if ( EditixFrame.THIS == null )
			return;
		
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;
		
		boolean state = container.getDocumentIntegrity().isProtectTag();
		if ( state ) {
			// Disabled it
			container.getDocumentIntegrity().setProtectTag( false );
			EditixStatusBar.ACCESSOR.setXPathLocation( "Tag update enabled" );
		} else {
			// Enabled it
			container.getDocumentIntegrity().setProtectTag( true );
			EditixStatusBar.ACCESSOR.setXPathLocation( "Tag update disabled" );
		}
		// Save it
		container.setProperty( getClass().getName(), "" + !state );
		refreshState( container );		
	}

	public void synchronizeState(Object source) {
		if ( source instanceof XMLContainer ) {
			XMLContainer container = ( XMLContainer )source;
			
			boolean state = refreshState( container );
			
			// Check for state
			if ( state ) {
				if ( !container.getDocumentIntegrity().isProtectTag() ) {
					container.getDocumentIntegrity().setProtectTag( true );
				}
			} else {
				if ( container.getDocumentIntegrity().isProtectTag() ) {
					container.getDocumentIntegrity().setProtectTag( false );
				}
			}
			
			SwingUtilities.invokeLater(
				new DelayedRepaint( container ) );
			
		}
	}	

	class DelayedRepaint implements Runnable {
		private XMLContainer container;
		
		public DelayedRepaint( XMLContainer container ) {
			this.container = container;
		}
		
		public void run() {
			container.getEditor().requestFocus();
			container.getEditor().repaint();
			container = null;
		}
	}
	
	private Icon okIcon = null;

	private boolean refreshState( XMLContainer container ) {
		boolean state = 
			"true".equals( 
					container.getProperty( getClass().getName(), "false" ) );
		if ( !state ) {
			if ( okIcon == null ) {
				okIcon = ( Icon )getValue( Action.SMALL_ICON );
			}
			putValue( Action.SMALL_ICON, okIcon );
		} else {
			if ( okIcon == null ) {
				okIcon = ( Icon )getValue( Action.SMALL_ICON );
			}			
			putValue( Action.SMALL_ICON, getValue( Action.SMALL_ICON + "2" ) );
		}
		
		SwingUtilities.invokeLater(
				new DelayedRepaint( container ) );		
		
		return state;
	}

}
