package com.japisoft.editix.action.xml;

import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;


/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NamespaceAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {
		
		if ( Manager.isFree() ) {

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			return;
			
		}			

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;
		FPNode node = container.getCurrentNode();
		if ( node == null ) {
			EditixFactory.buildAndShowErrorDialog( "No selected node" );
			return;
		}
		
		if ( container.hasErrorMessage() ) {
			EditixFactory.buildAndShowErrorDialog( "Please fix your document before" );
			return;
		}
		
		NamespacePanel panel = new NamespacePanel( node );

		if ( DialogManager.showDialog(
				EditixFrame.THIS,
				"Namespace manager",
				"Namespace manager",
				"Add or remove a namespace definition. The current one is for the namespace of the selected node",
				null,
				panel,
				new Dimension( 400, 400 )) == 
					DialogManager.OK_ID ) {
			panel.updateNode();
			XMLPadDocument doc = container.getXMLDocument();
			String openPart = node.openDeclaration();
			String closePart = node.closeDeclaration();
			if ( node.isAutoClose() )
				closePart = null;
			doc.updateElement( openPart, closePart, node.getStartingOffset(), node.getStoppingOffset() );
		}
	}

}
