package com.japisoft.editix.action.xml;

import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import javax.swing.AbstractAction;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.framework.xml.refactor.Refactor;
import com.japisoft.framework.xml.refactor.RefactorManager;
import com.japisoft.framework.xml.refactor.elements.RefactorObj;
import com.japisoft.framework.xml.refactor.ui.RefactorUI;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.xml.FormatAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RefactorAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {

		if ( Manager.isFree() ) {		

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			
		} else {
			//
			final XMLContainer container = EditixFrame.THIS
					.getSelectedContainer();
	
			if (container == null)
				return;
	
			if (EditixFactory.mustSaveDialog(container))
				return;
	
			boolean ok = com.japisoft.xmlpad.action.ActionModel.activeActionByName(
					com.japisoft.xmlpad.action.ActionModel.SAVE_ACTION, container,
					container.getEditor());
	
			if (!ok)
				return;
	
			ArrayList al = (ArrayList) container.getDocumentInfo().getProperty(
					"refactor");
			RefactorManager.removeNonDefaultRefactors();
			if (al != null) {
				for (int i = 0; i < al.size(); i++) {
					RefactorManager.addRefactorAt((RefactorObj) al.get(i), i);
				}
			}
	
			final RefactorUI ui = new RefactorUI(container.getCurrentElementNode());
	
			if (DialogManager
					.showDialog(
							EditixFrame.THIS,
							"Refactor",
							"XML refactoring",
							"Choose a refactor element and click on the Action column\nYou can set several refactor elements\nATTR is for ATTRIBUTE",
							null, ui) == DialogManager.OK_ID) {
	
				ui.dispose();
	
				try {
					Refactor r = new Refactor();
					Boolean b = (Boolean) container
							.getProperty(FormatAction.PREF_APOSENTITY);
					if (b != null)
						r.setReplaceAPos(b.booleanValue());
	
					r.setReplaceAmp(Preferences.getPreference("xmlconfig",
							"format-replaceAmp", true));
					r.setReplaceGt(Preferences.getPreference("xmlconfig",
							"format-replaceGt", true));
					r.setReplaceLt(Preferences.getPreference("xmlconfig",
							"format-replaceLt", true));
					r.setReplaceAPos(Preferences.getPreference("xmlconfig",
							"format-replaceAPos", true));
					r.setReplaceQuote(Preferences.getPreference("xmlconfig",
							"format-replaceQuote", true));
	
					Refactor.INDENT_SIZE_PROPERTY = Preferences.getPreference(
							"xmlconfig", "format-space", 1);
	
					String xpath = null;
	
					if (ui.getModel().isRelativeRefactoring()) {
						xpath = container.getCurrentElementNode()
								.getXPathLocation();
						if (xpath != null) {
							r.setContext(xpath);
						}
					}
	
					String result = r.refactor(new File(container
							.getCurrentDocumentLocation()), ui.getModel()
							.getRefactorObjs());
	
					if (xpath == null) {
						container.setText(result);
					} else {
	
						int start = container.getCurrentElementNode()
								.getStartingOffset();
						int endOffset = container.getCurrentElementNode()
								.getStoppingOffset();
						container.getEditor().select(start, endOffset);
						container.getEditor().replaceSelection(result);
						container.getEditor().select(container.getCaretPosition(),
								container.getCaretPosition());
	
					}
	
				} catch (Exception e1) {
					EditixFactory.buildAndShowErrorDialog("Cannot refactor "
							+ e1.getMessage());
				}
			}
			//
		}
		
	}
}
