package com.japisoft.editix.action.xsl;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.xml.transform.ErrorListener;

import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.editix.action.xquery.XQueryAction;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.xslt.XSLTEditor;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSLTXQueryLastAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {
		
		if ( Manager.isFree() ) {

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			return;
			
		}		

		//
		
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		IXMLPanel panel = EditixFrame.THIS.getSelectedPanel();

		if ( container == null )
			return;

		if ( EditixFactory.mustSaveDialog( container ) ) {
			return;
		}	
		
		boolean ok = com.japisoft.xmlpad.action.ActionModel.activeActionByName(
			com.japisoft.xmlpad.action.ActionModel.SAVE_ACTION,
			container,
			container.getEditor() );
		
		if ( !ok )
			return;

		ErrorListener xsltAction = ( ErrorListener )ActionModel.restoreAction( "transformWithXSLT" );		
		ErrorListener xqueryAction = ( ErrorListener )ActionModel.restoreAction( "transformWithXQuery" );		
		
		if ( !"true".equals( container.getProperty( "xslt.ok" ) ) && 
				!"true".equals( container.getProperty( "xquery.ok" ) ) ) {
			
			XSLTDialog dialog = EditixFactory.getConfigDialog( 
					"XQR".equals( panel.getMainContainer().getDocumentInfo().getType() )		
			);

			// Show the configure dialog
			if ( "XQR".equals( panel.getMainContainer().getDocumentInfo().getType() ) ) {
				
			} else {
				
				if ( panel.getMainContainer().getDocumentInfo().getType().startsWith( "XSLT" ) ) {
					panel.setProperty( "xslt.xslt.file", container.getCurrentDocumentLocation() );
				}
								
			}
			
			dialog.init( panel );
			dialog.setVisible( true );
			dialog.dispose();
			
			if ( dialog.isOk() ) {
				dialog.store( panel );
			}
		}

		if ( "true".equals( container.getProperty( "xslt.ok" ) ) ) {
			// Repeat the last XSLT config
			XSLTAction.applyTransformation( panel, true, false, false, xsltAction );
		}

		if ( "true".equals( container.getProperty( "xquery.ok" ) ) ) {
			( (XQueryAction)xqueryAction ).transform(
					panel,
					true,
					false,
					false,
					xqueryAction );
		}

		//
	}

}
