package com.japisoft.editix.db.berkley;

import com.japisoft.editix.db.AbstractNodeDb;
import com.japisoft.editix.db.FileNodeDb;
import com.japisoft.editix.db.ui.Editable;
import com.sleepycat.dbxml.XmlDocument;
import com.sleepycat.dbxml.XmlException;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BerkleyFileNodeImpl extends AbstractNodeDb implements FileNodeDb, Editable {

	private XmlDocument doc;

	public BerkleyFileNodeImpl( XmlDocument doc ) {
		this.doc = doc;
	}

	public BerkleyFileNodeImpl( XmlDocument doc, boolean editable ) {
		this.doc = doc;
		setEdit( editable );
	}

	public String getContent() throws Exception {
		// return doc.getContentAsString();
		
		BerkleyContainerDbImpl parent = ( BerkleyContainerDbImpl )getParent();
		return parent.getContent( name );
	}

	public boolean delete() throws Exception {

		BerkleyContainerDbImpl parent = ( BerkleyContainerDbImpl )getParent();
		if ( parent.delete( doc.getName() ) ) {
			super.delete();
			return true;
		} else
			return false;
		
	}

	public void setContent(String content) throws Exception {
		
		BerkleyContainerDbImpl parent = ( BerkleyContainerDbImpl )getParent();
		doc.setContent( content );
		parent.setContent( doc );
//		parent.setContent( name, content );
		
	}

	protected void prepareName() {
		try {
			this.name = doc.getName();
		} catch (XmlException e) {
			this.name = "error";
		}
	}	
	
	private boolean editable = true;
	
	public void setEdit( boolean editable ) {
		this.editable = editable;
	}
	
	public boolean canEdit() {
		return editable;
	}		
	
	public void refresh() throws Exception {
	}

}
