package com.japisoft.editix.db.ui;

import java.awt.Component;

import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeCellRenderer;

import com.japisoft.editix.db.ContainerNodeDb;
import com.japisoft.editix.db.FileNodeDb;
import com.japisoft.editix.db.RootNodeDb;
import com.japisoft.editix.document.DocumentModel;
import com.japisoft.framework.app.toolkit.Toolkit;
import com.japisoft.framework.ui.FastLabel;
import com.japisoft.xmlpad.XMLDocumentInfo;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DbBrowserTreeRenderer implements TreeCellRenderer {

	Icon connection = null;
	Icon collection = null;

	FastLabel label = new FastLabel();
	
	public DbBrowserTreeRenderer() {
		connection = Toolkit.getImageIcon( "images/data.png" );
		collection = Toolkit.getImageIcon( "images/data_table.png" );
		label.setOpaque( true );
	}

	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean selected, boolean expanded, boolean leaf, int row,
			boolean hasFocus) {
		
		if ( value instanceof DefaultMutableTreeNode ) {
			label.setText( value.toString() );
			label.setIcon( connection );
		} else
		if ( value instanceof RootNodeDb ) {
			label.setText( value.toString() );
			label.setIcon( connection );
		} else
		if ( value instanceof ContainerNodeDb ) {
			label.setText( value.toString() );
			label.setIcon( collection );
		} else
		if ( value instanceof FileNodeDb ) {
			String file = value.toString();
			label.setText( file );
			XMLDocumentInfo xdi = 
				DocumentModel.getDocumentByFileName( file );
			if ( xdi == null )
				label.setIcon( null );
			else
				label.setIcon( xdi.getDocumentIcon() );
		}
		
		if ( selected ) {
			
			label.setForeground(UIManager
					.getColor("List.selectionForeground"));
			label.setBackground(UIManager
					.getColor("List.selectionBackground"));			
			
		} else {
			
			label.setForeground(  
				tree.getForeground()	
			);
			label.setBackground(
					tree.getBackground() );
			
		}
		
		//label.setUnderlineMode( selected );
		
		return label;
	}

}
