package com.japisoft.editix.db.xmldb;

import java.util.ArrayList;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Resource;
import org.xmldb.api.base.ResourceIterator;
import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.Service;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XPathQueryService;

import com.japisoft.editix.db.NodeDb;
import com.japisoft.editix.db.RootNodeDb;
import com.japisoft.editix.ui.EditixFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XmlDbRootDbImpl extends XmlDbContainerDbImpl implements RootNodeDb {

	public static String XMLRESOURCE_TYPE = "XMLResource";
	public static String BINARYRESOURCE_TYPE = "BinaryResource";

	public XmlDbRootDbImpl( String driverName, String url, String user, String password ) throws Exception {
		
		super( null );
		
		this.driverName = driverName;
		this.url = url;
		this.user = user;
		this.password = password; 

		int i = url.indexOf( "/" );
		String protocol = url.substring( 0, i );
		i = url.indexOf( "//" );
		int j = url.indexOf( "/", i + 2 );
		String host = url.substring( i + 2, j );
		name = protocol + "-" + host; 		

		if ( url.endsWith( "/" ) )
			url = url + "db";
/*		else
			if ( !url.endsWith( "/db" ) )
				url = url + "/db"; */
		
		col =  
			DatabaseManager.getCollection( url, user, password );
		
		//DatabaseManager.getCollection( url, null, null );
	}
	
	public String toString() {
		return name;
	}

	public boolean canConnect() throws Exception {
		return ( col != null ) && ( col.isOpen() && col.getResourceCount() >= 0 );
	}	

	public NodeDb[] request(String request) throws Exception {
		return request( col, request );
	}

	public NodeDb[] request(Collection col, String request) throws Exception {

       Service[] services = col.getServices();
       if ( services == null )
    	   throw new Exception( "Can't execute a query for this database" );
       
	   XPathQueryService service = null;
	   
	   // Get the first request service

	   for ( int i = 0; i < services.length; i++ ) {
		   if ( services[ i ] instanceof XPathQueryService ) {
			   service = ( XPathQueryService )services[ i ];
			   break;
		   }
	   }

	   if ( service == null )
		   service = ( XPathQueryService ) col.getService( "XPathQueryService", "1.0" );
	   
	   if ( service == null )
		   throw new Exception( "Can't find a service for this query" );
	   
       service.setProperty("indent", "yes");  

       ResourceSet result = service.query( request );  
       ResourceIterator i = result.getIterator();
       ArrayList al = new ArrayList();
       
       XmlDbContainerDbImpl parentNode = null;
       String previousColName = null;
       
       while ( i.hasMoreResources() ) {  
    	   Resource r = i.nextResource();  
    	     
/*    	   System.out.println( r.getClass() );
    	   System.out.println( r.getResourceType() );
    	   System.out.println( r.getId() ); */
    	   
    	   Collection pcol = r.getParentCollection();
    	   if ( pcol == null )
    		   continue;
    	   
    	   // The parent collection may not work (eXist problem)

    	   if ( previousColName == null || !( 
    			   pcol.getName().equals( previousColName ) ) ) {

    		   // Avoid duplicated container in memory
    		   pcol = getCollectionByName( pcol.getName() );

    		   if ( pcol == null ) {
    			   // May be a numeric result
    			   EditixFactory.buildAndShowInformationDialog( "" + r.getContent() );
    			   continue;
    		   }

    		   parentNode = new XmlDbContainerDbImpl( pcol );
    	   }

    	   previousColName = pcol.getName();

    	   // The id may be null : Xindice problem
    	   if ( r.getId() != null ) {    	   
	    	   XmlDbFileNodeDbImpl
	    	   	file = new XmlDbFileNodeDbImpl( 
	    	   			parentNode, 
	    	   			r.getId() 
	    	   );
	    	   parentNode.addChild( file );
	           al.add( file );
    	   }
       }  		

       NodeDb[] nodes = new NodeDb[ al.size() ];
       for ( int j = 0; j < nodes.length; j++ ) {
    	   nodes[ j ] = ( NodeDb )al.get( j );
       }

       return nodes;

	}		

	public String toXml() {
		return "<connection driver='" + driverName + "' url='" + url + "' user='" + user + "' password='" + password + "'/>";
	}

	public boolean canBeDeleted() {
		return true;
	}

}
