package com.japisoft.editix.diff;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLDiffBar extends JComponent implements MouseListener, MouseMotionListener {

	@Override
	public void addNotify() {
		super.addNotify();
		addMouseListener( this );
		addMouseMotionListener( this );
	}

	@Override
	public void removeNotify() {
		super.removeNotify();
		removeMouseListener( this );
		removeMouseMotionListener( this );
		listener = null;
	}

	public void addRange(
		boolean left,
		int startLine,
		Rectangle startView,
		Rectangle stopView,
		int maxHeight,
		Color col,
		String message ) {

		if ( items == null ) {
			items = new ArrayList<XMLDiffBar.RangeItem>();
		}

		items.add( 
			new RangeItem( 
				left, 
				startLine, 
				startView, 
				stopView, 
				maxHeight, 
				col, 
				message )
		);
		repaint();
	}

	List<RangeItem> items = null;

	public void clear() {
		items = null;
		repaint();
	}

	class RangeItem {
		boolean left;		
		int startLine; 
		Rectangle startView; 
		Rectangle stopView; 
		int maxHeight; 
		Color col; 
		String message;
		Rectangle r;

		RangeItem(
			boolean left,
			int startLine, 
			Rectangle startView, 
			Rectangle stopView, 
			int maxHeight, 
			Color col, 
			String message ) {
			this.left = left;			
			this.startLine = startLine;
			this.startView = startView;
			this.stopView = stopView;
			this.maxHeight = maxHeight;
			this.col = col;
			this.message = message;
		}
	}

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent( g );

		if ( items != null ) {
			for ( int i = 0; i < items.size(); i++ ) {
				RangeItem ri = items.get( i );
				int startRange = ( int )( ( ( double )ri.startView.y / ( double )ri.maxHeight ) * ( double )getHeight() );
				int stopRange = ( int )( ( ( double )ri.stopView.y / ( double )ri.maxHeight ) * ( double )getHeight() );

				if ( ri.left ) {				
					g.setColor( ri.col );
					g.fillRect( 0, startRange, getWidth() / 2, stopRange - startRange );
					if ( ri.r == null ) {
						ri.r = new Rectangle( 0, startRange, getWidth() / 2, stopRange - startRange );
					}
				} else {
					g.setColor( ri.col );
					g.fillRect( getWidth() / 2, startRange, getWidth() / 2, stopRange - startRange );
					if ( ri.r == null ) {
						ri.r = new Rectangle( getWidth() / 2, startRange, getWidth() / 2, stopRange - startRange );
					}
				}
			}
		}

	}

	public void mouseClicked(MouseEvent e) {
		for ( int i = 0; i < items.size(); i++ ) {
			RangeItem ri = items.get( i );
			if ( ri.r != null && ri.r.contains( e.getPoint() ) ) {
				if ( listener != null ) {
					listener.notifyLocation( ri.left, ri.startLine );
				}
			}
		}
	}
	
	XMLDiffBarListener listener = null;
	
	void setXMLDiffBarListener( XMLDiffBarListener listener ) {
		this.listener = listener;
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}

	public void mouseDragged(MouseEvent e) {
	}

	public void mouseMoved(MouseEvent e) {
		setToolTipText( null );
		
		if ( items != null )
		for ( int i = 0; i < items.size(); i++ ) {
			RangeItem ri = items.get( i );
			int startRange = ( int )( ( ( double )ri.startView.y / ( double )ri.maxHeight ) * ( double )getHeight() );
			int stopRange = ( int )( ( ( double )ri.stopView.y / ( double )ri.maxHeight ) * ( double )getHeight() );

			if ( ri.r != null && ri.r.contains( e.getPoint() ) ) {
				setToolTipText( ri.message );
			}
		}
		
	}

}
