package com.japisoft.editix.document.json;

import java.util.List;

import org.json.JSONKey;
import org.json.JSONObject;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SchemaHandler extends AbstractHelperHandler {
	
	private String getPointer( FPNode currentNode ) {
		StringBuffer sb = new StringBuffer();
		Object jsonObj = currentNode.getApplicationObject();
		
		if ( currentNode.getApplicationObject() instanceof JSONKey ) {
			JSONKey key = ( JSONKey )currentNode.getApplicationObject();
			sb.insert( 0, key.getKey() );
			jsonObj = key.getParent();
		}
		
		if ( jsonObj instanceof JSONObject ) {
			JSONObject subject = ( JSONObject )jsonObj;
			// Build the JSON pointer
			
			while ( currentNode.getParent() != null ) {
				currentNode = ( FPNode )currentNode.getParent();
				if ( currentNode.getApplicationObject() != null ) {
					if ( currentNode.getApplicationObject() instanceof JSONKey ) {
						JSONKey key = ( JSONKey )currentNode.getApplicationObject();
						sb.insert(0, key + "/" );
					}
				}
			}
			
		}

		sb.insert( 0, "#/" );
		return sb.toString();
	}
	
	protected void installDescriptors( 
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String addedString ) {	
		
		XMLContainer container = document.getContainer();
		
		if ( currentNode == null ) {
			currentNode = container.getRootNode(); 
		}
		
		String schemaPath = null;

		if ( ( schemaPath = ( String )container.getProperty( "json-schema" ) ) != null ) {
			
			SchemaResolver jsonSchema = ( SchemaResolver )container.getProperty( "json-resolver" );
			if ( jsonSchema == null ) {
				try {
					container.setProperty( "json-resolver", jsonSchema = new SchemaResolver( schemaPath ) );
				} catch( Exception exc ) {
				}
			}

			if ( jsonSchema != null && currentNode != null ) {

				
				String pointer = getPointer( currentNode ); 
									
				List<SchemaProperty> l = jsonSchema.resolve( currentNode.getApplicationObject(), pointer );
				for ( SchemaProperty sp : l ) {
					BasicDescriptor bd = new BasicDescriptor( sp.name );
					bd.setRawContent( sp.toString() );
					bd.setComment( sp.description );
					addDescriptor( bd );
				}

			}
		}
		
	}
	
	public String getTitle() {
		return "JSON content";
	}	

	protected String getActivatorSequence() {
		return null;
	}	
	
	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset, 
			String activatorString ) {

		return activatorString == null;
	}

	
	
}
