package com.japisoft.editix.document.schema;

import java.util.Enumeration;
import java.util.Iterator;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.walker.NodeNameCriteria;
import com.japisoft.framework.xml.parser.walker.OrCriteria;
import com.japisoft.framework.xml.parser.walker.TreeWalker;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SchemaTypeHandler extends AbstractHelperHandler {

	protected void installDescriptors( 
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String addedString ) {	

		XMLContainer container = document.getContainer();
		if ( container == null )
			return;
		FPNode root = container.getRootNode();
		if ( root == null )
			return;

		String prefix = root.getNameSpacePrefix();
		if ( prefix == null )
			prefix = "";
		else
			prefix = prefix + ":";
		
		if ( addedString == null )
			addedString = "";
	
		// Add complex type
		TreeWalker tw = new TreeWalker( root );
		Enumeration enume = tw.getNodeByCriteria(
				new OrCriteria(
						new NodeNameCriteria( "complexType" ),
						new NodeNameCriteria( "simpleType" ) )
						, false );
		String prefixComplexType = "";
		if ( enume != null ) {
			// Check for targetNameSpace
			if ( root.hasAttribute( "targetNamespace" ) ) {
				String namespace = root.getAttribute( "targetNamespace" );
				Iterator<String> enum2 = root.getNameSpaceDeclaration();
				// Search a prefix
				if ( enum2 != null ) {
					while ( enum2.hasNext() ) {
						String p = ( String )enum2.next();
						if ( namespace.equals( root.getNameSpaceDeclarationURI( p ) ) ) {
							prefixComplexType = p + ":";
							break;
						}
					}
				}
			}
			while ( enume.hasMoreElements() ) {
				FPNode n = ( FPNode )enume.nextElement();
				String name = n.getAttribute( "name" );
				if ( name != null )
					addDescriptor( new BasicDescriptor( prefixComplexType + name ) );
			}
		}

		for ( int i = 0; i < PRIM_TYPES.length; i++ ) {
			addDescriptor(
					new BasicDescriptor( 
							addedString + 
							prefix + 
							PRIM_TYPES[ i ] + 
							addedString ) );					
		}
	}

	protected String getActivatorSequence() {
		return null;
	}

	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset,
			String activatorString ) {
		if ( activatorString == null ) {
			return match(
				document, 
				offset, 
				"", 
				"type=\"" );
		} else {
			if  ( "\"".equals( activatorString ) ) {
				return match(
						document, 
						offset, 
						"", 
						"type=" );
			} else
				return false;
		}
	}

	public String getTitle() {
		return "W3C Schema Types";
	}

	public int getPriority() {
		return 1;
	}

	private static String[] PRIM_TYPES = {
		"string",
		"boolean",
		"float",
		"double",
		"decimal",
		"duration",
		"dateTime",
		"time",
		"date",
		"gYearMonth",
		"gYear",
		"gMonthDay",
		"gDay",
		"gMonth",
		"hexBinary",
		"base64Binary",
		"anyURI",
		"QName",
		"NOTATION",
		"normalizedString",
		"token",
		"language",
		"IDREFS",
		"ENTITIES",
		"NMTOKEN",
		"NMTOKENS",
		"Name",
		"NCName",
		"ID",
		"IDREF",
		"ENTITY",
		"integer",
		"nonPositiveInteger",
		"negativeInteger",
		"long",
		"int",
		"short",
		"byte",
		"nonNegativeInteger",
		"unsignedLong",
		"unsignedInt",
		"unsignedShort",
		"unsignedByte",
		"positiveInteger",
	};
	
}
