package com.japisoft.editix.document.xslt;

import java.util.List;

import javax.swing.text.BadLocationException;

import com.japisoft.framework.xml.grammar.GrammarAttribute;
import com.japisoft.framework.xml.grammar.GrammarElement;
import com.japisoft.framework.xml.grammar.xsd.XSDGrammar;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.AttDescriptor;
import com.japisoft.xmlpad.helper.model.TagDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SchemaHandler extends AbstractHelperHandler {

	public SchemaHandler() {
		super();
		// System.out.println( "aa" );
	}

	@Override
	protected String getActivatorSequence() {
		return null;
	}

	@Override
	public String getTitle() {
		return "DTD/Schema helper";
	}

	private XSDGrammar g = null;
	private String gHref = null;
	
	private void loadGrammar( String href ) {
		if ( g != null ) {
			// The same grammar, don't reload
			if ( href.equalsIgnoreCase( gHref ) )
				return;
		}

		try {
			
			g = new XSDGrammar( href );

		} catch( Exception exc ) {
		}

		gHref = href;
	}

	@Override
	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset,
			String activatorString ) {
		
		if ( !( "<".equals( activatorString ) || activatorString == null ) ) {
			return false;
		}

		String mark = "<?schema ";
		try {
			String header = ( document.getText( 0, Math.max( 1, document.nextTag( 0 ) ) ) );
			int i = header.indexOf( mark );
			
			boolean found = false;
			
			if ( i > -1 ) {
				int j = header.indexOf( "?>", i + 1 );
				if ( j > -1 ) {
					String piParameter = header.substring( i + mark.length(), j );
					piParameter = piParameter.trim();
					i = piParameter.indexOf( "'" );
					if ( i == -1 ) {
						i = piParameter.indexOf( "\"" );
					}
					if ( i > -1 ) {
						j = piParameter.indexOf( "'", i + 1 );
						if ( j == -1 ) {
							j = piParameter.indexOf( "\"", i + 1 );
						}
						if ( j > -1 ) {
							String schemaLocation = piParameter.substring( i + 1, j );
							loadGrammar( schemaLocation );
							found = true;
						}
					}
				}
			}
			
			if ( !found ) {
				// remove grammar reference
				g = null;
				gHref = null;
			}

		} catch( BadLocationException exc ) {
			
		}

		return ( g != null );
	}

	@Override
	protected void installDescriptors(FPNode currentNode,
			XMLPadDocument document, int offset, String activatorString) {	
		if ( g != null ) {
			
			if ( document.isInsideTag( offset ) ) {
			
				// Only attributes
				String name = currentNode.getContent();
				if ( name != null ) {
					
					GrammarElement e = g.getGlobalElement( name );
					if ( e != null ) {
						List<GrammarAttribute> l = e.getAttributes( false );
						if ( l != null ) {
							for ( GrammarAttribute ga : l ) {
								String dv = ga.getDefaultValue() == null ? "" : ga.getDefaultValue().getValue();
								AttDescriptor ad = new AttDescriptor( ga.getName(), dv );
								addDescriptor( ad );
							}
						}
					}

				}
				
			} else {
			
				List<GrammarElement> l = g.getGlobalElements();
				if ( l != null ) {
					for ( GrammarElement ge : l ) {					
						TagDescriptor td = new TagDescriptor( ge.getName(), false );
						
						String inferRes = ge.infer();
						td.setRawContent( inferRes );
	
						addDescriptor( td );
					}
				}
				
			}
		}
	}

	@Override
	public void dispose() {
		g = null;
		gHref = null;
		super.dispose();
	}

}
