package com.japisoft.editix.editor.css;

import java.util.ArrayList;

import javax.swing.Action;
import javax.swing.JFrame;
import javax.swing.JToolBar;
import javax.swing.text.EditorKit;
import javax.swing.text.PlainDocument;

import com.japisoft.editix.editor.css.helper.CommentHandler;
import com.japisoft.editix.editor.css.helper.PropertiesHandler;
import com.japisoft.editix.editor.css.helper.SelectorHandler;
import com.japisoft.editix.editor.css.helper.ValuesHandler;
import com.japisoft.editix.editor.css.kit.CssEditorKit;

import com.japisoft.editix.ui.EditixErrorPanel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.ComponentFactory;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.EditorContext;
import com.japisoft.xmlpad.editor.XMLEditor;
import com.japisoft.xmlpad.elementview.ElementView;
import com.japisoft.xmlpad.tree.parser.Parser;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CSSEditor extends XMLContainer {

	static boolean STANDALONE = false;
	
	private Action parseAction = null;
	
	public CSSEditor( Action parseAction ) {
		super( new ComponentFactoryForCSSEditor() );
		this.parseAction = parseAction;
		getUIAccessibility().setToolBarAvailable( false );
		setStatusBarAvailable( false );
		getUIAccessibility().setPopupAvailable( false );
		getUIAccessibility().setTreePopupAvailable( false );
		setAutoNewDocument( false );
		setErrorPanelAvailable( true );
		getDocumentInfo().setRealTimeTree( true );		
		setEnabledTreeLocationForCaret( true );
		setTreeDragDrop( false );
		
		setDisposeAction(false);
		setTreeAvailable( true );
		setTreePopupAvailable( false );
		setTreeElementViewAvailable( false );

		getDocument().putProperty(PlainDocument.tabSizeAttribute,
				new Integer(Preferences.getPreference("file", "tab-size", 2)));

		ArrayList assistant = new ArrayList();
		assistant.add( new CommentHandler() );
		assistant.add( new PropertiesHandler() );
		assistant.add( new ValuesHandler() );
		assistant.add( new SelectorHandler() );
		
		getHelperManager().resetHandlers( assistant, false );
		
		getUIAccessibility().setErrorView(new EditixErrorPanel());
	}

	public Action getAction(String actionId) {
		if ( "parse".equals( actionId ) ) {
			return parseAction;
		} else
			return super.getAction( actionId );
	}
		
	@Override
	public Parser createNewParser( boolean lightweightMode ) {
		return new CSSParser();
	}	
	
	static class CustomXMLEditorForCSS extends XMLEditor {
		CustomXMLEditorForCSS( EditorContext context ) {
			super( context );
		}
		public EditorKit getEditorKit() {
			return new CssEditorKit( "CSSEditor" );
		}
	}

	static class ComponentFactoryForCSSEditor extends ComponentFactory {
		public XMLEditor getNewXMLEditor( EditorContext context ) {
			return new CustomXMLEditorForCSS( context );
		}
		@Override
		public ElementView getNewElementView(XMLContainer container) {
			return null;
		}
		@Override
		public JToolBar getNewTreeToolBar() {
			return null;
		}
	}
	
	public static void main( String[] args ) {
		ApplicationModel.SHORT_APPNAME = "test";
		JFrame f = new JFrame();
		f.add( new CSSEditor( null ).getView() );
		f.setSize( 300, 300 );
		f.setVisible( true );
	}

}
