package com.japisoft.editix.editor.html.helper;

import java.awt.Color;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.List;

import javax.swing.text.BadLocationException;

import com.japisoft.dtdparser.DTDParser;
import com.japisoft.dtdparser.node.AttributeDTDNode;
import com.japisoft.dtdparser.node.ElementDTDNode;
import com.japisoft.dtdparser.node.RootDTDNode;
import com.japisoft.editix.document.DocumentModel;
import com.japisoft.framework.collection.FastVector;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLDocumentInfo;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.handler.schema.dtd.DTDAttDescriptor;
import com.japisoft.xmlpad.helper.handler.schema.dtd.DTDTagDescriptor;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;
import com.japisoft.xmlpad.helper.model.TagDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class HTMLHandler extends AbstractHelperHandler {

	private RootDTDNode node = null;

	String[] html5 = null;
	
	public HTMLHandler() {
		html5 = new String[] {
				"canvas", "Used to draw graphics, on the fly, via scripting (usually JavaScript)",
				"audio", "Defines sound content",
				"video", "Defines a video or movie",
				"source", "Defines multiple media resources for <video> and <audio>",
				"embed", "Defines a container for an external application or interactive content (a plug-in)",
				"track", "Defines text tracks for <video> and <audio>",
				"datalist", "Specifies a list of pre-defined options for input controls",
				"keygen", "Defines a key-pair generator field (for forms)",
				"output", "Defines the result of a calculation",
				"article", "Defines an article",
				"aside", "Defines content aside from the page content",
				"bdi", "Isolates a part of text that might be formatted in a different direction from other text outside it",
				"command", "Defines a command button that a user can invoke",
				"details", "Defines additional details that the user can view or hide",
				"dialog", "Defines a dialog box or window",
				"summary", "Defines a visible heading for a <details> element",
				"figure", "Specifies self-contained content, like illustrations, diagrams, photos, code listings, etc.",
				"figcaption", "Defines a caption for a <figure> element",
				"footer", "Defines a footer for a document or section",
				"header", "Defines a header for a document or section",
				"mark", "Defines marked/highlighted text",
				"meter", "Defines a scalar measurement within a known range (a gauge)",
				"nav", "Defines navigation links",
				"progress", "Represents the progress of a task",
				"ruby", "Defines a ruby annotation (for East Asian typography)",
				"rt", "Defines an explanation/pronunciation of characters (for East Asian typography)",
				"rp", "Defines what to show in browsers that do not support ruby annotations",
				"section", "Defines a section in a document",
				"time", "Defines a date/time",
				"wbr", "Defines a possible line-break"
		};
	}
	
	private boolean isHTML5( XMLPadDocument document ) {
		try {
			String tmp = document.getText( 0, document.getLength() ).toLowerCase();
			return tmp.contains( "<!doctype html>" ) || tmp.contains( "<!doctype  html>" );
		} catch( BadLocationException ble ) {
			return false;
		}
	}

	protected void installDescriptors( 
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String addedString ) {
				
		if ( "#".equals( addedString ) ) {
			
			List<FPNode> fv = currentNode.getDocument().getFlatNodes();
			if ( fv != null ) {
				for ( int i = 0; i < fv.size(); i++ ) {
					FPNode node = ( FPNode )fv.get( i );
					if ( node.hasAttribute( "id" ) ) {
						String idValue = node.getAttribute( "id" );
						
						BasicDescriptor bd = new BasicDescriptor( "#" + idValue );
						addOrderedDescriptor( bd );
					}
				}
			}
			
			return;
		}
		
		
		// Parse the HTML DTD
		XMLDocumentInfo info = 
			DocumentModel.getDocumentForType( "XHTML" );
		String HTMLDTD = info.getDefaultDTDLocation();
		if ( HTMLDTD.startsWith( "file:" ) )
			HTMLDTD = HTMLDTD.substring( 5 );
		if ( HTMLDTD.indexOf( ":" ) > -1 ) {
			if ( HTMLDTD.startsWith( "/" ) )
				HTMLDTD = HTMLDTD.substring( 1 );
		}

		DTDParser parser = new DTDParser();
		try {
			parser.parse( new FileInputStream( HTMLDTD ) );
			node = parser.getDTDElement();
		} catch (IOException e) {
			return;
		}

		if ( node != null ) {
			
			if ( !attributesAssistant ) {
			
				for ( int i = 0; i < node.getDTDNodeCount(); i++ ) {
					Color pink2 = Color.PINK.darker();
					if ( node.getDTDNodeAt( i ).isElement() ) {
						DTDTagDescriptor dtd = ( DTDTagDescriptor )addOrderedDescriptor( 
								new DTDTagDescriptor( 
									( ElementDTDNode )node.getDTDNodeAt( i ) ) 
						);
						dtd.setColor( pink2 );
						dtd.setAddedPart( "<" );
					}
				}

				if ( isHTML5( document ) ) {
				
					Color tmp = Color.ORANGE.darker();
					
					for ( int i = 0; i < html5.length; i += 2 ) {
						String element = html5[ i ];
						String comment = html5[ i + 1 ];
	
						TagDescriptor td = new TagDescriptor( element, false );
						td.setColor( tmp );
						td.setComment( "HTML 5 Element : " + comment );
						addOrderedDescriptor( td );
						td.setAddedPart( "<" );

					}

				}
				
			} else {
				
				// Search for the matchingNode

				try {
				
					String matchingNode = document.getPreviousOpeningTag(offset);
					boolean found = false;
					
					for ( int i = 0; i < node.getDTDNodeCount(); i++ ) {
						if ( node.getDTDNodeAt( i ).isElement() ) {
							ElementDTDNode e = ( ElementDTDNode )node.getDTDNodeAt( i );
							if ( e.getName().equalsIgnoreCase( matchingNode ) ) {
								found = true;
								break;
							}
						}
					}
					
					if ( !found )	// For HTML 5 element
						matchingNode = "div";

					for ( int i = 0; i < node.getDTDNodeCount(); i++ ) {
						if ( node.getDTDNodeAt( i ).isElement() ) {
	
							ElementDTDNode e = ( ElementDTDNode )node.getDTDNodeAt( i );
							if ( e.getName().equalsIgnoreCase( matchingNode ) ) {
								
								// We got it
								for ( int j = 0; j < e.getChildCount(); j++ ) {
									
									if ( e.getDTDNodeAt( j ).isAttribute() ) {
										
										DTDAttDescriptor dad = ( DTDAttDescriptor )addOrderedDescriptor( new DTDAttDescriptor(
												( AttributeDTDNode )e.getDTDNodeAt( j ) ) );
										dad.setAddedPart( " " );
	
									}
	
								}
								
								break;
								
							}
							
						}
	
					}
					
				} catch( BadLocationException ble ) {

				}
			}
		}
	}

	protected String getActivatorSequence() {
		return null;
	}

	private boolean attributesAssistant;
	
	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset,
			String activatorString ) {
				
		attributesAssistant = false;

		if ( "#".equals( activatorString ) )
			return true;
		
		// Check for attributes
		if ( null == activatorString || 
						"<".equals( activatorString )
				) {

			// Ignore attribute value
			if ( document.isInsideAttributeValue( offset ) )

				return false;
					
			
			return true;
		}
		
		if ( " ".equals( activatorString ) ) {

			if  ( document.isInsideTag(
					offset,
					true,
					true ) ) {

				attributesAssistant = true;
				return true; 
				
			}

		}

		return false;
	}

	public String getTitle() {
		return "HTML";
	}

	public int getPriority() {
		return -1;
	}

}
