package com.japisoft.editix.editor.jsx;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.japisoft.editix.editor.jsx.domapi.Document;
import com.japisoft.editix.editor.jsx.domapi.Element;
import com.japisoft.editix.editor.jsx.domapi.Node;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DOMHandler extends AbstractHelperHandler {

	private List<String> fields;
	private List<String> methods;
	
	public DOMHandler() {
		addClass( Node.class );
		addClass( Element.class );
		addClass( Document.class );
		
		if ( fields != null )
			Collections.sort( fields );
		
		if ( methods != null )
			Collections.sort( methods );
	}
	
	@Override
	public boolean haveDescriptors(FPNode currentNode, XMLPadDocument document, boolean insertBefore, int offset,
			String activatorString) {
		return activatorString == null;
	}

	@Override
	protected void installDescriptors(FPNode currentNode, XMLPadDocument document, int offset, String activatorString) {
		if ( fields != null ) {
			for ( String f : fields ) {				
				( ( BasicDescriptor )addDescriptor( new BasicDescriptor( f, getRaw( activatorString, f ) ) ) ).setComment( "DOM Property" );
			}
		}
		if ( methods != null ) {
			for ( String f : methods ) {
				( ( BasicDescriptor )addDescriptor( new BasicDescriptor( f, getRaw( activatorString, f ) ) ) ).setComment( "DOM Method" );
			}			
		}
	}
	
	private String getRaw( String activatorString, String label ) {
		String prefix = "";
		if ( activatorString != null )
			prefix = activatorString;
		
		int i = label.lastIndexOf( ":" );
		if ( i > -1 )
			return prefix + label.substring( 0, i );
		return prefix + label;
	}
	
	private void addClass( Class cl ) {
		Field[] ff = cl.getDeclaredFields();
		for ( Field f : ff ) {
			if ( Modifier.isPublic( f.getModifiers() ) ) {
				 if ( fields == null )
					 fields = new ArrayList<String>();
				 fields.add( f.getName() + " : " + cl.getSimpleName() );
			}
		}
		Method[] mm = cl.getDeclaredMethods();
		for ( Method m : mm ) {
			if ( Modifier.isPublic( m.getModifiers() ) ) {
				if ( methods == null )
					methods = new ArrayList<String>();
				methods.add( m.getName() + " : " + cl.getSimpleName() );
			}
		}
	}
	
}
