package com.japisoft.editix.editor.jsx;

import java.awt.*;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import com.japisoft.editix.ui.xslt.Factory;
import com.japisoft.framework.ui.text.FileTextField;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.LocationEvent;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.look.LookManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLDataSourcePanel extends JPanel 
		implements TreeSelectionListener {

	JLabel lblPath = new JLabel();
	FileTextField file = null;
	XMLContainer xmlContainer = null;
	GridBagLayout gridBagLayout1 = new GridBagLayout();
	Factory factory = null;

	public XMLDataSourcePanel( Factory factory ) {
		this.factory = factory;
		initUI();
	}

	JTree tree = new JTree();
	JTabbedPane tp = new JTabbedPane( JTabbedPane.BOTTOM );
	
	private void initUI() {
		
		file = new FileTextField( null, null, new String[] { "xml" }, factory.getPathBuilder() );
		
		lblPath.setText( "Path :" );
		this.setLayout( gridBagLayout1 );
		file.setText( "" );		
		xmlContainer = factory.buildNewContainer( null ).getMainContainer();
		xmlContainer.setAutoResetAction( false );
		xmlContainer.setPopupAvailable( false );
		xmlContainer.setTreeAvailable( false );
		xmlContainer.setEditableDocumentMode( false );
		xmlContainer.getUIAccessibility().setEnableDragNDropForRoot( true );
		xmlContainer.getDocumentInfo().setTreeAvailable( false );
		xmlContainer.setAutoNewDocument( false );
		
		LookManager.getCurrentLook().install( xmlContainer, tree );
		
		xmlContainer.setTreeDelegate( tree );
		
		tp.addTab( "Tree", new JScrollPane( tree ) );
		tp.addTab( "Text", xmlContainer.getView() );
		tp.setSelectedIndex( 0 );

		this.add(lblPath, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4,
						6, 0, 9), 0, 0));
		this.add(file, new GridBagConstraints(1, 0, 5, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(4, 0, 0, 5), 0, 0));
		this.add(tp, new GridBagConstraints(0, 2, 6, 1, 1.0, 1.0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						3, 3, 3, 3), 0, 0));		
	}

	public void valueChanged( TreeSelectionEvent e ) {
		FPNode n = ( FPNode )e.getPath().getLastPathComponent();
		xmlContainer.notifyLocationListener( new LocationEvent( xmlContainer, n ) );
	}

	public void dispose() {
		xmlContainer.dispose();
	}

	public void setAutoDisposeMode( boolean disposeMode ) {
		xmlContainer.setAutoDisposeMode( disposeMode );
	}

	public void addNotify() {
		super.addNotify();
		tree.addTreeSelectionListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		tree.removeTreeSelectionListener( this );
	}

	// From the debugger part
	public void showSourceLine(int line) {
		tp.setSelectedIndex( 1 );
		xmlContainer.getEditor().highlightLine( line );
	}
}
