package com.japisoft.editix.editor.jsx.action;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;

import javax.script.Bindings;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptException;

import com.japisoft.editix.editor.jsx.domapi.Document;
import com.japisoft.editix.javascript.JavaScriptFactory;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;

import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.error.ErrorManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RunAction extends AbstractAction {

	@Override
	public void actionPerformed(ActionEvent e) {
		IXMLPanel panel = EditixFrame.THIS.getSelectedPanel();
		
		ErrorManager rm = panel.getMainContainer().getErrorManager();
		rm.notifyNoError(false);
		rm.initErrorProcessing();
		
		String xmlDataSource = (String)panel.getProperty( "jsx.data.file" );

		if ( xmlDataSource == null ) {
			EditixFactory.buildAndShowErrorDialog( "No data file ?" );
			return;
		}
		
		File f = new File( xmlDataSource );
		if ( !f.exists() ) {
			EditixFactory.buildAndShowErrorDialog( "Can't find " + xmlDataSource + " ?" );
			return;
		}
		
		ScriptEngine engine = JavaScriptFactory.newFactory().engine();
		
		Document doc = null;
		
		try {
			Bindings b = engine.getBindings(ScriptContext.GLOBAL_SCOPE);
			
			try {

				doc = NodeFactory.newInstance().getDocument( xmlDataSource );					
				b.put( "document", doc );
			} catch( Exception exc ) {
				EditixFactory.buildAndShowErrorDialog( "Can't parse this data file ?" );
				return;
			}
			
        	engine.eval( panel.getMainContainer().getText() );
        	
        	doc.save();
        	
        	// Reload it
			panel.setProperty( "jsx.data.file", xmlDataSource );
        	
		} catch( ScriptException se ) {
			int line = se.getLineNumber();
			String message = se.getMessage();
			int i = message.lastIndexOf( ":" );
			if ( i > -1 )
				message = message.substring( i + 1 );
			rm.notifyError( this, true, null, line, se.getColumnNumber(), -1, message, false );
			EditixFactory.buildAndShowErrorDialog( "Error(s) found" );
		}
		
		rm.stopErrorProcessing();

	}
	
}
