package com.japisoft.editix.editor.jsx.domapi;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NamedNodeMap {

	private org.w3c.dom.NamedNodeMap dom;
	private Element owner;

	public int length;
	
	public NamedNodeMap( Element owner, org.w3c.dom.NamedNodeMap nnm ) {
		this.owner = owner;
		this.dom = nnm;
		updateLength();
	}
	
	public Attr getNamedItem( String name ) {
		org.w3c.dom.Node node = dom.getNamedItem( name );
		if ( node == null )
			return null;
		return new Attr( owner, node );
	}
	
	public void setNamedItem( Attr att ) {
		dom.setNamedItem( att.getDOMNode() );
		updateLength();
	}
	
	public void removeNamedItem( String name ) {
		dom.removeNamedItem( name );
		updateLength();
	}
	
	public Attr item( int index ) {
		org.w3c.dom.Node node = dom.item( index );
		if ( node == null )
			return null;
		
		Object tmp = node.getUserData( "node" );
		if ( tmp != null ) {
			if ( tmp instanceof Attr )
				return (Attr)tmp;
		}
		
		return new Attr( owner, node );
	}
	
	public Attr getNamedItemNS( String namespace, String localName ) {
		org.w3c.dom.Node node = this.dom.getNamedItemNS( namespace, localName );
		if ( node == null )
			return null;
		
		Object tmp = node.getUserData( "node" );
		if ( tmp != null ) {
			if ( tmp instanceof Attr )
				return (Attr)tmp;
		}
		
		return new Attr( owner, node );
	}
	
	public void setNamedItemNS( Attr att ) {
		dom.setNamedItemNS( att.getDOMNode() );
		updateLength();
	}
		
	public void removeNamedItemNS( String namespace, String localName ) {
		dom.removeNamedItemNS(namespace, localName);
		updateLength();
	}
	
	private void updateLength() {
		length = dom.getLength();
	}
	
}
