package com.japisoft.editix.editor.jsx.domapi;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class Node {

	protected org.w3c.dom.Node node;

	public String baseURI;
	public NodeList childNodes;
	
	public String localName;
	public String nodeName;
	public String tagName;
	public String namespaceURI;
	public short nodeType;
	public String nodeValue;
	public Node firstChild = null;
	public Node lastChild = null;
	public Node parentNode = null;
	public String textContent = null;
	
	protected Node( Document owner, org.w3c.dom.Node node ) {
		node.setUserData( "node", this, null );
		node.setUserData( "owner", owner, null );
		
		this.node = node;
		this.baseURI = node.getBaseURI();
		this.nodeValue = node.getNodeValue();
		this.nodeType = node.getNodeType();
		this.tagName = node.getNodeName();
		this.localName = node.getLocalName();
		this.nodeName = node.getNodeName();
		this.namespaceURI = node.getNamespaceURI();
		this.textContent = node.getTextContent();
		
		if ( this.node.getParentNode() != null )
			this.parentNode = (Node)this.node.getParentNode().getUserData( "node" );
		
		updateFromDOM();		
	}
	
	private void updateFromDOM() {
		org.w3c.dom.NodeList nl = node.getChildNodes(); 
		NodeList bnl = new NodeList();
		if ( nl != null )
			for ( int i = 0; i < nl.getLength(); i++ ) {				
				Node newNode = bnl.addNode( getOwner().createNode( nl.item( i ) ) );
				newNode.parentNode = this;
				nl.item( i ).setUserData( "node", newNode, null );
			}
		updateFirstLast();
		this.childNodes = bnl;
	}
	
	private void updateFirstLast() {
		if ( this.childNodes != null ) {
			for ( int i = 0; i < this.childNodes.length; i++ ) {
				if ( i == 0 )
					firstChild = this.childNodes.item( i );
				if ( i == this.childNodes.length - 1 )
					lastChild = this.childNodes.item( i );
			}
		}
	}		
	
	public Node appendChild( Node newChild ) {
		this.childNodes.addNode( newChild );
		this.node.appendChild( newChild.getDOMNode() );
		updateFirstLast();
		return newChild;
	}
	
	public Node cloneNode() {
		return getOwner().createNode( this.node.cloneNode( true ) );
	}
	
	public boolean hasChildNodes() {
		return this.childNodes != null && this.childNodes.length > 0;
	}
	
	public Node insertBefore( Node newNode, Node refNode ) {
		this.node.insertBefore( newNode.getDOMNode(), refNode.getDOMNode() );
		updateFromDOM();
		return newNode;
	}
	
	public boolean isDefaultNamespace( String namespaceURI ) {
		return this.node.isDefaultNamespace( namespaceURI );
	}
	
	public boolean isEqualNode( Node otherNode ) {
		return this.node.isEqualNode( otherNode.getDOMNode() );
	}

	public boolean isSameNode( Node otherNode ) {
		return this.node.isSameNode( otherNode.getDOMNode() );
	}
	
	public String lookupPrefix( String namespaceURI  ) {
		return this.node.lookupPrefix( namespaceURI );
	}
	
	public String lookupNamespaceURI( String prefix ) {
		return this.node.lookupNamespaceURI( prefix );
	}
	
	public void normalize() {
		this.node.normalize();
		updateFromDOM();
	}
	
	public void removeChild( Node child ) {
		this.node.removeChild( child.getDOMNode() );
		updateFromDOM();
	}
	
	public void replaceChild( Node newChild, Node oldChild ) {
		this.node.replaceChild( newChild.getDOMNode(), oldChild.getDOMNode() );
		updateFromDOM();
	}
	
	protected void afterTree() { }
	
	org.w3c.dom.Node getDOMNode() { return this.node; }
	
	Document getOwner() { return (Document)node.getUserData( "owner" ); }
	
}
