package com.japisoft.editix.editor.svg;

import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.StringReader;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import javax.swing.JPanel;
import javax.swing.JToolBar;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.batik.swing.JSVGCanvas;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import com.japisoft.xmlpad.IXMLPanel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SVGPreview extends JPanel {

	private boolean autoRefresh = false;
	private IXMLPanel editor = null;
	private JSVGCanvas svgView = null;
	
	SVGPreview( IXMLPanel editor ) {
		this.editor = editor;
		setLayout( new BorderLayout() );

		editor.getMainContainer().getEditor().getActionMap().put( "refresh", new RefreshAction() );
		editor.getMainContainer().getEditor().getInputMap().put( KeyStroke.getKeyStroke( KeyEvent.VK_F5, 0), "refresh" );

		JToolBar tb = new JToolBar();
		tb.add( editor.getMainContainer().getEditor().getActionMap().get( "refresh" ) );

		add( tb, BorderLayout.SOUTH );
	}

	public void dispose() {
		editor = null;
	}

	void refresh() {
		if ( svgView == null ) {
			svgView = new JSVGCanvas();
			add( svgView, BorderLayout.CENTER );
			invalidate();
			validate();
			repaint();
		}

		try {
			String xml = editor.getMainContainer().getText();
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware( true );
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document doc = db.parse( new InputSource( new StringReader( xml ) ) );
			svgView.setDocument( doc );
		} catch( Exception exc ) {
			exc.printStackTrace();
		}
	}
	
	/////////////////////////////////////////////////////////////////////
	
	class RefreshAction extends AbstractAction {
	
		public RefreshAction() {
			putValue( 
				Action.SHORT_DESCRIPTION, 
				"Refresh the CSS Preview" 
			);
			putValue( 
				Action.SMALL_ICON, 
				new ImageIcon( 
					getClass().getResource( "refresh.png" )
				) 
			);
		}

		public void actionPerformed(ActionEvent e) {
			refresh();
		}
		
	}
	
}
