package com.japisoft.editix.editor.xquery.kit;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.JTextComponent;
import javax.swing.text.PlainView;
import javax.swing.text.Segment;
import javax.swing.text.Utilities;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XQueryView extends PlainView {

	private SyntaxLexer sl;
	private Segment line;	
	
	public XQueryView(Element e) {
		super(e);
		line = new Segment();
		this.sl = new SyntaxLexer();
	}
	
	public void drawLine(int lineIndex, Graphics g, int x, int y) {
		( ( Graphics2D )g ).setRenderingHint( 
			RenderingHints.KEY_TEXT_ANTIALIASING, 
			RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB 
		);
		
		Document d = getDocument();
		Color defColor = getContainer().getForeground();
		JTextComponent cc = (JTextComponent) getContainer();

		Element lineElement = getElement().getElement(lineIndex);
		int start = lineElement.getStartOffset();
		int end = lineElement.getEndOffset();
		try {
			d.getText(start, end - (start + 1), line);
			int offset = 0;
			Token[] tks =
				sl.getTokenForLine(
					d.getText(start, end - (start + 1)),
					lineIndex);

			if (tks == null) {
				return; // ??
			}
			FontMetrics fm = g.getFontMetrics();
			int h = fm.getHeight();
			boolean mustRepaint = false;
			for (int i = 0; i < sl.getTokenCount(); i++) {
				Token t = tks[i];
				mustRepaint = mustRepaint || t.mustRepaint;
				line.count = t.text.length();
				Color c = t.color;
				
				g.setColor(c);
				x = Utilities.drawTabbedText(line, x, y, g, this, offset);

				offset += line.count;
				line.offset += line.count;

			}
			if ( mustRepaint )
				getContainer().repaint();
		} catch (BadLocationException e) {
		}
	}

}
