package com.japisoft.editix.editor.xsd.view.designer;

import org.w3c.dom.Element;

import com.japisoft.editix.editor.xsd.view.designer.container.XSDAttributeGroupComponentImpl;
import com.japisoft.editix.editor.xsd.view.designer.container.XSDComplexContentComponentImpl;
import com.japisoft.editix.editor.xsd.view.designer.container.XSDComplexTypeComponentImpl;
import com.japisoft.editix.editor.xsd.view.designer.container.XSDExtensionComponentImpl;
import com.japisoft.editix.editor.xsd.view.designer.container.XSDGroupComponentImpl;
import com.japisoft.editix.editor.xsd.view.designer.container.XSDRestrictionComponentImpl;
import com.japisoft.editix.editor.xsd.view.designer.container.XSDSimpleContentComponentImpl;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSDComponentFactory {

	public static XSDComponent getComponent( 
			Element e, 
			XSDComponentListener designer ) {
		String name = e.getLocalName();
		XSDComponent c = null;
		if ( "element".equals( name ) ) {
			c = new XSDElementComponentImpl();
		} else
		if ( "attribute".equals( name ) ) {
			c = new XSDAttributeComponentImpl();
		} else
		if( "complexType".equals( name ) ) {
			c = new XSDComplexTypeComponentImpl();
		} else
		if ( "sequence".equals( name ) ) {
			c = new XSDSequenceComponentImpl();
		} else
		if ( "choice".equals( name ) ) {
			c = new XSDChoiceComponentImpl();
		} else
		if ( "all".equals( name ) ) {
			c = new XSDAllComponentImpl();
		} else
		if ( "simpleContent".equals( name ) ) {
			c = new XSDSimpleContentComponentImpl();
		} else
		if ( "complexContent".equals( name ) ) {
			c = new XSDComplexContentComponentImpl();
		} else
		if ( "restriction".equals( name ) ) {
			c = new XSDRestrictionComponentImpl();
		} else
		if ( "extension".equals( name ) ) {
			c = new XSDExtensionComponentImpl();
		} else
		if ( "group".equals( name ) ) {
			c = new XSDGroupComponentImpl();
		} else
		if ( "attributeGroup".equals( name ) ) {
			c = new XSDAttributeGroupComponentImpl();
		} else
		if ( "anyAttribute".equals( name ) ) {
			c = new XSDAnyAttributeComponentImpl();
		} else
		if ( "key".equals( name ) ) 
			c = new XSDKeyComponentImpl();
		else
		if ( "keyref".equals( name ) ) 
			c = new XSDKeyrefComponentImpl();
		else
		if ( "unique".equals( name ) ) {
			c = new XSDUniqueComponentImpl();
		} else
		if ( "any".equals( name ) ) {
			c = new XSDAnyComponentImpl();
		}

		if ( c != null ) {
			c.setElement( e );
			e.setUserData( "ui", c, null );
			c.setComponentListener( designer );
		}
		return c;
	}

}
