package com.japisoft.editix.editor.xsd.view.element.simpletype;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.EventListener;
import java.util.EventObject;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.EventListenerList;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.w3c.dom.Element;

import com.japisoft.editix.editor.xsd.Changeable;
import com.japisoft.editix.editor.xsd.Factory;
import com.japisoft.editix.editor.xsd.toolkit.SchemaHelper;
import com.japisoft.editix.editor.xsd.view.View;
import com.japisoft.framework.ui.table.ExportableTable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RestrictionViewImpl extends ExportableTable implements View, MouseListener, Changeable {
	private Element initE = null;
	private Factory factory = null;	
	private ImageIcon deleteIcon = new ImageIcon( getClass().getResource( "element_delete.png" ) );

	public RestrictionViewImpl( Factory factory ) {
		this.factory = factory;
		setModel( new RestrictionModel() );
		getColumnModel().getColumn( 2 ).setCellRenderer( new CustomIconRenderer() );
		getColumnModel().getColumn( 2 ).setMaxWidth( 20 );
		getColumnModel().getColumn( 0 ).setCellEditor( new CustomFacetEditor() );
	}

	private boolean changed = false;
	
	public void init(Element schemaNode) {
		this.changed = false;
		this.initE = schemaNode;
		( ( RestrictionModel )getModel() ).fireTableDataChanged();
	}

	public boolean isChanged() { 
		return changed;
	}
	
	public JComponent getView() {
		return this;
	}

	public void dispose() {
		( ( CustomFacetEditor )getColumnModel().getColumn( 0 ).getCellEditor() ).dispose();
		initE = null;
	}

	public void stopEditing() {}

	public void addNotify() {
		super.addNotify();
		addMouseListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		removeMouseListener( this );
	}	

	public void refreshModel() {
		int selection = getSelectedRow();
		((RestrictionModel)getModel()).fireTableChanged(
				new TableModelEvent( getModel() ) );
		getSelectionModel().setSelectionInterval( selection, selection );
	}
	
	public void mouseClicked(MouseEvent ev) {
		int row = rowAtPoint( ev.getPoint() );
		int col = columnAtPoint( ev.getPoint() );
		if ( row > -1 && col == 2 ) {
			String element = ( String )getValueAt( row, 0 );
			String value = ( String )getValueAt( row, 1 );
			if ( element == null )
				return;
			if ( factory.confirmDialog( "Delete " + element + " with value '" + value + "' ?" ) ) {
				Element r = SchemaHelper.getRestrictionElement( initE );
				Element e = SchemaHelper.getChildAt(
						r, row, SchemaHelper.FACETS );
				if ( e != null ) {
					r.removeChild( e );
					refreshModel();
					changed = true;
					if ( row > 0 )
						getSelectionModel().setSelectionInterval( row - 1, row - 1 );
					else
						getSelectionModel().setSelectionInterval( 0, 0 );					
				}
			}
		}
	}

	class RestrictionModel extends AbstractTableModel {
		public int getColumnCount() {
			return 3;
		}
		public String getColumnName(int column) {
			if ( column == 0 )
				return "Facet";
			else
			if ( column == 1 )
				return "Value";
			else
				return "D";
		}
		public int getRowCount() {
			if ( initE == null )
				return 0;
			else {
				Element r = SchemaHelper.getRestrictionElement( initE );
				int res = 0;
				if ( r != null ) {
					res = SchemaHelper.getCountForChildren(
							r, SchemaHelper.FACETS );
				}
				return ( res + 1 );
			}
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			Element r = SchemaHelper.getRestrictionElement( initE );
			if ( r != null ) {
				Element e = SchemaHelper.getChildAt(
						r, rowIndex, SchemaHelper.FACETS );
				if ( e != null ) {
					if ( columnIndex == 1 )
						return e.getAttribute( "value" );
					else
						return e.getLocalName();
				}
				else
					return null;
			}
			return null;
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			Element r = SchemaHelper.getRestrictionElement( initE );
			changed = true;
			
			if ( r == null ) {
				Element parent = initE;
				SchemaHelper.removeChildren( initE );
				// Create a restriction component part
				// Create a simpleType ?
				if ( "element".equals( initE.getLocalName() ) || 
						"attribute".equals( initE.getLocalName() ) ) {
					Element st = SchemaHelper.createSimpleType( initE, null );
					initE.appendChild( st );
					parent = st;
				}
				r = SchemaHelper.createTag( initE, "restriction" );
				parent.appendChild( r );
				String type = initE.getAttribute( "type" );
				if ( type == null || "".equals( type ) )
					type = SchemaHelper.getSimpleType( initE, "string" );
				else
					if ( !"".equals( type ) ) {
						// Remove it
						initE.removeAttribute( "type" );
					}
				r.setAttribute( "base", type );
			}

			if ( r != null ) {
				Element ee = SchemaHelper.getChildAt(
						r, rowIndex, SchemaHelper.FACETS );
				if ( ee != null ) {
					if ( columnIndex == 1 )
						ee.setAttribute( "value", ( String )aValue );
				}
				else {
					if ( columnIndex == 0 ) {
						// Add a new facet
						Element facetElement = SchemaHelper.createTag( initE, ( String )aValue );
						r.appendChild( facetElement );
						refreshModel();
					}
				}
			}
			
			changed = true;
		}
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return true;
		}
	}

	class CustomIconRenderer extends JButton implements TableCellRenderer {
		public Component getTableCellRendererComponent(
				JTable table, 
				Object value, 
				boolean isSelected, 
				boolean hasFocus, 
				int row, 
				int column ) {
			if ( column == 2 ) {
				setIcon( deleteIcon );
			}
			setEnabled( row != ( getRowCount() - 1 ) );
			return this;
		}
	}

	class CustomFacetEditor implements TableCellEditor, ActionListener {
		private JComboBox cbb = new JComboBox( SchemaHelper.FACETS );		

		CustomFacetEditor() {
			cbb.addActionListener( this );
		}
		
		void dispose() {
			cbb.removeActionListener( this );
		}
		
		public void cancelCellEditing() {}

		public Object getCellEditorValue() {
			return cbb.getSelectedItem();
		}

		public boolean isCellEditable(EventObject anEvent) {
			if ( anEvent instanceof MouseEvent ) {
				MouseEvent e = ( MouseEvent )anEvent;
				int row = rowAtPoint( e.getPoint() );
				if ( row == getRowCount() -1 )
					return true;
				else
					return false;
			}
			return true;
		}

		public boolean shouldSelectCell(EventObject anEvent) {
			return false;
		}

		public boolean stopCellEditing() {
			return true;
		}

		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
			return cbb;
		}

		public void actionPerformed(ActionEvent e) {
			fireEditingStopped(new ChangeEvent(this));
			changed = true;
		}


		private EventListenerList listenerList = new EventListenerList();

		private void fireEditingStopped(ChangeEvent ee) {
			EventListener[] el = listenerList
					.getListeners(CellEditorListener.class);
			if (el != null) {
				for (int i = 0; i < el.length; i++) {
					((CellEditorListener) el[i]).editingStopped(ee);
				}
			}
		}

		public void addCellEditorListener(CellEditorListener l) {
			listenerList.add(CellEditorListener.class, l);
		}

		public void removeCellEditorListener(CellEditorListener l) {
			listenerList.remove(CellEditorListener.class, l);
		}

	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	@Override
	public void cut() {
	}

	@Override
	public void copy() {
	}

	@Override
	public void paste() {
	}


	
}
