package com.japisoft.editix.editor.xsd.view.element.simpletype;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.japisoft.editix.editor.xsd.Changeable;
import com.japisoft.editix.editor.xsd.Factory;
import com.japisoft.editix.editor.xsd.toolkit.SchemaHelper;
import com.japisoft.editix.editor.xsd.view.View;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SimpleTypeViewImpl extends JPanel implements View, ItemListener, Changeable {

	private JComboBox cb = new JComboBox(
			new String[] {
				"None",
				"Restriction",
				"List",
				"Union"
			} );
	private ContentPanel content;
	private Factory factory = null;

	public SimpleTypeViewImpl( Factory factory ) {
		this.factory = factory;
		this.content = new ContentPanel( factory );
		setLayout( new BorderLayout() );
		add( cb, BorderLayout.NORTH );
		add( content, BorderLayout.CENTER );
	}

	public void addNotify() {
		super.addNotify();
		cb.addItemListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		cb.removeItemListener( this );
	}

	private Element initE;
	private boolean changed = false;
	
	public void init( Element schemaNode ) {
		this.initE = schemaNode;
		content.init( schemaNode );
		changed = false;
		
		if ( schemaNode != null ) {
			cb.removeItemListener( this );
			Element parent = schemaNode;
			// Switch to the right content
			if ( SchemaHelper.hasChild( schemaNode, "simpleType" ) ) {
				parent = SchemaHelper.getChildAt( parent, 0, new String[] { "simpleType"} );
				if ( parent == null ) {
					parent = schemaNode;
				}
			}
			if ( SchemaHelper.hasChild( parent, "restriction" ) ) {
				content.show( "Restriction" );
				cb.setSelectedItem( "Restriction" );
			} else
			if ( SchemaHelper.hasChild( parent, "list" ) ) {				
				content.show( "List" );
				cb.setSelectedItem( "List" );
			} else
			if ( SchemaHelper.hasChild( parent, "union" ) ) {
				content.show( "Union" );
				cb.setSelectedItem( "Union" );
			} else {
				content.show( "None" );
				cb.setSelectedItem( "None" );
			}
			cb.addItemListener( this );
		} 
	}
	
	public boolean isChanged() { 
		return changed || content.isChanged();
	}

	public JComponent getView() {
		return this;
	}

	public void dispose() {
		this.initE = null;
		content.dispose();
	}
	public void stopEditing() {}

	public void itemStateChanged( ItemEvent e ) {
		if ( cb.getSelectedIndex() == 0 ) {
			// Remove all simpleType
			NodeList nl = initE.getChildNodes();
			for ( int i = 0; i < nl.getLength(); i++ ) {
				Node n = nl.item( i );
				if ( n instanceof Element ) {
					Element _ = ( Element )n;
					if ( "simpleType".equals( _.getLocalName() ) )
						initE.removeChild( _ );
				}
			}
		}
		content.show( ( String )e.getItem() );
		changed = true;
	}

	class ContentPanel extends JPanel implements Changeable {
		
		private RestrictionViewImpl re = null;
		private ListViewImpl lv = null;
		private UnionViewImpl uv = null;
		
		public ContentPanel( Factory factory ) {
			setLayout( new CardLayout() );
			add( new JPanel(), "None" );
			add( new JScrollPane( re = new RestrictionViewImpl( factory ) ), "Restriction" );
			add( new JScrollPane( ( lv = new ListViewImpl() ).getView() ), "List" );
			add( new JScrollPane( uv = new UnionViewImpl( factory ) ), "Union" );
		}
		
		@Override
		public boolean isChanged() {
			return re.isChanged() || lv.isChanged() || uv.isChanged();
		}
		
		public void show( String name ) {
			( ( CardLayout )getLayout() ).show( this, name );
		}

		public void init( Element schemaNode ) {
			for ( int i = 0; i < getComponentCount(); i++ ) {
				JComponent cc = ( JComponent )getComponent( i );
				if ( cc instanceof JScrollPane ) {
					cc = ( JComponent )( ( JScrollPane )cc ).getViewport().getView();
				}
				if ( cc instanceof View ) {
					( ( View )cc ).init( schemaNode );
				}
			}
		}
		public void dispose() {
			for ( int i = 0; i < getComponentCount(); i++ ) {
				JComponent cc = ( JComponent )getComponent( i );
				if ( cc instanceof JScrollPane ) {
					cc = ( JComponent )( ( JScrollPane )cc ).getViewport().getView();
				}
				if ( cc instanceof View ) {
					( ( View )cc ).dispose();
				}				
			}			
		}
	}

	@Override
	public void cut() {
	}

	@Override
	public void copy() {
	}

	@Override
	public void paste() {
	}
	
}
