package com.japisoft.editix.editor.xsd.view.element.simpletype;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.EventListener;
import java.util.EventObject;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.EventListenerList;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.w3c.dom.Element;

import com.japisoft.editix.editor.xsd.Changeable;
import com.japisoft.editix.editor.xsd.Factory;
import com.japisoft.editix.editor.xsd.toolkit.SchemaHelper;
import com.japisoft.editix.editor.xsd.view.View;
import com.japisoft.framework.ui.table.ExportableTable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class UnionViewImpl extends ExportableTable implements View, MouseListener, Changeable {
	private Element initE;
	private Factory factory = null;	
	private ImageIcon deleteIcon = new ImageIcon( getClass().getResource( "element_delete.png" ) );

	public UnionViewImpl( Factory factory ) {
		this.factory = factory;
		setModel( new UnionModel() );
		getColumnModel().getColumn( 0 ).setCellEditor( new SimpleTypeEditor() );
		getColumnModel().getColumn( 1 ).setCellRenderer( new CustomIconRenderer() );
		getColumnModel().getColumn( 1 ).setMaxWidth( 20 );		
		addMouseListener( this );
	}

	private boolean changed = false;
	
	public void init(Element schemaNode) {
		this.initE = schemaNode;
		changed = false;
		( ( UnionModel )getModel() ).fireTableDataChanged();
	}

	public boolean isChanged() { return changed; }
	
	public JComponent getView() {
		return this;
	}

	public void dispose() {
		( ( SimpleTypeEditor )getColumnModel().getColumn( 0 ).getCellEditor() ).dispose();
		removeMouseListener( this );
		initE = null;
	}

	public void stopEditing() {
	}

	public void mouseClicked(MouseEvent ev) {
		int row = rowAtPoint( ev.getPoint() );
		int col = columnAtPoint( ev.getPoint() );
		if ( row > -1 && col == 1 ) {
			String element = ( String )getValueAt( row, 0 );
			if ( factory.confirmDialog( "Delete " + element + " ?" ) ) {
				SchemaHelper.deleteUnionType( initE, row );
				( ( UnionModel )getModel() ).fireTableDataChanged();
				changed = true;
			}
		}
	}

	public void mouseEntered(MouseEvent e) {}

	public void mouseExited(MouseEvent e) {}

	public void mousePressed(MouseEvent e) {}

	public void mouseReleased(MouseEvent e) {}
	
	class UnionModel extends AbstractTableModel {
		public int getColumnCount() {
			return 2;
		}
		public String getColumnName(int column) {
			if ( column == 0 )
				return "Item Type";
			else
				return "D";
		}
		public int getRowCount() {
			if ( initE == null )
				return 0;
			else {
				// Search for union element
				String[] qt = SchemaHelper.getUnionTypes( initE );	
				return ( qt.length + 1 );
			}
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			String[] qt = SchemaHelper.getUnionTypes( initE );
			if ( rowIndex < qt.length ) {
				return qt[ rowIndex ];
			} else
				return null;
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			boolean mustFire = ( rowIndex == getRowCount() - 1 );
			SchemaHelper.updateUnion( initE, rowIndex, ( String )aValue );
			if ( mustFire ) {
				( ( UnionModel )getModel() ).fireTableDataChanged();
			}
		}

		public boolean isCellEditable(int rowIndex, int columnIndex) {
			if ( columnIndex == 0 )
				return true;
			else
				return false;
		}
	}
	
	class CustomIconRenderer extends JButton implements TableCellRenderer {
		public CustomIconRenderer() {
			setIcon( deleteIcon );
		}
		public Component getTableCellRendererComponent(
				JTable table, 
				Object value, 
				boolean isSelected, 
				boolean hasFocus, 
				int row, 
				int column ) {
			setEnabled( row != ( getRowCount() - 1 ) );
			return this;
		}
	}

	class SimpleTypeEditor implements TableCellEditor, ActionListener {
		private JComboBox cbb = new JComboBox();		

		SimpleTypeEditor() {
			cbb.setEditable( false );	// Should be true but bug
			cbb.addActionListener( this );
		}

		void dispose() {
			cbb.removeActionListener( this );
		}
		
		public void cancelCellEditing() {}

		public Object getCellEditorValue() {
			return cbb.getSelectedItem();
		}

		public boolean isCellEditable(EventObject anEvent) {
			return true;
		}

		public boolean shouldSelectCell(EventObject anEvent) {
			return false;
		}

		public boolean stopCellEditing() {
			return true;
		}

		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
			String[] content = SchemaHelper.getType( initE.getOwnerDocument().getDocumentElement(), false, true );
			cbb.removeAllItems();
			for ( int i = 0; i < content.length; i++ ) {
				cbb.addItem( content[ i ] );
			}
			cbb.setSelectedItem( value );
			return cbb;
		}

		public void actionPerformed(ActionEvent e) {
			fireEditingStopped(new ChangeEvent(this));
		}

		private EventListenerList listenerList = new EventListenerList();

		private void fireEditingStopped(ChangeEvent ee) {
			EventListener[] el = listenerList
					.getListeners(CellEditorListener.class);
			if (el != null) {
				for (int i = 0; i < el.length; i++) {
					((CellEditorListener) el[i]).editingStopped(ee);
				}
			}
		}

		public void addCellEditorListener(CellEditorListener l) {
			listenerList.add(CellEditorListener.class, l);
		}

		public void removeCellEditorListener(CellEditorListener l) {
			listenerList.remove(CellEditorListener.class, l);
		}
	}

	@Override
	public void cut() {
	}

	@Override
	public void copy() {
	}

	@Override
	public void paste() {
	}

}
