package com.japisoft.editix.javascript;

import javax.script.Bindings;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

import com.japisoft.editix.plugin.EditiXManager;
import com.japisoft.editix.ui.EditixFrame;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JavaScriptFactory {

	private static JavaScriptFactory THIS = null;
	
	private JavaScriptFactory() {
		THIS = this;
	}
	
	public static JavaScriptFactory newFactory() {
		if ( THIS == null )
			THIS = new JavaScriptFactory();
		return THIS;
	}
	
	private String[] defaultFunctions = {
		"function alert(msg) {com.japisoft.editix.ui.EditixFactory.buildAndShowInformationDialog(msg);}",
		"function prompt() {"
		+ "if (arguments.length > 1)"
			+ "return com.japisoft.editix.ui.EditixFactory.buildAndShowInputDialog(arguments[0],arguments[1]);"
		+ "if (arguments.length == 1)"
			+ "return com.japisoft.editix.ui.EditixFactory.buildAndShowInputDialog(arguments[0],\"\");"
		+ "return \"\";}"
	};
	
	public ScriptEngine engine() {
		ScriptEngineManager manager = new ScriptEngineManager();
		ScriptEngine engine = manager.getEngineByExtension( "js" );
		if ( engine == null )
			engine = manager.getEngineByName( "javascript" );
		if ( engine == null )
			engine = new EmptyEngine();

		Bindings b = engine.getBindings(ScriptContext.GLOBAL_SCOPE);
		try {
			for ( String func : defaultFunctions )
				engine.eval( func );
		} catch( ScriptException exc ) {
			System.err.println( "Can't use this JavaScript function :" + exc.getMessage() );
		}
		if ( b != null ) {
			b.put( "console", new Console() );
			b.put( "EditiXManager", EditiXManager.getInstance());
			b.put( "EditixManager", EditiXManager.getInstance());
		}
		return engine;
	}
	
}
