package com.japisoft.editix.main.steps;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.URL;

import org.xml.sax.InputSource;

import com.japisoft.dtdparser.DTDMapper;
import com.japisoft.dtdparser.node.RootDTDNode;
import com.japisoft.framework.xml.SchemaLocator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditixDTDMapper implements DTDMapper {

	private EditixDTDMapper() {
		super();
	}

	private static EditixDTDMapper MAPPER = null;

	public static EditixDTDMapper getInstance() {
		if (MAPPER == null)
			MAPPER = new EditixDTDMapper();
		return MAPPER;
	}
	
	public boolean isCachedEnabled() {
		return true;
	}
	
	private String cacheLocation = null;
	
	public File getPathForURL( String httpURL ) {
		if ( cacheLocation == null ) {
			URL u = ClassLoader.getSystemClassLoader().getResource( "dtd" );
			
			if ( u == null )
				return null;
			
			String cache = u.toExternalForm();
			cache = cache.replaceAll( "%20", " " );

		File p = null;
		if ( cache.startsWith( "file://" ) ) 
			p =
				new File(
					cache.substring( 7 ) );
		else if ( cache.startsWith( "file:" ) )
			p =
				new File(
					cache.substring( 5 ) );
		
		if ( p != null )
			cacheLocation = p.toString();
		else 
			return null;
		}
		
		String _ = httpURL.substring( 7 );
		return new File( cacheLocation, _ );
	}
	
	/** @return the directory for the cache */
	public File updateCache( RootDTDNode root, String url ) {
		if ( !url.toLowerCase().startsWith( "http://" ) )
			return null;
		File f = getPathForURL( url );
		if ( f == null )
			return null;
		if ( f.exists() )
			return f;
		File parent = f.getParentFile();
		parent.mkdirs();
		try {
			FileWriter writer = new FileWriter( f );
			try {
				root.writeDTD( new PrintWriter( writer ) );
			} finally {
				writer.close();
			}
		} catch( IOException exc ) {
		}
		return f;
	}

	public InputStream getStream( String url ) throws IOException {
		if (url.startsWith( "http://" ) ) {
			String _ = url.substring( 7 );
			File path = getPathForURL( url );
			if ( path != null && path.exists() ) {
				return new FileInputStream( path.toString() );
			} else {
				
				String resolvedPath = EditixEntityResolver.getInstance().resolveCatalog(null, url );
				if ( resolvedPath != null ) {
					File f = new File( resolvedPath );
					if ( f.exists() )
						return new FileInputStream( resolvedPath );
				}
				
//				InputSource source = 
//					EditixEntityResolver.getInstance().resolveEntity( null, url );
//				if ( source != null ) {
//					try {
//						InputStream input = source.getByteStream();
//						if ( input != null )
//							return input;
//					} catch (Exception e) {
//					}
//				}

				SchemaLocator locator = new SchemaLocator(
						null,
						url,
						null );				
				
//				return new URL( url ).openStream();
				try {
					return locator.getInputStream();
				} catch( Exception exc ) {
					throw new IOException( exc );
				}
			}
		} else {
			String file = url;
			File f = new File( file );
			if ( f.exists() ) {
				return new FileInputStream( f );
			}
		}
		return null;
	}
	
}
