package com.japisoft.editix.main.steps;

import java.util.ArrayList;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;

import org.netbeans.swing.plaf.aqua.AquaLFCustoms;

import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.ui.xmlpad.EditixLook;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationStep;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.toolkit.Logger;
import com.japisoft.xmlpad.look.LookManager;
import com.jgoodies.looks.plastic.PlasticLookAndFeel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class LookAndFeelApplicationStep implements ApplicationStep {

	public boolean isFinal() {
		return false;
	}

	public void start(String[] args) {
		// Build the default list
		UIManager.LookAndFeelInfo[] uf = UIManager
				.getInstalledLookAndFeels();
		ArrayList l = new ArrayList();
		l.add("DEFAULT");
		for (int i = 0; uf != null && i < uf.length; i++) {
			l.add(uf[i].getName());
		}
		String[] _ = new String[l.size()];
		for (int i = 0; i < l.size(); i++)
			_[i] = (String) l.get(i);

		String className = Preferences.getPreference( "interface",
				"lookAndFeel", _)[0];

		if ( EditixApplicationModel.MACOSX_MODE ) {
			l.add( "EDITIX" );
			UIManager.put( "tabContainer.controlButtons.disabled", true );
			try {
				Class.forName(
						"com.japisoft.editix.MacDocumentHandler" )
						.newInstance();
			} catch ( Throwable th ) {
				System.err
						.println( "Can't managed MacDocumentHandler : "
								+ th.getMessage() );
			}
			AquaLFCustoms aqua = new AquaLFCustoms();
			Object[] keyValue = aqua
					.createApplicationSpecificKeysAndValues();
			for (int i = 0; i < keyValue.length; i += 2) {
				UIManager.put(keyValue[i], keyValue[i + 1]);
			}

		}

		LookAndFeel look = null;
		if ("DEFAULT".equals(className)) {
			if (EditixApplicationModel.MACOSX_MODE) {
				look = UIManager.getLookAndFeel();
			} else {
				className = EditiXLookAndFeel.class.getName();
			}
		} else
		if ( "EDITIX".equals( className ) ) {
			className = EditiXLookAndFeel.class.getName();
		}
		try {

			if (look == null) {
				// Search for the class name
				for ( int i = 0; i < uf.length; i++ ) {
					if ( uf[ i ].getName().equals( className ) )
						className = uf[ i ].getClassName();
				}
				if ( className != null )
					look = ( LookAndFeel ) Class.forName( className ).newInstance();
			}

		} catch (Throwable th) {
			Logger.addWarning("Can't use this lookAndFeel "
					+ className);
			look = new PlasticLookAndFeel();
		}
		if ( look != null ) {
			try {
				UIManager.setLookAndFeel(look);
			} catch( Exception exc ) {
				ApplicationModel.debug( exc );
			}
		}

		LookManager.setCurrentLook( new EditixLook() );

	}

	public void stop() {
	}
	
	@Override
	public void quit() {
	}

}
