package com.japisoft.editix.project;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class Path {

	public static String getAbsolutePath( String refPath, String relPath ) {
		
		if ( !relPath.startsWith( "." ) )
			return relPath;
		
		File rp = new File( refPath ).getParentFile();
		String[] rps = relPath.split( "/" );
		for ( int i = 0; i < rps.length; i++ ) {
			if ( ".".equals( rps[ i ] ) ) {
				continue;
			}
			if ( "..".equals( rps[ i ] ) ) {
				rp = rp.getParentFile();
			} else {
				rp = new File( rp, rps[ i ] );
			}
			if ( rp == null ) {
				return relPath;	// ??
			}
		}
		return rp.toString();

	}

	public static String getRelativePath( String refPath, String absPath ) {
		
		List lrp = getPathContent( refPath );
		List lp = getPathContent( absPath );
		
		int maxCommon = -1;
		
		for ( int i = 0; i < lp.size(); i++ ) {
			
			File f = ( File )lp.get( i );
			if ( i == lrp.size() )
				break;
			File fr = ( File )lrp.get( i );
			if ( f.equals( fr ) ) {
				maxCommon = i;
			} else
				break;

		}
		
		if ( maxCommon == -1 )
			return absPath;
		
		StringBuffer sb = new StringBuffer();		
		
		for ( int j = maxCommon + 1; j < lrp.size() - 1; j++ ) {
			if ( sb.length() > 0 )
				sb.append( "/" );
			sb.append( ".." );
		}

		if ( sb.length() == 0 )
			sb.append( "." );
		
		for ( int j = maxCommon + 1; j < lp.size(); j++ ) {
			sb.append( "/" );
			sb.append( ( ( File )lp.get( j ) ).getName() );
		}
		
		return sb.toString();
	}
	
	private static List getPathContent( String path ) {
		
		ArrayList r = new ArrayList();
		File f = 
			new File( path );

		while ( f != null ) {
			
			r.add( 0, f );
			f = f.getParentFile();
			
		}
		
		return r;
		
	}
	
	public static void main( String[] args ) {

		System.out.println( 
				Path.getRelativePath( "c:/to/test.prj", "c:/toto/titi/oo.txt" ) );
				
		System.out.println( 
				Path.getRelativePath( "c:/toto/test.prj", "c:/toto/titi/oo.txt" ) );

		System.out.println( 
				Path.getRelativePath( "c:/toto/titi/test.prj", "c:/toto/titi/oo.txt" ) );
		
		System.out.println( 
				Path.getRelativePath( "c:/toto/titi/tata/test.prj", "c:/toto/titi/oo.txt" ) );

		System.out.println( 
				Path.getRelativePath( "c:/toto/titi/tata/test.prj", "c:/tot/oo.txt" ) );
		
		System.out.println(
				Path.getAbsolutePath( "c:/toto/test.prj", "./titi/oo.txt" ) );

		System.out.println(
				Path.getAbsolutePath( "c:/toto/test.prj", "../titi/oo.txt" ) );
		
		System.out.println(
				Path.getAbsolutePath( "c:/toto/test.prj", "../../../titi/oo.txt" ) );
		
		System.out.println(
				Path.getAbsolutePath( "/a/b/c.txt", "../../../d.txt" ) );
		
	}
	
}
