package com.japisoft.editix.script;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenu;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.xml.parser.FPParser;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ScriptModel {

	private static ScriptModel THIS = null;

	public static ScriptModel getInstance() {
		if ( THIS == null )
			THIS = new ScriptModel();
		return THIS;
	}

	private File path = new File( ApplicationModel.getAppUserPath(), "scripts.xml" );
	
	private List<Script> scripts = null;
	
	private ScriptModel() {
		load();
	}
	
	private void load() {
		scripts = new ArrayList<Script>();
		if ( path.exists() ) {
			try {
				FPParser fp = new FPParser();
				Document doc = fp.parse( new FileInputStream( path ) );
				FPNode root = ( FPNode )doc.getRoot();
				for ( int i = 0; i < root.childCount(); i++ ) {
					FPNode script = root.childAt( i );
					scripts.add( new BasicScript( script.getAttribute( "name" ), new File( script.getAttribute( "path" ) ), script.getAttribute( "shortkey" ) ) );
				}
			} catch( Exception exc ) {
				System.err.println( "Can't parse " + path );
				ApplicationModel.debug( exc );
			}
		}
	}
	
	public List<Script> getScripts() {
		return scripts;
	}
	
	public void setScripts( List<Script> s ) throws IOException {
		this.scripts = s;
		save();
	}

	public void save() throws IOException {
		FPNode root = new FPNode( FPNode.TAG_NODE, "scripts" );
		for ( Script s : scripts ) {
			root.appendChild( new FPNode( FPNode.TAG_NODE, "script" ) ).att( "path", s.getPath().toString() ).att( "name", s.getName() ).att( "shortkey", s.getShortkey() );
		}
		FileWriter fw = new FileWriter( path );
		fw.write( root.getRawXML() );
		fw.close();
		synchroMenu();
	}

	public void synchroMenu() {
		JMenu menu = ApplicationModel.INTERFACE_BUILDER.getMenu( "runscript" );
		if ( menu == null ) {
			System.err.println( "Can't find the menu runscript ?" );
		} else {
			menu.removeAll();
			for ( int i = 0; i < getScripts().size(); i++ ) {
				menu.add( new ScriptAction( getScripts().get( i ) ) ); 
			}
			menu.setEnabled( menu.getItemCount() > 0 );
		}
	}

}
