package com.japisoft.editix.ui;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.filechooser.FileFilter;

import com.japisoft.editix.document.DocumentModel;
import com.japisoft.xmlpad.CaretListener;
import com.japisoft.xmlpad.DocumentStateListener;
import com.japisoft.xmlpad.LocationEvent;
import com.japisoft.xmlpad.LocationListener;

import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.XMLDocumentInfo;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class Factory {

	public static XMLContainer buildNewContainer() {
		return buildNewContainer(false);
	}

	public static XMLContainer buildNewContainer( String file ) {
		int i = file.lastIndexOf( "." );
		String ext = null;
		if ( i > -1 )
			ext = file.substring( i + 1 );
		XMLContainer container = new EditixXMLContainer();
		XMLDocumentInfo info = DocumentModel.getDocumentForExt( ext );
		if ( info != null )
			container.setDocumentInfo( info );
		return container;
	}

	/** @return a new XMLContainer */
	public static XMLContainer buildNewContainer(boolean def) {
		XMLContainer container = new EditixXMLContainer();
		if (def) {
			container.setDocumentInfo( DocumentModel.getDefaultDocument() );
			container.getDocumentInfo().setCurrentDocumentLocation("Empty.xml");
		}
		return container;
	}

	private static JFileChooser fileChooser = null;

	/** @return a FileChooser for opening file */
	public static JFileChooser buildFileChooser() {
		if (fileChooser != null)
			return fileChooser;
		JFileChooser fc = new JFileChooser();
		fileChooser = fc;
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setMultiSelectionEnabled(true);
		
		FileFilter first = null;
		
		// Create one filter by documentInfo
		for ( int i = 0; i < DocumentModel.getDocumentCount(); i++ ) {
			XMLDocumentInfo info = DocumentModel.getDocumentAt( i );
			fc.addChoosableFileFilter( info.getFileFilter() );
			if ( first == null )
				first = info.getFileFilter();
		}
		fc.setFileFilter( first );
		return fc;
	}

	private static JFileChooser projectChooser = null;

	/** @return a FileChooser for opening a project file */
	public static JFileChooser buildProjectFileChooser() {
		if (projectChooser != null)
			return projectChooser;
		JFileChooser fc = new JFileChooser();
		projectChooser = fc;
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setFileFilter(new FileFilter() {
			public String getDescription() {
				return "*.pre (editix project file)";
			}
			public boolean accept(File f) {
				return ( f.isDirectory() || f.toString().toLowerCase().endsWith(".pre") );
			}
		});
		return fc;
	}

	public static void buildAndShowErrorDialog(String message) {
		JOptionPane.showMessageDialog(
			EditixFrame.THIS,
			message,
			"Error",
			JOptionPane.ERROR_MESSAGE);
	}

	public static String buildAndShowInputDialog( String title ) {
		return JOptionPane.showInputDialog(
			EditixFrame.THIS,
			title );
	}

	public static void buildAndShowInformationDialog(String message) {
		JOptionPane.showMessageDialog(
			EditixFrame.THIS,
			message,
			"Info",
			JOptionPane.INFORMATION_MESSAGE );
	}

	public static int buildAndShowChoiceDialog(String message) {
		return JOptionPane.showConfirmDialog(
			EditixFrame.THIS,
			message,
			"Choice",
			JOptionPane.YES_NO_OPTION);
	}

	/////////////////////////////////////////////////////////////////////////////////

	static class EditixXMLContainer extends XMLContainer implements 
			LocationListener, DocumentStateListener, CaretListener {
		
		public EditixXMLContainer() {
			super();
			setToolBarAvailable( false );
			setStatusBarAvailable( false );
			setPopupAvailable( false );
			setTreePopupAvailable( false );
		}

		public void addNotify() {
			setLocationListener( this );	
			addDocumentStateListener( this );
			setCaretListener( this );
		}

		public void removeNotify() {
			unsetLocationListener();
			removeDocumentStateListener( this );
			unsetCaretListener();
		}

		public void setUIReady( boolean ok ) {
			super.setUIReady( ok );
			if ( ok )
				addNotify();
			else
				removeNotify();
		}
		
		public void locationChanged(LocationEvent e) {
			EditixStatusBar.ACCESSOR.setXPathLocation( e.getXPathLocation() );
		}
		public void documentModified( XMLContainer source ) {
		}

		public void newDocument( XMLContainer source ) {
		}
		
		public void errorFound(
			String errorMessage,
			int line,
			boolean temporary) {
			EditixStatusBar.ACCESSOR.setError( null, true, null, errorMessage, line );
			int i = EditixFrame.THIS.getMainTabbedPane().getSelectedIndex();
			//EditixFrame.ACCESSOR.getMainTabbedPane().setForegroundAt( i, Color.RED );
			if ( temporary ) {
				getEditor().highlightErrorLine( line );
			}
		}

		public JPopupMenu getCurrentPopup() {
			return EditixFrame.THIS.getBuilder().getPopup( "EDITOR" );
		}

		protected boolean useCustomPopupMenu() {
			return true;
		}


		public void noErrorFound(boolean temporary) {
			EditixStatusBar.ACCESSOR.setError( null, true, null, null, 0 );
			int i = EditixFrame.THIS.getMainTabbedPane().getSelectedIndex();
			//EditixFrame.ACCESSOR.getMainTabbedPane().setForegroundAt( i, Color.BLACK );
			if ( temporary )
				getEditor().removeHighlightedErrorLine();
		}

		public void caretLocation(int col, int line) {
			EditixStatusBar.ACCESSOR.setLocation( col, line );
		}

	}
	
}
