package com.japisoft.editix.ui.panels;

import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.Action;

import com.japisoft.editix.action.panels.PanelAction;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.PanelStateListener;
import com.japisoft.editix.ui.PanelStateManager;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PanelManager {

	public static void showByAction( String actionName ) {
		Panel p = getPanelByAction( actionName );
		if ( p != null )
			p.showPanel();
	}

	public static void hideByAction( String actionName ) {
		Panel p = getPanelByAction( actionName );
		if ( p != null )
			p.hidePanel();		
	}

	public static void initByAction( String actionName ) {
		Panel p = getPanelByAction( actionName );
		if ( p != null )
			p.init();		
	}

	// Save state for the 'init' method
	public static void saveState( boolean resetState ) {
		Iterator it = ActionModel.getActionsName();
		StringBuffer state = new StringBuffer();
		while ( it.hasNext() ) {
			String actionName = ( String )it.next();
			Action a = ActionModel.restoreAction( actionName );
			if ( ( a instanceof PanelAction ) && 
					( ( PanelAction ) a ).isPrepared() ) {
				( ( PanelAction )a ).preparePanel().stop();
				if ( state.length() > 0 )
					state.append( ";" );
				state.append(
						actionName ).append( 
							"=" ).append( 
								( ( PanelAction )a ).preparePanel().isShown() );
				if ( resetState )
					( ( ( PanelAction )a ).preparePanel() ).setState( false );
			}
		}
		Preferences.setPreference( 
				Preferences.SYSTEM_GP,
				"panelmanager.state",
				state.toString() );
		if ( shownPanel != null )
			Preferences.setPreference( 
				Preferences.SYSTEM_GP,
				"panelmanager.shownpanel",
				shownPanel );		
	}

	private static String shownPanel;
	
	public static void saveShownState( String id ) {
		shownPanel = id;
	}

	public static void restoreState() {
		String oldState = Preferences.getPreference( 
				Preferences.SYSTEM_GP,
				"panelmanager.state",
				( String )null );
		if ( oldState != null ) {
			StringTokenizer st = new StringTokenizer(
					oldState, ";", false );
			while ( st.hasMoreTokens() ) {
				String t = st.nextToken();
				int i = t.indexOf( "=" );
				String actionName = t.substring( 0, i );
				boolean state = "true".equals( t.substring( i + 1 ) );
				if ( state )
					showByAction( actionName );
			}
		}
		shownPanel = Preferences.getPreference( 
				Preferences.SYSTEM_GP,
				"panelmanager.shownpanel",
				(String)null );		
		if ( shownPanel != null ) {
			EditixFrame.THIS.dockingSpace.showPane( shownPanel );
		}
	}

	public static Panel getPanelByAction( String actionName ) {
		Action a = ActionModel.restoreAction( actionName );
		if ( a instanceof PanelAction ) {
			return ( ( PanelAction )a ).preparePanel();
		}
		return null;
	}

	public static Panel getShownPanelByAction( String actionName ) {
		Action a = ActionModel.restoreAction( actionName );
		if ( a instanceof PanelAction ) {
			PanelAction pa = ( PanelAction )a;
			if ( pa.isPrepared() ) {
				return pa.preparePanel();
			}
		}
		return null;
	}

	private static void resetCurrentXMLContainerForPanels( XMLContainer container ) {
		Iterator it = ActionModel.getActionsName();
		while ( it.hasNext() ) {
			String actionName = ( String )it.next();
			Action a = ActionModel.restoreAction( actionName );
			if ( ( a instanceof PanelAction ) && 
					( ( PanelAction ) a ).isPrepared() ) {
				( ( ( PanelAction )a ).preparePanel() ).setCurrentXMLContainer( 
						container );
			}
		}
	}

	static {
		PanelStateManager.addPanelStateListener( new RawPanelStateListener() );
	}

	static class RawPanelStateListener implements PanelStateListener {
		public void newPath(String previousPath, String newPath) {
		}
		public void setCurrentXMLContainer(XMLContainer container) {
			resetCurrentXMLContainerForPanels( container );
		}
		public void close(XMLContainer container) {
		}
	}

}
