package com.japisoft.editix.ui.panels.project2.synchro;

import java.io.File;
import java.io.IOException;
import org.apache.commons.io.FileUtils;
import com.japisoft.editix.ui.panels.project2.Node;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FileSynchronizer extends AbstractSynchronizer {

	public String getName() {
		return "LOCAL/NETWORK";
	}

	public void uploadIt( File rootPath, Node node ) throws IOException {
		String relativePath = getRelativePath( rootPath, node.getPath() );
		File source = new File( rootPath, relativePath );
		File target = new File( getRemotePath() + relativePath );
		if ( source.isDirectory() ) {
			if ( !target.exists() )
				target.mkdirs();
			FileUtils.copyDirectoryToDirectory( source, target.getParentFile() );
		} else {
			File parentPath = target.getParentFile();
			if ( !parentPath.exists() )
				parentPath.mkdirs();
			FileUtils.copyFile( source, target );
		}		
	}

	public void downloadIt( File rootPath, Node node ) throws IOException {
		String relativePath = getRelativePath( rootPath, node.getPath() );
		File target = new File( rootPath, relativePath );
		File source = new File( getRemotePath() + relativePath );
		if ( source.isDirectory() ) {
			if ( !target.exists() )
				target.mkdirs();
			FileUtils.copyDirectoryToDirectory( source, target.getParentFile() );
		} else {
			File parentPath = target.getParentFile();
			if ( !parentPath.exists() )
				parentPath.mkdirs();			
			FileUtils.copyFile( source, target );
		}
	}

	public String getRemotePath() {
		String tmp = getProperty( "path" );
		if ( !tmp.endsWith( "/" ) )
			tmp += "/";
		return tmp;
	}

	public boolean test() {
		return false;
	}

	@Override
	public String toString() {
		return super.toString() + " [" + getRemotePath() + "]";
	}

}
