package com.japisoft.editix.xslt.debug;

import javax.swing.SwingUtilities;
import com.japisoft.editix.ui.xslt.XSLTEditor;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.bookmark.DefaultBookmarkContext;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
abstract class CommonTraceListener implements TracableListener {

	boolean waitMode = false;
	boolean noWait = false;
	boolean continueMode = false;
	IXMLPanel panel;
	DefaultBookmarkContext bmContext;

	public CommonTraceListener(IXMLPanel panel) {
		this.panel = panel;
		bmContext = 
			(DefaultBookmarkContext) panel.getBookmarkContext();
	}

	protected void showCurrentLine(String uri,int cl) {
		if ( cl <= 0 )
			return;
		final int currentLine = cl;
		final String furi = uri;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				panel.setProperty( 
						XSLTEditor.DEBUG_CURRENT_LINE, 
						new Object[] { furi, currentLine } 
				);
			}
		});
	}

	protected void unhighLight(boolean background) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				if ( panel != null ) {
					panel.setProperty( 
						XSLTEditor.DEBUG_CURRENT_LINE, 
						new Object[] { null, -1 } 
					);
				}
			}
		});		
	}

	protected void refreshResult() {
		if (panel instanceof XSLTEditor) {
			( ( XSLTEditor ) panel ).loadResultFile();
		}
	}

	protected void waitForDebug() {
		try {

			waitMode = true;
			
			while ( waitMode ) {
				Thread.sleep( 50 );
			}
			
		} catch (InterruptedException exc) {
		}
	}

	public synchronized void continueBreakpoint() {
		continueMode = false;
		waitMode = false;
		notify();
	}

	public synchronized void continueNextElement() {
		continueMode = true;
		waitMode = false;
		notify();
	}

	public synchronized void terminateAll() {
		unhighLight( false );
		waitMode = false;
		noWait = true;
		notify();
	}

	public void dispose() {
		refreshResult();
		unhighLight( false );
		panel = null;
		bmContext = null;
	}

}
