package com.japisoft.editix.xslt.debug;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.sf.saxon.expr.LetExpression;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.expr.instruct.TraceExpression;

import net.sf.saxon.om.Item;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.om.SequenceIterator;
import net.sf.saxon.om.StructuredQName;
import net.sf.saxon.trace.InstructionInfo;
import net.sf.saxon.trans.XPathException;
import net.sf.saxon.xpath.XPathEvaluator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class Saxon2NodeDebug implements NodeDebug {

	private TraceExpression ii;
	private XPathContext xc;
	private String systemUri;
	private VariablesContext cv;
	
	public Saxon2NodeDebug(
			TraceExpression ii,
			XPathContext xc,
			VariablesContext cv
	) {
		systemUri = 
			ii.getSystemId();
		this.ii = ii;
		this.xc = xc;
		this.cv = cv;
	}

	public String getSystemUri() {	
		return systemUri;
	}	
	
	public String getLocalName() {
		TraceExpression te = ( TraceExpression )ii;
		InstructionInfo info = te.getInstructionInfo();
		StructuredQName qName = info.getObjectName();
		String name = (String)info.getProperty("name");
		if ( name != null )
			return name;
		if ( qName != null ) {
			if ( isVariable() ) {
				getValue();				
				return "xsl:variable [" + qName.getDisplayName() + "]";
			}
			return qName.getDisplayName();
		}
		Iterator props = info.getProperties();
		String addOn = "";
		while ( props.hasNext() ) {
			String prop = ( String )props.next();
			Object val = info.getProperty( prop );
			if ( "match".equals( prop ) ) {
				addOn += " [" + val + "]";
			} else
			if ( "name".equals( prop ) ) {
				addOn += " [" + val + "]";
			}
		}
		return net.sf.saxon.om.StandardNames.getDisplayName( 
			te.getConstructType() 
		) + addOn;			
	}

	public String getVariableName() {
		TraceExpression te = ( TraceExpression )ii;
		InstructionInfo info = te.getInstructionInfo();
		StructuredQName qName = info.getObjectName();
		return qName.getDisplayName();
	}
	
	public boolean isVariable() {
		return ii.getConstructType() == 2013;
	}
	
	public String getPrefix() {
		return null;
	}
	
	public String getNamespace() {
		TraceExpression te = ( TraceExpression )ii;
		InstructionInfo info = te.getInstructionInfo();		
		String name = (String)info.getProperty("name");
		if ( name == null ) {
			StructuredQName qName = info.getObjectName();
			if ( qName != null ) {
				name = qName.getDisplayName();
			}
		}
		if ( name == null || isVariable() ) {
			return "http://www.w3.org/1999/XSL/Transform";			
		}
		return null;
	}	

	public int getLineNumber() {		
		return ii.getLineNumber();
	}

	public String getAttributeValue(String attName) {
		return null;
	}
	
	public Object getValue() {	
		TraceExpression te = ( TraceExpression )ii;
		InstructionInfo info = te.getInstructionInfo();
		Iterator props = info.getProperties();
		while ( props.hasNext() ) {
			String prop = ( String )props.next();
			Object val = info.getProperty( prop );
			
			if ( val instanceof LetExpression ) {
				LetExpression le = ( LetExpression )val;
				try {
					Object value = le.eval(xc);
					return value;
				} catch( XPathException exc ) {
					return null;
				}
			}
		}
		return null;
	}

	public Object evalXPath(String xpath) throws Exception {
		SequenceIterator it = xc.getCurrentIterator();
		Item current = it.current();
		XPathEvaluator xp = new XPathEvaluator();
		return xp.evaluate( xpath, current );
	}

	public ArrayList getXPathContext() {

		ArrayList res = null;
		
		SequenceIterator it = xc.getCurrentIterator();
		try {
			Item current = it.current();
			it = it.getAnother();
			Item i;			
			do {
				i = it.next();
				if ( i != null ) {
					if ( i instanceof NodeInfo ) {
						NodeInfo ni = ( NodeInfo )i;
						if ( res == null )
							res = new ArrayList();						
						res.add(							
							new XPathNodeContextImpl(
									ni.getNodeKind() == 3 ? "text()" : ni.getDisplayName(),
									ni.getLineNumber(),
									i.equals( current ) )								
						);
					}
				}

			} while ( i != null );			
		} catch (XPathException e) {
		}

		return res;
	}	

	public List<Variable> getVariables() {
		return cv.getVariables();
	}

	public List<Variable> getParameters() {
		return cv.getParameters();
	}

}
