package com.japisoft.framework.css;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Element;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SimpleRule implements Rule {

	private List<Property> properties = null;
	private Map<String,Property> map = null;

	public SimpleRule() {
		super();
	}

	public SimpleRule( Rule r ) {
		this();
		addProperties( r.getProperties() );
	}

	private void addProperties( List<Property> l ) {
		if ( l != null ) {
			for ( Property p : l ) {
				if ( getProperty( p.getName() ) == null )	// Don't override
					addProperty( p );
			}
		}		
	}

	private String[] borderDirections = 
		new String[] {
			"border-top", "border-right", "border-left", "border-bottom"
	};

	public void removeProperty(String name) {
		name = name.toLowerCase();
		if ( map != null ) {
			Property p = map.get( name );
			if ( p != null ) {
				properties.remove( p );
				map.remove( name );
			}
		}
	}
	
	public void addProperty( Property property ) {
		if ( properties == null ) {
			properties = new ArrayList<Property>();
		}
		properties.add( property );
		if ( map == null ) {
			map = new HashMap<String, Property>();
		}
		map.put( 
			property.getName().toLowerCase(), 
			property
		);
		for ( String borderDir : borderDirections ) {
			// Border part
			if ( property.getName().startsWith( borderDir + "-" ) ) {
				Property borderProperty = map.get( borderDir );
				if ( borderProperty == null ) {
					borderProperty = new SimpleProperty( borderDir, new CSSBorder() );
					addProperty( borderProperty );
				}
				( ( CSSBorder )borderProperty.getValue() ).merge( ( CSSBorder )property.getValue() );
			} else {
				// Full border
				if ( property.getName().equals( borderDir ) ) {
					Property borderProp = map.get( "border" );
					CSSBorder border = null;
					if ( borderProp == null ) {
						borderProp = new SimpleProperty( "border", border = new CSSBorder() );
						addProperty( borderProp );
					} else
						border = ( CSSBorder )borderProp.getValue();
					border.setSide( property.getName(), ( CSSBorder )property.getValue() );
				}
			}
		}
	}

	public List<Property> getProperties() {
		return properties;
	}
	
	/**
	 * r is ancestor, low priority
	 */
	public Rule merge( Rule r ) {
		SimpleRule sr = new SimpleRule( this );
		sr.addProperties( r.getProperties() );
		return sr;
	}

	public Property getProperty( String name ) {
		if ( map == null )
			return null;
		return map.get( name );
	}

	private Selector selector;
	
	public void setSelector( Selector selector ) {
		this.selector = selector;
	}

	public Selector getSelector() {
		return selector;
	}
	
	public boolean match(Element node) {
		if ( selector != null )
			return selector.match( node );
		return false;
	}

}
