package com.japisoft.framework.css;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SimpleSelector implements Selector {
	
	private int specificity = 1;
	private String mask = null;
	
	public SimpleSelector( String value ) {
		parse( value );
		this.mask = value;
	}

	private int l;

	List<Token> tokens = null;
	
	private void parse( String value ) {
		tokens = new ArrayList<Token>();
		l = 0;
		Token token = null;
		while ( ( token = nextToken( value ) ) != null ) {
			tokens.add( token );
		}		
		specificity = tokens.size();
		if ( tokens.size() == 1 ) {
			if ( "*".equals( tokens.get( 0 ).value ) )
				specificity = 0;
		}
	}

	public void dump() {
		for ( Token t : tokens ) {
			System.out.println( "-token [" + t.value + "]:" + t.type );
		}
	}

	private int lastTokenType = 0;

	private Token nextToken( String value ) {
		String tmp = "";
		int type = 0, i;
		
		if ( lastTokenType == 1 )
			type = 7;
		if ( lastTokenType == 3 )
			type = 8;		

		main : for ( i = l; i < value.length(); i++ ) {
			char c = value.charAt( i );
			if ( !Character.isSpaceChar( c ) ) {
				boolean tokenFound = false;
				
				if ( c == '[' || 
						c == ']' || 
							c == ':' || 
								c == '>' || 
									c == '+' ||
										c == '~' ) {
					if ( !"".equals( tmp ) ) {
						i--;
						break main;
					}
				}
				
				switch( c ) {
					case '[' :
						type = 1;
						tokenFound = true;
						break;
					case ']' :
						type = 2;
						tokenFound = true;
						break;
					case ':' :
						type = 3;
						tokenFound = true;
						break;
					case '>' :
						type = 4;
						tokenFound = true;
						break;
					case '+' :
						type = 5;
						tokenFound = true;
						break;
					case '~' :
						type = 6;
						tokenFound = true;
						break;
				}

				if ( tokenFound ) {
					tmp += c;
					break main;
				} else {
					tmp += c;
				}
			} else {
				if ( !"".equals( tmp ) ) {
					break main;
				}
			}
		}

		if ( "".equals( tmp ) ) {
			return null;
		}

		Token t = new Token();
		t.value = tmp.trim();
		t.type = type;
		lastTokenType = type;
		
		l = i + 1;
		
		return t;
	}

	private boolean equals( String cssName, Element node ) {
		String name = node.getLocalName();
		if ( name == null )
			name = node.getNodeName();
		if ( name.equalsIgnoreCase( 
			cssName ) ) {
			return true;
		} else {
			if ( node.getParentNode() instanceof Element ) {
				return equals( cssName, ( Element )node.getParentNode() );
			} else
				return false;
		}			
	}
	
	public boolean match( Element node ) {

		boolean match = true;

		for ( Token t : tokens ) {
			if ( t.type == 0 ) {
				if ( "*".equals( t.value ) )
					match = match && true;
				else
					match = match && equals( t.value, node ); 
			}
		}

		return match;
	}

	public int getSpecificity() {
		return specificity;
	}

	@Override
	public String toString() {
		return mask;
	} 
	
	public static void main( String[] args ) {
		SimpleSelector ss = new SimpleSelector( "" );
		ss.dump();
	}

}
