package com.japisoft.framework.dialog;

import javax.swing.*;

import com.japisoft.framework.ApplicationMain;
import com.japisoft.framework.dialog.actions.CancelAction;
import java.awt.*;
import java.awt.event.*;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicDialogComponent 
		extends JDialog implements 
			WindowListener, DialogComponent, IdDialog {


	
	public BasicDialogComponent( String dialogTitle, boolean modalMode ) {
		super();
		setTitle( dialogTitle );
		setModal( modalMode );
		init();
	}
	
	public BasicDialogComponent(
		java.awt.Dialog owner, String dialogTitle, boolean modalMode ) {
		super( owner, dialogTitle );
		setModal( modalMode );
		init();
	}
	
	public BasicDialogComponent(
		Frame owner,
		String dialogTitle, boolean modalMode ) {
		super( owner, dialogTitle );
		setModal( modalMode );
		init();
	}
	
	public BasicDialogComponent(
			Frame owner,
			String dialogTitle ) {
		this( owner, dialogTitle, true );
	}

	private void init() {
		// Register a cancel action for the Escape key
		getRootPane().registerKeyboardAction(new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				setLastAction( DialogManager.getDefaultClosingAction() );
				setVisible( false );
// Can introduce bug if the dialog is reused
//				dispose();
			}
		},
		KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false),
		JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		getContentPane().setLayout(
				new BorderLayout( 0, 0 ) );
	}

	public void windowActivated(WindowEvent e) {
	}
	public void windowClosed(WindowEvent e) {
	}
	public void windowClosing(WindowEvent e) {
		setLastAction( CancelAction.ID );
	}
	public void windowDeactivated(WindowEvent e) {
	}
	public void windowDeiconified(WindowEvent e) {
	}
	public void windowIconified(WindowEvent e) {
	}
	public void windowOpened(WindowEvent e) {
	}

	public void addNotify() {
		super.addNotify();
		addWindowListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		removeWindowListener( this );
	}

	private DialogFooter footer = null;
	
	Dimension iniDim = null;

	public void setInitialSize( int width, int height ) {
		iniDim = new Dimension( width, height );
	}

	public DialogComponent init( 
			DialogHeader header, 
			JComponent content, 
			DialogFooter footer ) {

		if ( header != null )
			getContentPane().add( header.getView(), BorderLayout.NORTH );
		if ( content != null ) {			
			getContentPane().add( content, BorderLayout.CENTER );
			if  (iniDim != null )
				content.setPreferredSize( iniDim );
		}

		if ( footer != null ) {
			this.footer = footer;
			getContentPane().add( footer.getView(), BorderLayout.SOUTH );
		}
		
		pack();
		
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation( 
				( dim.width - getWidth() ) / 2,
				( dim.height - getHeight() ) / 2 );

		return this;
	}
	
	public void setSize( int width, int height, boolean center ) {
		super.setSize( width, height );
		if ( center ) {
			Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
			setLocation( 
				( dim.width - getWidth() ) / 2,
				( dim.height - getHeight() ) / 2 
			);
		}
	}

	private int lastAction = CancelAction.ID;

	public void setLastAction( int lastActionId ) {
		this.lastAction = lastActionId;
		actionPerformed( lastActionId );
	}
	
	protected void actionPerformed( int actionId ) {}
	
	public int getLastAction() {
		return lastAction;
	}

	/** Invoke the following action from the footer part. A runtimeException
	 * may be thrown depending on the implementation for an unknown actionId
	 */
	public void invokeAction( int actionId ) {
		footer.invokeAction( actionId );
	}

	/** Called before the dialog is closed */
	protected void beforeClosing() {
		footer.dialogHidden();
	}	

	/** Called before the dialog is visible */
	protected void beforeShowing() {
		if ( footer != null )
			footer.dialogShown();
	}

	public void setVisible( boolean visible ) {
		if ( visible ) {
			beforeShowing();
		}// else
		 //	footer.dialogHidden();
		try {
			super.setVisible( visible );
		} catch( NullPointerException exc ) {
			System.err.println( exc.getMessage() );
		}
	}

	public void setVisible( boolean visible, int lastAction ) {
		this.lastAction = lastAction;
		setVisible( visible );
	}
		
	/**
	 * Enabled/Disabled an action 
	 * @param actionId a dialog action id
	 * @param enabled enabled or disable this action
	 */
	public void setEnabled( int actionId, boolean enabled ) {
		if ( footer != null )
			footer.setEnabled( actionId, enabled );
	}

	/** @return <code>true</code> if the last action was the 'OK' button */
	public boolean isOk() {
		return getLastAction() == DialogManager.OK_ID;
	}

	/** @return <code>true</code> if the last action was the 'CANCEL' button */
	public boolean isCancel() {
		return getLastAction() == DialogManager.CANCEL_ID;
	}

	/** Dispose all the inner resource */
	public void dispose() {
		if  (footer != null ) {
			beforeClosing();
			super.dispose();
			footer.dispose();
			footer = null;		
		}
	}

	public void dispose( int lastAction ) {
		this.lastAction = lastAction;
		setVisible( false );
		dispose();
	}
	
	public Component getView() {
		return this;
	}	

	public String getId() {
		return getTitle();
	}
	
	public static void main( String[] a ) {
		DialogManager.showDialog( null, "TEST", "TEST", "TEST", null, new JPanel() );
	}
	
}
