package com.japisoft.framework.dialog.actions;

import java.awt.Container;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;

import javax.swing.Action;
import javax.swing.ImageIcon;

import com.japisoft.framework.dialog.BasicDialogComponent;
import com.japisoft.framework.dialog.DialogComponent;
import com.japisoft.framework.dialog.DialogFooter;
import com.japisoft.framework.dialog.IdDialog;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class StoringLocationAction extends AbstractDialogAction implements DialogVisibilityObserver, ToggleActionTransformer {

	public static final int ID = 100;
	public static String preferenceCat = "dialog";
	
	public StoringLocationAction() {
		super( ID, false );
		putValue( Action.SMALL_ICON, new ImageIcon( getClass().getResource( "lock2.png" ) ) );
		putValue( Action.SHORT_DESCRIPTION, "When selecting, it fixes the last dialog location" );
		setSpecial( true );
	}

	public void dialogHidden( DialogComponent dialog, DialogFooter footer ) {
		if ( footer.isDialogActionSelected( ID ) ) {
			Preferences.setRawPreference(
				preferenceCat,
				getDialogName( dialog ),
				dialog.getView().getBounds() );
		}		
	}

	public void dialogShown( DialogComponent dialog, DialogFooter footer ) {
		Rectangle r = 
			Preferences.getPreference(
				preferenceCat,
				getDialogName( dialog ),
				(Rectangle)null );
		if ( r != null )
			dialog.getView().setBounds( r );		
	}

	private String getDialogName( DialogComponent dialog ) {
		if ( dialog instanceof IdDialog )
			return ( ( IdDialog )dialog ).getId();
		Class cl = dialog.getClass();
		String _ = cl.getName();
		int i = _.lastIndexOf( "." );
		if ( i > -1 )
			return _.substring( i + 1 );
		return _;
	}

	public void actionPerformed( ActionEvent e ) {
	}

}
