package com.japisoft.framework.dialog.basic;

import java.awt.Dimension;
import java.awt.Frame;
import java.io.IOException;
import java.net.URL;

import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import com.japisoft.framework.dialog.BasicOKDialogComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class HTMLDialog extends BasicOKDialogComponent 
		implements HyperlinkListener {

	private JEditorPane pane = null;

	/** Will load this HTML page from the classpath only */
	public HTMLDialog( Frame owner, String title, String pagePath ) { 
		this( owner, title, pagePath, true );
	}

	/** User can decide to use an URL for the path or searching from the classpath */
	public HTMLDialog( Frame owner, String title, String pagePath, boolean classPathAccess ) {
		super( owner, title, title, null, null );
		try {
			if ( classPathAccess )
				pane = new JEditorPane(
						ClassLoader.getSystemResource( pagePath ) );
			else
				pane = new JEditorPane(
						new URL( pagePath ) );
			pane.setEditable( false );
			JScrollPane sp = new JScrollPane( pane );
			sp.setPreferredSize( new Dimension( 670, 500 ) );
			setUI( sp  );
		} catch( IOException exc ) {
			exc.printStackTrace();
		}	
	}
	
	public void addNotify() {
		super.addNotify();
		pane.addHyperlinkListener( this );
	}
	
	public void removeNotify() {
		super.removeNotify();
		pane.removeHyperlinkListener( this );
	}

	public void hyperlinkUpdate( HyperlinkEvent e ) {
	    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
	    	pane.scrollToReference( e.getDescription().substring(1) );
	    }
	}

}
