package com.japisoft.framework.dialog.console;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;

import com.japisoft.framework.ApplicationModel;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ConsolePanel extends JPanel implements ActionListener {
	private JTextArea ta;
	private static JTextArea VISIBLE_TEXTE = null;
	
	private ConsolePanel() {
		prepareUI();
		setPreferredSize( new Dimension( 500, 200 ) );
	}
	
	private static ConsolePanel instance = null;
	
	public static ConsolePanel instance() {
		if ( instance == null )
			instance = new ConsolePanel();
		if ( instance.getParent() != null ) {
			Container parent = instance.getParent();
			parent.remove( instance );
			parent.invalidate();
			parent.revalidate();
			parent.repaint();
		}
		return instance;
	}
	
	public ConsolePanel setText( String msg ) {
		bo = null;
		System.out.print( msg );
		return this;
	}
	
	public void addNotify() {
		super.addNotify();
		VISIBLE_TEXTE = ta;
		if ( bo != null ) {
			ta.setText( bo.toString() );
			try {
				ta.setCaretPosition( ta.getDocument().getLength() );
			} catch (RuntimeException e) {
			}
		}	
		cbDebugMode.addActionListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		VISIBLE_TEXTE = null;
		cbDebugMode.removeActionListener( this );
	}

	public void actionPerformed(ActionEvent e) {
		System.getProperty( "application.debug", Boolean.toString( cbDebugMode.isSelected() ).toLowerCase() );
		ApplicationModel.DEBUG_MODE = cbDebugMode.isSelected();
	}
	
	private JCheckBox cbDebugMode = null;
	
	private void prepareUI() {
		setLayout( new BorderLayout() );
		ta = new JTextArea();
		ta.setBackground( Color.BLACK );
		ta.setForeground( Color.WHITE );
		ta.setFont( new Font( "courier", Font.PLAIN, 12 ) );
		ta.setEditable( false );
		add( new JScrollPane( ta ) );		
		JToolBar tb = new JToolBar();
		tb.add( new CleanAction() );
		tb.add( new CopyAction() );
		
		tb.add( cbDebugMode = new JCheckBox( "Debug mode" ) );
		
		add( tb, BorderLayout.SOUTH );
		setPreferredSize( new Dimension( 300, 400 ) );
	}

	/** Clean the console */
	class CleanAction extends AbstractAction {
		public CleanAction() {
			putValue( Action.NAME, "Clean" );
		}
		public void actionPerformed(ActionEvent e) {
			ta.setText( "" );
			bo = null;
		}
	}

	/** Copy the console content */
	class CopyAction extends AbstractAction {
		public CopyAction() {
			putValue( Action.NAME, "Copy" );
		}
		public void actionPerformed(ActionEvent e) {
			if ( ta.getSelectedText() == null )
				ta.selectAll();
			ta.copy();
		}
	}
	
	static StringBuffer bo = null;
	
	/** Size of the buffer, outside the buffer the data are lost */
	public static int CONSOLE_OUTPUT_MAX_BUFFER = 4096 * 2;

	private static PrintStream previousErrorState = null;
	private static PrintStream previousOutputState = null;

	/** Must be called for routing all the console message to the dialog content. Note that
	 * after calling this method you will not have output on the standard console */
	public static void initConsoleState() {
		ConsoleOutputStream o = new ConsoleOutputStream();
		PrintStream ps = new PrintStream( o );
		previousErrorState = System.err;
		previousOutputState = System.out;
		System.setErr( ps );
		System.setOut( ps );
	}

	/** Restore the default console state. Must be called when terminating using the console dialog */
	public static void restoreConsoleState() {
		if ( previousErrorState != null ) {
			System.setErr( previousErrorState );
			System.setOut( previousOutputState );
		}
	}

	static class ConsoleOutputStream extends OutputStream {
		
		@Override
		public void flush() throws IOException {
			if ( bo != null )
				if ( VISIBLE_TEXTE != null )
					VISIBLE_TEXTE.setText( bo.toString() );
		}
		
		public void write( int b ) throws IOException {
			if ( bo == null )
				bo = new StringBuffer();
			if( bo.length() > CONSOLE_OUTPUT_MAX_BUFFER ) {
				bo.deleteCharAt( 0 );
			}
			bo.append( ( char )b );
			
			if ( ApplicationModel.DEBUG_MODE ) {
				previousOutputState.write( b );
			}
			
		}
	}

	public static void main( String[] args ) {
		JFrame f = new JFrame();
		f.getContentPane().add(
				new ConsolePanel() );
		ConsolePanel.initConsoleState();
		for ( int i = 0; i < 100; i++ ) {
			System.out.println( ">sdsdfsdfsfdsfdsfdsdfsdfdf" + i );
			System.err.println( ">>>>>>>>>>>>>>>" + i );
		}
		f.setSize( new Dimension( 400, 400 ) );
		f.setVisible( true );
	}

}
