package com.japisoft.framework.dialog.help;

import javax.swing.*;

import com.japisoft.framework.ApplicationMain;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;
import java.io.*;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TipOfTheDayDialog extends JDialog implements ActionListener {
	


	JLabel lblTitleTip = new JLabel();
	JLabel lblImage = new JLabel();
	JCheckBox cbShow = new JCheckBox();
	JButton btnPreviousTip = new JButton();
	JButton btnNextTip = new JButton();
	JButton btnClose = new JButton();
	JEditorPane txtEditor = new JEditorPane();
	GridBagLayout gridBagLayout1 = new GridBagLayout();
	JScrollPane scEditor = new JScrollPane( txtEditor );

	/** This is a path from the classpath */
	public static String DEF_TIPS_PATH = "tips/tip";
	
	public TipOfTheDayDialog(boolean showNextTime) {
		super( ApplicationModel.MAIN_FRAME );
		try {
			init();
			setTitle( "Tip of the day" );
			initTips();
			if (tips.size() == 0)
				txtEditor.setText( "No TIP?" );
			else {
				int r = (int) (Math.random() * tips.size());
				showTip(r);
			}
			cbShow.setSelected(showNextTime);
			setModal( true );
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void addNotify() {
		super.addNotify();
		btnPreviousTip.addActionListener( this );
		btnNextTip.addActionListener( this );
		btnClose.addActionListener( this );
		cbShow.addActionListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		btnPreviousTip.removeActionListener( this );
		btnNextTip.removeActionListener( this );
		btnClose.removeActionListener( this );
		cbShow.removeActionListener( this );
	}

	public boolean isAvailableForTheNextTime() {
		return cbShow.isSelected();
	}
	
	int currentTip = 0;

	private void showTip(int r) {
		this.currentTip = r;
		URL url = ( URL ) tips.get( r );
		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(url
					.openStream()));
			String title = br.readLine();

			int i1 = title.indexOf("<!--");
			int i2 = title.lastIndexOf("-->");

			if (i1 > -1 && i2 > -1) {
				title = title.substring(i1 + 4, i2);
			}

			lblTitleTip.setText(title);

			String line;
			StringWriter sw = new StringWriter();
			while ((line = br.readLine()) != null) {
				sw.write(line);
				sw.write(System.getProperty("line.separator"));
			}

			txtEditor.setContentType("text/html");
			txtEditor.setText(sw.toString());

		} catch (IOException exc) {
		}
	}

	private ArrayList tips;

	private void initTips() {
		tips = new ArrayList();
		for (int i = 1; i < 30; i++) {
			URL url = ClassLoader.getSystemResource( DEF_TIPS_PATH + i + ".html" );
			if (url == null)
				break;
			tips.add( url );
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnClose) {
			setVisible(false);
			dispose();
		} else if (e.getSource() == btnPreviousTip) {
			currentTip--;
			if (currentTip < 0)
				currentTip = (tips.size() - 1);
			showTip(currentTip);
		} else if (e.getSource() == btnNextTip) {
			currentTip++;
			if (currentTip > (tips.size() - 1)) {
				currentTip = 0;
			}
			showTip(currentTip);
		} else if ( e.getSource() == cbShow ) {
			Preferences.setRawPreference( "interface", "tipOfTheDay", new Boolean( isAvailableForTheNextTime() ) );
		}
	} 

	private void init() throws Exception {
		this.getContentPane().setLayout(gridBagLayout1);
		lblTitleTip.setFont(new java.awt.Font("Dialog", 1, 16));
		
		lblImage.setIcon(
				new ImageIcon( 
						ClassLoader.getSystemResource( "images/help2.png" ) ) );
		
		cbShow.setText("Show \"Tip of the day\" after launching");
		btnPreviousTip.setText("Previous tip");
		btnNextTip.setText("Next tip");
		btnClose.setText("Close");
		txtEditor.setBackground( Color.WHITE );
		txtEditor.setText(null);

		this.getContentPane().add(
				scEditor,
				new GridBagConstraints(1, 1, 5, 1, 1.0, 1.0,
						GridBagConstraints.CENTER, GridBagConstraints.BOTH,
						new Insets(0, 11, 0, 5), 0, 0));

		this.getContentPane().add(
				lblTitleTip,
				new GridBagConstraints(1, 0, 5, 1, 0.0, 0.0,
						GridBagConstraints.WEST, GridBagConstraints.NONE,
						new Insets(5, 10, 0, 0), 0, 0));

		this.getContentPane().add(
				lblImage,
				new GridBagConstraints(0, 0, 1, 4, 0.0, 0.0,
						GridBagConstraints.WEST, GridBagConstraints.VERTICAL,
						new Insets(0, 0, 0, 0), 0, 0));
		this.getContentPane().add(
				cbShow,
				new GridBagConstraints(1, 2, 3, 1, 0.0, 0.0,
						GridBagConstraints.CENTER, GridBagConstraints.NONE,
						new Insets(0, 11, 0, 46), 7, 0));
		this.getContentPane().add(
				btnPreviousTip,
				new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0,
						GridBagConstraints.WEST, GridBagConstraints.NONE,
						new Insets(8, 29, 6, 0), 0, 0));
		this.getContentPane().add(
				btnNextTip,
				new GridBagConstraints(3, 3, 1, 1, 0.0, 0.0,
						GridBagConstraints.CENTER, GridBagConstraints.NONE,
						new Insets(8, 0, 6, 0), 0, 0));
		this.getContentPane().add(
				btnClose,
				new GridBagConstraints(4, 3, 2, 1, 0.0, 0.0,
						GridBagConstraints.CENTER, GridBagConstraints.NONE,
						new Insets(8, 13, 6, 5), 0, 0));
	}
	
}
