package com.japisoft.framework.dockable;

import java.awt.Color;
import java.awt.Rectangle;

import javax.swing.Icon;
import javax.swing.JComponent;

import com.japisoft.framework.dockable.action.ActionModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface Windowable {

	/** @return the id of the component */
	public String getId();

	/** @return the real UI component */	
	public JComponent getUserView();	

	/** @return the global component */
	public JComponent getView();
	
	/** @return the current title */
	public String getTitle();

	/** Reset for a new title */
	public void setTitle( String title );

	/** Fixed a window, so it cannot be swapped with another window */
	public void setFixed( boolean fixed );
	
	/** @return <code>true</code> if the window cannot be swapped with another window */
	public boolean isFixed();

	/** The window can be resized */ 
	public void setResizable( boolean resize );

	/** @return true if you can Resize the window ? */
	public boolean isResizable();
	
	/** @return the current icon */
	public Icon getIcon();
	
	/** Reset for a new icon */
	public void setIcon( Icon icon );

	/** @return the set of available actions */
	public	ActionModel getActionModel();	

	/** Update the backgrond for this window */
	public void setBackground( Color background );	

	/** Update the foreground for this window */
	public void setForeground( Color foreground );	

	/** Reset the focus */
	public void requestFocus();

	/** @return the container with the user UI panel */
	public JComponent getContentPane();	
	
	/** Repaint the view */
	public void repaint();
	
	/** If the inner window has been transformed inside a frame */
	public Rectangle getFrameBounds();

	/** Last location and size when the inner window has been transformed inside a frame */
	public void setFrameBounds( Rectangle r );

	/** Reset the final content */
	public void setContentPane( JComponent container );	

	/** Maximized state */
	public void setMaximized( boolean max );
	
	/** @return the maximized state */
	public boolean isMaximized();

	void fireDockEvent( String id, int type );	
	
}	
