package com.japisoft.framework.spreadsheet;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.table.TableModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class AbstractSpreadsheet implements Spreadsheet {

	private List<String> columns = null;
	
	public int getColumnCount() {
		if ( delegate != null )
			return delegate.getColumnCount();
		if ( columns == null )
			return 0;
		return columns.size();
	}

	protected void addColumn( String name ) {
		if ( columns == null )
			columns = new ArrayList<String>();
		columns.add( name );
	}
	
	public String getColumnName(int columnIndex) {
		if ( delegate != null )
			return delegate.getColumnName( columnIndex );
		return columns.get( columnIndex );
	}

	private TableModel delegate = null;
	
	public void reset( TableModel tm ) {
		this.delegate = tm;
	}
	
	protected void setColumns( String[] content ) {
		for ( int i = 0;i < content.length; i++ ) {
			addColumn( content[ i ] );
		}
	}
	
	private List<List<String>> rows = null;


	private boolean isEmptyLine( String[] content ) {
		for ( int i = 0; i < content.length; i++ ) {
			if ( !"".equals( content[ i ] ) ) {
				return false;
			}
		}
		return true;
	}	

	private boolean checkHeader = false;

	protected void readLine( String[] content ) {
		if ( !isEmptyLine( content ) ) {
			if ( !checkHeader ) {
				int location = -1;
				if ( location == -1 ) {
					// First line for the header
					setColumns( content );
				} else {
					// No header, overwrite it
					String[] tmp = new String[ content.length ];
					for ( int i = 0; i < content.length; i++ ) {
						tmp[ i ] = "Data " + ( i + 1 );
					}
					setColumns( tmp );
					addRow( content );
				}
				checkHeader = true;
			} else {
				addRow( content );
			}
		}
	}
	
	public int getRowCount() {
		if ( delegate != null )
			return delegate.getRowCount();
		if ( rows == null )
			return 0;
		return rows.size();
	}

	public String getValueAt(int rowIndex, int columnIndex) {
		if ( delegate != null ) {
			Object obj = delegate.getValueAt( rowIndex, columnIndex );
			if ( obj == null )
				return null;
			return obj.toString();
		}
		List<String> m = rows.get( rowIndex );
		if ( m == null || m.size() <= columnIndex ) {
			return null;
		}
		return m.get( columnIndex );
	}

	public void setValueAt( int rowIndex, int columnIndex, String value ) {
		List<String> m = rows.get( rowIndex );
		if ( m == null ) {
			rows.set( rowIndex, m = new ArrayList<String>() );
		}
		m.set( columnIndex, value );
	}
	
	private void addRow( String[] content ) {
		if ( rows == null )
			rows =new ArrayList<List<String>>();
		rows.add( Arrays.asList( content ) );
	}

}
