package com.japisoft.framework.spreadsheet;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CsvSpreadsheet extends AbstractSpreadsheet {

	public void read(InputStream input) throws Exception {
		InputStreamReader r = new InputStreamReader( input, Preferences.getPreference( "CSV", "DefaultEncoding", "UTF-8" ) );
		BufferedReader reader = new BufferedReader( r );
		// Search for the first column with the @ value
		String line = null;
		while ( ( line = reader.readLine() ) != null ) {
			if ( !"".equals( line ) )
				readLine( line );
		}
	}

	public void write(OutputStream output) throws Exception {
		OutputStreamWriter w = new OutputStreamWriter( output, Preferences.getPreference( "CSV", "DefaultEncoding", "UTF-8" ) );
		BufferedWriter bw = new BufferedWriter( w );
		try {
			for ( int i = 0; i < getColumnCount(); i++ ) {
				if ( i > 0 )
					bw.write( "," );				
				bw.write( getColumnName( i ) );
			}
			bw.newLine();
			for ( int j = 0; j < getRowCount() - 1; j++ ) {
				if ( j > 0 )
					bw.newLine();				
				for ( int i = 0; i < getColumnCount(); i++ ) {
					bw.write( getValueAt( j, i ) );
					if ( i != getColumnCount() - 1 )
						bw.write( "," );
				}
			}
		} finally {
			bw.close();
		}
	};

	private String separator = ",|;";

	private void readLine( String line ) {
		String[] content = splitLine( line );
		readLine( content );
	}

	private String[] splitLine( String line ) {
		String[] datas = line.split( separator );
		for ( int i = 0; i < datas.length; i++ ) {
			if ( datas[ i ].startsWith( "\"" ) )
				datas[ i ] = datas[ i ].substring( 1 );
			if ( datas[ i ].endsWith( "\"" ) )
				datas[ i ] = datas[ i ].substring( 0, datas[ i ].length() - 1 );
		}
		return datas;
	}

}
