package com.japisoft.framework.step;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationStepAdapter;
import com.japisoft.framework.ApplicationStepException;
import com.japisoft.framework.application.descriptor.InterfaceBuilder;
import com.japisoft.framework.application.descriptor.InterfaceBuilderException;
import com.japisoft.framework.application.descriptor.helpers.ActionBuilder;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class InterfaceBuilderStep extends ApplicationStepAdapter {
	private String descriptorPath = null;
	private URL descriptorURL = null;
	
	private ActionBuilder actionBuilder = null;

	public InterfaceBuilderStep() {
		/* empty */
	}

	public InterfaceBuilderStep( URL descriptorURL ) {
		this.descriptorURL = descriptorURL;
	}
	
	public InterfaceBuilderStep(String descriptorPath) {
		this.descriptorPath = descriptorPath;
	}

	public InterfaceBuilderStep(ActionBuilder actionBuilder) {
		this.actionBuilder = actionBuilder;
	}

	public InterfaceBuilderStep(
			String descriptorPath,
			ActionBuilder actionBuilder) {
		this(descriptorPath);
		this.actionBuilder = actionBuilder;
	}

	public void start(String[] args) {
		String path = descriptorPath;
		if (path == null)
			path = ApplicationModel.USERINTERFACE_FILE;

		
		try {
			URL url = descriptorURL;
			if ( url == null ) {
				url = ClassLoader.getSystemResource(path);	
			}

			if ( url == null ) {
				File f = new File(path);
				if (f.exists())
					url = f.toURI().toURL();
				else
					throw new ApplicationStepException(
							"Can't find " + path, true );
			}
			if ( url == null )
				throw new ApplicationStepException(
						"Can't find " + path, true );
			InterfaceBuilder builder = new InterfaceBuilder(
					url, 
					actionBuilder,
					null);
			ApplicationModel.INTERFACE_BUILDER = builder;
		} catch (MalformedURLException e) {
			throw new ApplicationStepException(e, true);
		} catch (InterfaceBuilderException e) {
			throw new ApplicationStepException(e, true);
		}
	}
}
