package com.japisoft.framework.step;

import java.util.ArrayList;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationStepAdapter;
import com.japisoft.framework.log.Logger;
import com.japisoft.framework.preferences.Preferences;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class LookAndFeelStep extends ApplicationStepAdapter {
	
	private String defaultLookAndFeel;

	/** Use the provided lookAndFeel */
	public LookAndFeelStep( String defaultLookAndFeelClassName ) {
		this.defaultLookAndFeel = defaultLookAndFeelClassName;
	}

	/** Use the system lookAndFeel by default */
	public LookAndFeelStep() {
		this.defaultLookAndFeel = 
				UIManager.getSystemLookAndFeelClassName();
	}

	public void start(String[] args) {
		// Build the default list
		UIManager.LookAndFeelInfo[] uf = UIManager
				.getInstalledLookAndFeels();
		ArrayList l = new ArrayList();
		l.add("DEFAULT");
		for (int i = 0; uf != null && i < uf.length; i++) {
			l.add(uf[i].getClassName());
		}
		String[] _ = new String[l.size()];
		for (int i = 0; i < l.size(); i++)
			_[i] = (String) l.get(i);

		String className = Preferences.getPreference( "interface",
				"lookAndFeel", _)[0];

		LookAndFeel look = null;
		if ( "DEFAULT".equals( className ) ) {
			if ( ApplicationModel.MACOSX_MODE ) {
				if ( defaultLookAndFeel != null )
					l.add( defaultLookAndFeel );
				look = UIManager.getLookAndFeel();
			} else
				className = defaultLookAndFeel;
		}
		try {
			if (look == null)
				look = (LookAndFeel) Class.forName(className)
						.newInstance();
			UIManager.setLookAndFeel(look);			
		} catch (Throwable th) {
			Logger.addWarning("Can't use this lookAndFeel "
					+ className);
		}
	}
	
}
