package com.japisoft.framework.step;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.japisoft.framework.ApplicationMain;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationStep;
import com.japisoft.framework.ApplicationStepListener;
import com.japisoft.framework.dialog.basic.Splashscreen;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SplashScreenApplicationStep implements ApplicationStep, ApplicationStepListener {
	private Icon image;

	private int registeredDay;

	private int delay = 0;

	public SplashScreenApplicationStep() {
		this(0);
	}

	/**
	 * Use the ApplicationModel.APP_IMG_PATH key, the delay is for delaying the
	 * starting in ms
	 * 
	 * @param delay
	 */
	public SplashScreenApplicationStep(int delay) {
		this(ApplicationModel.APP_IMG_PATH);
		this.delay = delay;
	}
	
	public SplashScreenApplicationStep(String imagePath) {
		this(new ImageIcon(ClassLoader.getSystemResource(imagePath)), -1);
	}

	public SplashScreenApplicationStep(String imagePath, int registeredDay) {
		this(new ImageIcon(ClassLoader.getSystemResource(imagePath)),
				registeredDay);
	}

	public SplashScreenApplicationStep(Icon image, int registeredDay) {
		this.image = image;
		this.registeredDay = registeredDay;
	}

	public void run(ApplicationStep step, int indice, int maxIndice) {
		Splashscreen.progress( indice, maxIndice );
	}	
	
	public boolean isFinal() {
		return false;
	}

	public void start(String[] args) {
		ApplicationMain.addApplicationStepListener( this );
		Splashscreen.start(image, registeredDay);

		if (delay != 0)
			try {
				Thread.sleep(delay);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

	}

	public void stop() {
		Splashscreen.stop(false);
		ApplicationMain.removeApplicationStepListener( this );
	}
	
	@Override
	public void quit() {
	}
	
}
