package com.japisoft.framework.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.ui.toolkit.BrowserCaller;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class InitPanel extends JPanel implements MouseListener, ActionListener {
	
	static final String PREFERENCE_GROUP = "interface";
	static final String PREFERENCE_NAME = "initialDocument";
	
	/** Depending the preference value */
	public static boolean canBeShown() {
		return Preferences.getPreference(
				PREFERENCE_GROUP, 
				PREFERENCE_NAME,
				true );
	}

	public InitPanel() {
		this( null );
	}

	public InitPanel( String message ) {
		initComponents( message );
	}

	public void mouseClicked(MouseEvent e) {
		if ( e.getSource() == openLbl ) {
			ActionModel.activeActionById( "new", null );
		} else
		if ( e.getSource() == helpLbl ) {
			ActionModel.activeActionById( "Manual", null );
		} else
		if ( e.getSource() == urlLbl ) {
			BrowserCaller.displayURL( ApplicationModel.COMPANY_URL );
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	public void actionPerformed(ActionEvent e) {
		Preferences.setPreference(
				PREFERENCE_GROUP, 
				PREFERENCE_NAME, 
				openNextCb.isSelected() );
	}	

    private void initComponents( String message ) {
        logoLbl = new javax.swing.JLabel();
        
        
        if ( ApplicationModel.APP_IMG_PATH != null ) {        
	        logoLbl.setIcon( 
	        		new ImageIcon( 
	        				ClassLoader.getSystemResource( 
	        						ApplicationModel.APP_IMG_PATH ) ) );
        }
        logoLbl.setHorizontalAlignment( JLabel.CENTER );
        
        openLbl = new javax.swing.JLabel();
        helpLbl = new javax.swing.JLabel();
        openNextCb = new javax.swing.JCheckBox();
        jLabel1 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        versionLbl = new javax.swing.JLabel();
        urlLbl = new javax.swing.JLabel();

        logoLbl.setBackground(new java.awt.Color(255, 255, 255));
        logoLbl.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        logoLbl.setOpaque(true);

        
        
        if ( message != null ) {
        	openLbl.setText( message );
        } else {
        
        	openLbl.setForeground(new java.awt.Color(51, 51, 255));
        	openLbl.setText("Create a new document");

        }
       	
        helpLbl.setForeground(new java.awt.Color(51, 51, 255));
       	helpLbl.setText("User Manual");
        	
        openNextCb.setSelected(true);
       	openNextCb.setText("Open next time");
        openNextCb.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        openNextCb.setMargin(new java.awt.Insets(0, 0, 0, 0));

       	jLabel1.setText("Version :");
        versionLbl.setFont(new java.awt.Font("Tahoma", 1, 11));
       	versionLbl.setText( ApplicationModel.getAppVersion() );

        urlLbl.setFont(new java.awt.Font("Tahoma", 0, 12));
        urlLbl.setForeground(new java.awt.Color(0, 102, 51));
        urlLbl.setText( ApplicationModel.COMPANY_URL );
        
        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(logoLbl, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 474, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(openLbl)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 275, Short.MAX_VALUE)
                        .add(openNextCb))
                    .add(helpLbl)
                    .add(layout.createSequentialGroup()
                        .add(jLabel1)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(versionLbl)
                        .add(47, 47, 47)
                        .add(urlLbl))
                    .add(jSeparator1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 474, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(logoLbl, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 170, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(openLbl)
                    .add(openNextCb))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(helpLbl)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(versionLbl)
                    .add(urlLbl))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>                        

    public void addNotify() {
    	super.addNotify();
    	openLbl.addMouseListener( this );
    	helpLbl.addMouseListener( this );
    	openNextCb.addActionListener( this );
    	urlLbl.addMouseListener( this );
    }

    public void removeNotify() {
    	super.removeNotify();
    	openLbl.removeMouseListener( this );
    	helpLbl.removeMouseListener( this );
    	urlLbl.removeMouseListener( this );
    	openNextCb.removeActionListener( this );
    }

    // Variables declaration - do not modify                     
    private javax.swing.JLabel helpLbl;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel urlLbl;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JLabel logoLbl;
    private javax.swing.JLabel openLbl;
    private javax.swing.JCheckBox openNextCb;
    private javax.swing.JLabel versionLbl;
    // End of variables declaration                   

}
