package com.japisoft.framework.ui.browser;

import javax.swing.JFrame;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BrowserFactory {

	private boolean jfxEnabled = false;
	
	private BrowserFactory() {
		// Webkit browser disabled by default
		if ( Preferences.getPreference( "interface", "webkitBrowser", false ) ) {
			try {
				Class.forName( "javafx.scene.web.WebEngine" );
				jfxEnabled = true;
			} catch( Exception exc ) {			
			}
		}
	}

	private static BrowserFactory INSTANCE = null;
	
	public static BrowserFactory getInstance() {
		if ( INSTANCE == null )
			INSTANCE = new BrowserFactory();
		return INSTANCE;
	}
	
	public Browser newBrowser() {
		/*
		ApplicationModel.debug( "Use JFX Browser :" + jfxEnabled );
		if ( !jfxEnabled ) {
			return new SwingBrowser();
		}
		else {
			try {
				return new JFXBrowser();
			} catch( Throwable th ) {
				jfxEnabled = false;
				return newBrowser();
			}
		}
		*/
		return new SwingBrowser();
	}

	public static void main( String[] args ) {

		JFrame f = new JFrame();
		Browser b = BrowserFactory.getInstance().newBrowser();
		b.setHTML( "<html><body><b style='box-shadow:1px 1px 10px red'>Hello</b> world!</b></body></html>", null );
		f.add( b.getView() );
		f.setSize( 300, 300 );
		f.setVisible( true );		

	}

}
