package com.japisoft.framework.ui.table;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FeatureTable extends JTable {

	private String featureType = null;
	private List<String> features = null;
	private Map<String,Boolean> enabled = null;
	
	/**
	 * @param features list of features
	 * @param featureType feature type name displayable
	 */
	public FeatureTable( List<String> features, String featureType ) {
		this.features = features;
		this.featureType = featureType;
		enabled = new HashMap<String, Boolean>();
		setModel( new FeatureTableModel() );
		getColumnModel().getColumn( 0 ).setMaxWidth( 30 );
	}

	public List<String> getSelectedFeatures() {
		List<String> r = new ArrayList<String>();
		TableModel tm = getModel();
		for ( int i = 0; i < tm.getRowCount(); i++ ) {
			if ( tm.getValueAt( i, 0 ) == Boolean.TRUE ) {
				r.add( ( String )tm.getValueAt( i, 1 ) );
			}
		}
		return r;
	}

	// ------------------------------------------------------------------

	class FeatureTableModel implements TableModel {
		
		
		public void addTableModelListener(TableModelListener l) {
		}

		public Class<?> getColumnClass(int columnIndex) {
			if ( columnIndex == 0 )
				return Boolean.class;
			return String.class;
		}

		public int getColumnCount() {
			return 2;
		}

		public String getColumnName(int columnIndex) {
			if ( columnIndex == 0 )
				return "Ok";
			return featureType;
		}

		public int getRowCount() {
			return features.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			if ( columnIndex == 0 ) {
				String featureName = features.get( rowIndex );
				return enabled.get( featureName );
			} else {
				return features.get( rowIndex );
			}
		}

		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return columnIndex == 0;
		}

		public void removeTableModelListener(TableModelListener l) {
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			String featureName = features.get( rowIndex );
			enabled.put( featureName, ( Boolean )aValue );
		}

	}
	
	public static void main( String[] args ) {
		List<String> l = new ArrayList<String>();
		l.add( "Rouge" );
		l.add( "Vert" );
		l.add( "Bleu" );
		JFrame f = new JFrame();
		f.add( new FeatureTable( l, "Couleurs" ) );
		f.setSize( 300, 200 );
		f.setVisible( true );
	}

}
