package com.japisoft.framework.wizard;

import javax.swing.Icon;
import javax.swing.JComponent;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicWizardStep implements WizardStep {

	private String name;
	private String shortTitle;
	private String longTitle;
	private Icon smallIcon;
	private Icon largeIcon;
	private StepView view;

	private BasicWizardStep() {}

	/** @param name Name of this wizard */
	public BasicWizardStep( String name ) {
		this.name = name;
	}

	/** 
	 * @param name Name of this wizard
	 * @param shortTitle Label for the left part, it can't be <code>null</code>
	 * @param longTitle Label for the top part
	 * @param view Main wizard component */
	public BasicWizardStep( 
			String name, 
			String shortTitle, 
			String longTitle, 
			StepView view ) {
		this( name );
		this.view = view;
		this.shortTitle = shortTitle;
		this.longTitle = longTitle;
	}

	/** 
	 * @param name Name of this wizard
	 * @param shortTitle Label for the left part, it can't be <code>null</code>
	 * @param longTitle Label for the top part
	 * @param minorIcon Step icon
	 * @param majorIcon main step view icon
	 * @param view Main wizard component */
	public BasicWizardStep( 
			String name, 
			String shortTitle, 
			String longTitle, 
			Icon smallIcon,
			Icon largeIcon,
			StepView view ) {
		this( name );
		this.view = view;
		this.smallIcon = smallIcon;
		this.largeIcon = largeIcon;
		this.shortTitle = shortTitle;
		this.longTitle = longTitle;
	}

	public String getName() {
		return name;
	}

	private boolean enabled = true;

	public boolean isEnabled() {
		return enabled;
	}

	/** Enabled or disable this wizard step. A non enabled wizard will be skipped */
	public void setEnabled( boolean enabled ) {
		this.enabled = enabled;
	}

	public void setShortTitle( String title ) {
		this.shortTitle = title;
	}

	public String getShortTitle() {
		return shortTitle;
	}

	public String getLongTitle() {
		return longTitle;
	}

	public void setLongTitle( String title ) {
		this.longTitle = title;
	}

	public StepView getStepView() {
		return view;
	}

	public void setView( StepView component ) {
		this.view = component;
	}

	public Icon getSmallIcon() {
		return smallIcon;
	}

	public void setSmallIcon( Icon icon ) {
		this.smallIcon = icon;
	}

	public Icon getLargeIcon() {
		return largeIcon;
	}

	public void setLargeIcon( Icon icon ) {
		this.largeIcon = icon;
	}

	protected WizardStepContext currentContext;
	
	private boolean acceptNext = true;

	/** Decide to accept the next action automatically. By default <code>true</code> */
	public void setAcceptNextActionByDefault( boolean acceptNext ) {
		this.acceptNext = acceptNext;
	}

	/** @return the default value for accepting a next step. By default <code>true</code> meaning the Next button will by activated */
	protected boolean mustAcceptNextByDefault() {
		return acceptNext;
	}

	private boolean acceptPrevious = true;

	/** Decide to accept the previous action automatically. By default <code>true</code> */
	public void setAcceptPreviousActionByDefault( boolean acceptPrevious ) {
		this.acceptPrevious = acceptPrevious;
	}

	/** @return the default value for accepting a next step. By default <code>true</code> meaning the Next button will by activated */
	protected boolean mustAcceptPreviousByDefault() {
		return acceptPrevious;
	}

	/** @return a default <code>true</code> value */
	public boolean canStart(WizardStepContext context) {
		return true;
	}

	public boolean start( WizardStepContext context ) {
		currentContext = context;
		context.acceptNext( mustAcceptNextByDefault() );
		context.acceptPrevious( mustAcceptPreviousByDefault() );
		return isEnabled();
	}

	public void stop( WizardStepContext context ) {
		currentContext = context;
	}

}
