package com.japisoft.framework.wizard;

import javax.swing.Icon;
import javax.swing.JComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface WizardStep {

	/** @return a name for this wizard */
	public String getName();
	
	/** @return a short title for this wizard. This title will appear on the left part */
	public String getShortTitle();
	
	/** @return a long title for this wizard. This title will appear on the center part */
	public String getLongTitle();

	/** This is the user interface part for the wizard step */
	public StepView getStepView();

	/** @return a small icon for the left part */
	public Icon getSmallIcon();

	/** @return a big icon for the center part */
	public Icon getLargeIcon();

	/** @return <code>true</code> if the wizard step is accessible */
	public boolean isEnabled();

	/**
	 * Called before the wizard step is started. If the step return <code>false</code> then the
	 * step will be canceled and the wizard will stay to the current one
	 * @param context The wizard step context
	 * @return <code>true</code> to start really the step or <code>false</code> to stay on the current one
	 */
	public boolean canStart( WizardStepContext context );

	/** Called when the wizard step is started, so the final user can interact with the view provided by the <code>getView</code> method 
	 * @param context Information about the wizard step
	 * @return <code>true</code> when it accept to be started else the current wizard step will be skipped
	 * */
	public boolean start( WizardStepContext context );

	/** Called before a new wizard step is started
	 * @param context Information about the wizard step
	 */
	public void stop( WizardStepContext context );

}
