package com.japisoft.framework.xml;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public final class XSLTTransformer {

	public static String DEF_TRANS_NAME = "XALAN";

	//public static String DEF_TRANS_CLASS = "org.apache.xalan.processor.TransformerFactoryImpl";
	// Use SAXON 1 due to wrong error messages
	public static String DEF_TRANS_CLASS = "com.icl.saxon.TransformerFactoryImpl";
	
	public static String DEF_LIBRARY_PATH = "xslt";

	private static Class DEF_TRANS = null;

	private static boolean ERROR_FOUND = false;

	public static final String CONFIG_FILE = "jaxp_transformer.dat";

	public static void resetDefaultTransformer() {

		// The default parser is xerces
		System.setProperty("javax.xml.parsers.SAXParserFactory",
				"org.apache.xerces.jaxp.SAXParserFactoryImpl");

		ApplicationModel.debug( "Reset the default transformer" );
				
		if (DEF_TRANS == null && !ERROR_FOUND) {

			if (!loadByConfigFile()) {

				String transformer = Preferences.getPreference("xmlconfig",
						"transformer", DEF_TRANS_NAME);

				if (!DEF_TRANS_NAME.equals(transformer)) {

					// Search for a library inside the ext/xslt directory

					ArrayList al = XMLToolkit.browseJars(DEF_LIBRARY_PATH);

					if (al.size() == 0) {

						try {
							DEF_TRANS = Class.forName(transformer);
						} catch (Throwable th) {
							System.err.println("No library found inside the "
									+ DEF_LIBRARY_PATH + " directory");
							System.err.println("Can't use the " + transformer
									+ " class");
							ERROR_FOUND = true;
						}
					} else {

						URL[] libs = new URL[al.size()];
						for (int i = 0; i < al.size(); i++)
							libs[i] = (URL) al.get(i);

						try {

							URLClassLoader loader = new URLClassLoader(libs);
							DEF_TRANS = loader.loadClass(transformer);
							
						} catch (Throwable th) {
							ERROR_FOUND = true;
							System.err.println("Can't use " + transformer
									+ " : " + th.getMessage());
						}

					}

				}

			}

		}

	}

	static boolean loadByConfigFile() {
		try {
			ApplicationModel.debug( "Load " + CONFIG_FILE );
			Class cl = XMLToolkit.loadByConfigFile(new File(ApplicationModel
					.getAppUserPath(), CONFIG_FILE));
			if (cl != null) {
				ApplicationModel.debug( "Class found " + cl );
				DEF_TRANS = cl;
				return true;
			} else
				ApplicationModel.debug( "No class found" );
		} catch (Exception exc) {
		}
		return false;
	}

	public static TransformerFactory getTransformerFactory() {

		resetDefaultTransformer();

		TransformerFactory factory = null;

		if (DEF_TRANS != null) {
			try {
				factory = (TransformerFactory) DEF_TRANS.newInstance();
			} catch (Throwable th) {
				th.printStackTrace();
				System.err.println("Can't use " + DEF_TRANS + " : "
						+ th.getMessage());
			}
		}

		if (DEF_TRANS == null) {
			try {
				DEF_TRANS = Class.forName(DEF_TRANS_CLASS);
				factory = (TransformerFactory) DEF_TRANS.newInstance();
			} catch (Throwable th) {
				th.printStackTrace();
				throw new RuntimeException("ERROR CAN'T FIND "
						+ DEF_TRANS_CLASS + "???");
			}
		}

		ApplicationModel.debug( "Use the factory " + factory );
		
		return factory;
	}

	private static TransformerFactory FACTORY = null;

	public static Transformer getTransformer() throws Exception {

		if (FACTORY == null)
			FACTORY = getTransformerFactory();

		Transformer transformer = FACTORY.newTransformer();

		ApplicationModel.debug( "Use the transformer " + transformer );
		
		return transformer;
	}

}
