package com.japisoft.framework.xml.grammar.xsd;

import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.japisoft.framework.xml.grammar.Grammar;
import com.japisoft.framework.xml.grammar.GrammarNode;
import com.japisoft.framework.xml.grammar.GrammarType;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class XSDGrammarNode implements GrammarNode {

	protected Node node = null;
	protected int minOccurs = 1;
	protected int maxOccurs = 1;
	protected XSDGrammar grammar = null;
	
	public XSDGrammarNode( XSDGrammar grammar, Node node ) {
		this.grammar = grammar;
		this.node = node;
		if ( node instanceof Element ) {
			Element e = ( Element )node;
			if ( e.hasAttribute( "minOccurs" ) ) {
				try {
					minOccurs = Integer.parseInt( e.getAttribute( "minOccurs" ) );
				} catch (NumberFormatException e1) {
				}
			}
			if ( e.hasAttribute( "maxOccurs" ) ) {
				if ( "unbounded".equalsIgnoreCase( 
						e.getAttribute( "maxOccurs" ) ) ) {
					maxOccurs = Integer.MAX_VALUE;
				} else {
					try {
						maxOccurs = Integer.parseInt( e.getAttribute( "maxOccurs" ) );
					} catch (NumberFormatException e1) {
					}					
				}
			}
		}
	}

	public Grammar getGrammar() {
		return grammar;
	}	
	
	public String getName() {
		return node.getLocalName();
	}

	public String getNamespace() {
		return null;
	}

	protected GrammarType type = null;
	
	public GrammarType getType() {
		if ( type == null )
			type = 
				new XSDGrammarType(
					grammar,
					( Element )node 
				);
		return type;
	}
	
	public int getMaxOccurs() {
		return maxOccurs;
	}

	public int getMinOccurs() {
		return minOccurs;
	}

	public Node getDOMSource() { 
		return node; 
	}

	public Element getDOMElement() { 
		return ( Element )node; 
	}
	
	public void dump() {
		dump( 0 );
	}
	
	public void dump( int indentation ) {
		for ( int i = 0; i < indentation; i++ )
			System.out.print( " " );
		
		System.out.println( "node " + getName() + " / " + getClass() );
		GrammarType gt = getType();
		
		List<GrammarNode> values = gt.getValues();
		if ( values != null ) {
			for ( GrammarNode gn : values ) {
				if ( gn instanceof XSDGrammarNode ) {
					( ( XSDGrammarNode )gn ).dump( indentation + 1 );
				}
			}
		}
	}

}
