package com.japisoft.framework.xml.parser.document;

import java.util.Stack;

import com.japisoft.framework.collection.FastVector;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NamespaceContext extends FastVector {
	private Stack defaultNamespaceSt;

	/**
	 * @return the default namespace from the attribute xmlns */
	public String currentDefaultNamespace() {
		if ( defaultNamespaceSt == null )
			return null;
		if ( defaultNamespaceSt.isEmpty() )
			return null;
		return (String)defaultNamespaceSt.peek();
	}

	/**
	 * Reset the default namespace
	 * @param defaultNamespace  */
	public void pushDefaultNamespace(String defaultNamespace) {
		if ( defaultNamespaceSt == null )
			defaultNamespaceSt = new Stack();
		defaultNamespaceSt.push( defaultNamespace );
	}

	/** Remove the last default namespace */
	public void popDefaultNamespace() {
		if ( defaultNamespaceSt != null )
			defaultNamespaceSt.pop();
	}

	/** Add a current prefix for this uri */
	public void addPrefixScope( String prefix, String uri ) {
		add( new PrefixURI( prefix, uri ) );
	}

	/** Remove a prefix that should not be accessible */
	public void removePrefixScope( String prefix ) {
		remove( prefix );
	}

	/** @return true if this prefix is available */
	public boolean isPrefixAvaiable( String prefix ) {
		return contains( prefix );
	}
	
	/** @return the URI for this prefix */
	public String getPrefixURI( String prefix ) {
		for ( int i = 0; i < size(); i++ ) {
			Object o;
			if ( ( o = get( i ) ).equals( prefix ) ) {
				return ((PrefixURI)o).uri;
			}
		}	
		return null;
	}

	// ----------------------------------------------------

	class PrefixURI {
		public String prefix;
		public String uri;

		PrefixURI( String prefix, String uri ) {
			this.prefix = prefix;
			this.uri = uri;
		}

		public boolean equals( Object o ) {
			if ( o instanceof String ) {
				return prefix.equals( o );
			} else
					return super.equals( o );
		}
	}

}
