package com.japisoft.framework.xml.parser.walker;

import com.japisoft.framework.collection.FastVector;
import com.japisoft.framework.xml.parser.node.*;

import java.util.Enumeration;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TreeWalker {
	private FPNode node;

	/** Reference node. It is illegal to use a <code>null</code> node */
	public TreeWalker(FPNode node) {
		super();
		this.node = node;
		if ( node == null )
			throw new RuntimeException("Illegal null node");
	}

	/** @param tag to find
	    @param deep deeply found tag
	@return a list of tag mathing the name */
	public Enumeration getTagNodeByName(String name, boolean deep) {
		return getCriteriaResult(new NodeNameCriteria(name), deep, false);
	}

	/** Search for the first node */
	public FPNode getFirstTagNodeByName( String name, boolean deep ) {
		Enumeration enume = getTagNodeByName( name, deep );
		if ( enume.hasMoreElements() )
			return ( FPNode )enume.nextElement();
		return null;
	}

	/** @return all text node containing the subcontent */
	public Enumeration getTextNode(String subcontent, boolean deep) {
		return getCriteriaResult(new TextCriteria(subcontent), deep, false);
	}

	/** 
	 * Sample of criteria :
	 * <code>new OrCriteria( new NodeNameCriteria( "aa" ), new NodeNameCriteria( "bb" ) )</code> for
	 * returning 'aa' or 'bb' node.
	 *  @param vc Criteria for searching node 
	 *  @param deep deeply found tag
	 *  @return SimpleNode enumeration */
	public Enumeration getNodeByCriteria(ValidCriteria vc, boolean deep) {
		return getCriteriaResult(vc, deep, false);
	}

	public FPNode getOneNodeByCriteria(ValidCriteria vc, boolean deep) {
		Enumeration enume = getNodeByCriteria( vc, deep );
		if ( enume.hasMoreElements() )
			return ( FPNode )enume.nextElement();
		return null;
	}

	public Enumeration getNodeByCriteria(ValidCriteria vc, boolean deep, boolean stopAtFirst ) {
		return getCriteriaResult( vc, deep, stopAtFirst );
	}

	private Enumeration getCriteriaResult(ValidCriteria vc, boolean deep, boolean stopAtFirst ) {
		FastVector v = new FastVector();
//		if ( vc.isValid(  node ) )
//			v.add( node );	
		matchTag(vc, v, node, deep,false);
		return v.elements();
	}

	// Browser the XML tree
	private void matchTag(
		ValidCriteria vc,
		FastVector v,
		FPNode node,
		boolean deep,
		boolean stopAtFirst ) {
		for (int i = 0; i < node.childCount(); i++) {
			FPNode n = node.childAt(i);
			if (vc.isValid(n)) {
				v.add(n);
				if ( stopAtFirst )
					return;
			}
			if (deep && !n.isLeaf() ) {
				matchTag(vc, v, n, deep,stopAtFirst);
			}
		}
		
	}

}

// TreeWalker ends here
