package com.japisoft.framework.xml.refactor.elements;

import org.w3c.dom.EntityReference;
import org.w3c.dom.Node;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EntityRefactor extends AbstractRefactor {
	private static final String RENAME_ACTION = "(V1) RENAME TO (V2)";
	private static final String DELETE_ACTION = "DELETE (V1)";
	private static final String CONVERT_VALUE_ACTION = "CONVERT (V1) TO DTD VALUE";

	public static String[] ACTIONS = new String[] { 
		RENAME_ACTION,
		DELETE_ACTION
		// CONVERT_VALUE_ACTION 
	};

	public EntityRefactor() {
		super(Node.ENTITY_REFERENCE_NODE);
	}

	public String[] getActions() {
		return ACTIONS;
	}

	public String getName() {
		return "Entity";
	}

	protected Node refactorIt( Node node, RefactorAction ra ) {
		if ( DELETE_ACTION.equals( ra.getAction() ) ) {
			if ( ra.matchOldValue( node.getNodeName() ) )
				return null;
		} else if ( RENAME_ACTION.equals( ra.getAction() ) ) {
			if ( ra.matchOldValue(node.getNodeName() ) ) {
				if ( !ra.isNewValueEmpty() )
					return new EntityProxyNode((EntityReference) node, ra.getNewValue());
			}
		} else if ( CONVERT_VALUE_ACTION.equals( ra.getAction() ) ) {
			// Create a text node with the entity value
			if ( ra.matchOldValue( node.getNodeName() ) ) {
				EntityReference er = ( EntityReference )node;				
				return node.getOwnerDocument().createTextNode( node.getNodeValue() );
			}
		}
		return node;
	}

}
