package com.japisoft.framework.xml.refactor.elements;

import org.w3c.dom.Attr;
import org.w3c.dom.Node;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.refactor.ui.RefactorTable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PrefixRefactor extends AbstractRefactor {

	private static final String ADD_ACTION = "(V1) ADD TO ELEMENT (V2)";
	private static final String RENAME_ACTION = "(V1) RENAME TO (V2)";
	private static final String DELETE_ACTION = "DELETE (V1)";

	public static String[] ACTIONS = new String[] {
		RENAME_ACTION,
		DELETE_ACTION,
		ADD_ACTION
	};

	public PrefixRefactor() {
		super( ANY );
	}

        public String[] getActions() {
            return ACTIONS;
        }

        public String getName() {
            return "Prefix";
        }

	protected Node refactorIt( Node node, RefactorAction ra ) {
		if ( RENAME_ACTION.equals( ra.getAction() ) ) {
			if ( ra.matchOldValue( node.getPrefix() ) ) {
				if ( !ra.isNewValueEmpty() ) {
					node.setPrefix( ra.getNewValue() );
					return node;
				}
			} else
			if ( node.getNodeType() == Node.ATTRIBUTE_NODE ) {
				if ( node.getNodeName().equals( "xmlns:" + ra.getOldValue() ) ) {
					return new AttrProxyNode( ( Attr )node, "xmlns:" + ra.getNewValue() );
				}
			}
		} else
		if ( DELETE_ACTION.equals( ra.getAction() ) ) {
			if ( ra.matchOldValue( node.getPrefix() ) ) {
				node.setPrefix( null );
				return node;
			}
			
			if ( node.getNodeType() == Node.ATTRIBUTE_NODE ) {
				if ( node.getNodeName().equals( "xmlns:" + ra.getOldValue() ) ) {
					return null;
				}
			}
			
		} else
		if ( ADD_ACTION.equals( ra.getAction() ) ) {
			if ( ra.matchNewValue( node.getLocalName() ) ) {
				if ( ra.getOldValue() == null )
					throw new RuntimeException( "Prefix Refactor : Invalid oldValue for " + ADD_ACTION );
				node.setPrefix( ra.getOldValue() );				
			}
		}
		return node;
	}
	
	public void initTable(RefactorTable table, FPNode context) {
		if ( context.getNameSpacePrefix() != null )
			table.init( 0, context.getNameSpacePrefix() );
	}
}
