package com.japisoft.framework.xml.refactor.ui;

import com.japisoft.framework.xml.refactor.RefactorManager;
import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.EventObject;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.event.CellEditorListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import com.japisoft.framework.xml.refactor.elements.RefactorAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RefactorTable extends JTable {

	private String[] actions;
 // private static int MAX = Preferences.getPreference( "refactor", "maxline", 30 );
    private static int MAX = 30;

	public RefactorTable( String type ) {
		super( 
                    new DefaultTableModel(
                        new String[] { 
                            "Old Value (V1)", 
                            "Action", 
                            "New Value (V2)" }, 
                        MAX ) );

        actions = RefactorManager.getRefactorActions( type );
		getColumnModel().getColumn( 1 ).setCellRenderer( new CustomTableRenderer() );
		getColumnModel().getColumn( 1 ).setCellEditor( new CustomTableEditor() );
		getColumnModel().getColumn( 1 ).setPreferredWidth( 120 );
	}

	public void init( int row, String value ) {
		if ( value != null )
			getModel().setValueAt( value, row, 0 );
	}
	
	private boolean isValidAction( int row ) {
		if ( getValueAt( row, 0 ) != null && 
				getValueAt( row, 1 ) != null ) {
			String action = ( String )getValueAt( row, 1 );
			if ( action.indexOf( "V2" ) > -1 )
				return getValueAt( row, 2 ) != null;
			return true;
		}
		return false;
	}

	public RefactorAction[] getActions() {
		if ( getCellEditor() != null ) 
			getCellEditor().stopCellEditing();
		int cpt = 0;
		for ( int i = 0; i < getRowCount(); i++ ) {
			if ( isValidAction( i ) )
				cpt++;
		}
		if ( cpt == 0 )
			return null;
		RefactorAction[] rat = new RefactorAction[ cpt ];
		cpt = 0;
		for ( int i = 0; i < getRowCount(); i++ ) {
			if ( isValidAction( i ) ) {
				RefactorAction ra = new RefactorAction(
						( String )getValueAt( i, 1 ),
						( String )getValueAt( i, 0 ),
						( String )getValueAt( i, 2 ) );
				rat[ cpt++ ] = ra;
			}
		}
		return rat;
	}

	public void setActions( RefactorAction[] actions ) {
		DefaultTableModel model = ( DefaultTableModel )getModel();
		while ( model.getRowCount() > 0 ) 
			model.removeRow( 0 );
		if ( actions != null )
		for ( int i = 0; i < actions.length; i++ ) {
			model.addRow(
					new Object[] {
							actions[ i ].getOldValue(),
							actions[ i ].getAction(),
							actions[ i ].getNewValue()
					} );
		}
	}

	class CustomTableRenderer implements TableCellRenderer {
		private JLabel cb = new JLabel();
		
		public CustomTableRenderer() {
			cb.setOpaque( true );
		}
		
		public Component getTableCellRendererComponent(
				JTable table, 
				Object value, 
				boolean isSelected, 
				boolean hasFocus, 
				int row, 
				int column ) {			
			cb.setText( ( String )value );
			if ( isSelected ) {
				cb.setBackground( table.getSelectionBackground() );
				cb.setForeground( table.getSelectionForeground() );
			} else {
				cb.setBackground( table.getBackground() );
				cb.setForeground( table.getForeground() );
			}
			return cb;
		}
	}

	class CustomTableEditor implements TableCellEditor,ItemListener,MouseListener {
		private JComboBox cb = new JComboBox( actions );

		CustomTableEditor() {
			cb.addItemListener( this );
			addMouseListener( this );
		}

		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {			
			cb.setSelectedItem( ( String )value );
			return cb;
		}
		CellEditorListener l;
		public void addCellEditorListener(CellEditorListener l) {
			this.l = l;
		}
		public void itemStateChanged(ItemEvent e) {
			if ( l != null )
				l.editingStopped( null );	
		}
		public void mouseClicked(MouseEvent e) {
			itemStateChanged( null );
		}		
		public void mouseEntered(MouseEvent e) {
		}
		public void mouseExited(MouseEvent e) {
		}
		public void mousePressed(MouseEvent e) {
		}
		public void mouseReleased(MouseEvent e) {
		}
		public void cancelCellEditing() {
		}
		public Object getCellEditorValue() {
			return cb.getSelectedItem();
		}
		public boolean isCellEditable(EventObject anEvent) {
			return true;
		}
		public void removeCellEditorListener(CellEditorListener l) {
		}
		public boolean shouldSelectCell(EventObject anEvent) {
			return false;
		}
		public boolean stopCellEditing() {
			return false;
		}
	}
}
