package com.japisoft.framework.xml.refactor2.elements.schema;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.refactor2.AbstractRefactor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TypeRefactor extends AbstractRefactor {

	public boolean requireNewValue() {
		return true;
	}

	public boolean process(FPNode node) {

		// For all references
		if ( node.matchContent( "complexType" )
				&& node.hasAttribute( "name" ) ) {
			oldValue = node.getAttribute( "name" );			
			return true;
		}

		if ( node.matchContent( "simpleType" )
				&& node.hasAttribute( "name" ) ) {
			oldValue = node.getAttribute( "name" );			
			return true;
		}

		if ( node.matchContent( "element" ) ||
				node.matchContent( "attribute" ) ) {
			if ( node.hasAttribute( "type" ) ) {
				oldValue = node.getAttribute( "type" );
				if ( !oldValue.startsWith( "xsd:" ) &&
						!oldValue.startsWith( "xs:" ) ) {	
					int i = oldValue.lastIndexOf( ":" );
					if ( i > -1 ) {
						oldValue = oldValue.substring( i + 1 );
					}
					return true;
				}
			}
		}

		if ( node.matchContent( "restriction" ) ||
				node.matchContent( "extension" ) ) {
			if ( node.hasAttribute( "base" ) ) {
				oldValue = node.getAttribute( "base" );
				if ( !oldValue.startsWith( "xsd:" ) &&
						!oldValue.startsWith( "xs:" ) ) {	
					int i = oldValue.lastIndexOf( ":" );
					if ( i > -1 ) {
						oldValue = oldValue.substring( i + 1 );
					}
					return true;
				}				
			}
		}

		return false;
	}

	public String getTitle(FPNode node) {
		return "The type '" + oldValue + "'";
	}

	public void startElement(
			String uri, 
			String localName, 
			String qName, 
			Attributes atts ) throws SAXException {

		if ( "complexType".equals( localName ) ) {
			if ( atts.getValue( "name" ) != null && 
					!atts.getValue( "name" ).equals( "" ) ) {
				atts = new AttributesProxy( atts, "name" );
			}
		} else
		if ( "simpleType".equals( localName ) ) {
			if ( atts.getValue( "name" ) != null && 
					!atts.getValue( "name" ).equals( "" ) ) {
				atts = new AttributesProxy( atts, "name" );
			}
		} else
		if ( "extension".equals( localName ) ) {
			if ( atts.getValue( "base" ) != null && 
					!atts.getValue( "base" ).equals( "" ) )
				atts = new AttributesProxy( atts, "base" );{
			}
		} else
		if ( "restriction".equals( localName ) ) {
			if ( atts.getValue( "base" ) != null && 
					!atts.getValue( "base" ).equals( "" ) )
				atts = new AttributesProxy( atts, "base" );{
			}
		} else
		if ( "element".equals( localName ) ) {
			if ( atts.getValue( "type" ) != null && 
					!atts.getValue( "type" ).equals( "" ) )
				atts = new AttributesProxy( atts, "type" );{
			}			
		} else
		if ( "attribute".equals( localName ) ) {
			if ( atts.getValue( "type" ) != null && 
					!atts.getValue( "type" ).equals( "" ) )
				atts = new AttributesProxy( atts, "type" );{
			}			
		}

		super.startElement(uri, localName, qName, atts);
	}

	// ---------------------------------------------------------------
	
	class AttributesProxy implements Attributes {

		private Attributes ref;
		private String attName;
		
		AttributesProxy( Attributes ref, String attName ) {
			this.ref = ref;
			this.attName = attName;
		}

		public int getIndex(String uri, String localName) {
			return ref.getIndex( uri, localName );
		}

		public int getIndex(String qName) {
			return ref.getIndex( qName );
		}

		public int getLength() {
			return ref.getLength();
		}

		public String getLocalName(int index) {
			return ref.getLocalName( index );
		}

		public String getQName(int index) {
			return ref.getQName( index );
		}

		public String getType(int index) {
			return ref.getType( index );
		}

		public String getType(String uri, String localName) {
			return ref.getType( uri, localName );
		}

		public String getType(String qName) {
			return ref.getType( qName );
		}

		public String getURI(int index) {
			return ref.getURI( index );
		}

		private String processValue( String name, String value ) {
			int i = value.lastIndexOf( ":" );
			String prefix = "";
			if ( i > -1 ) {
				prefix = value.substring( 0 , i + 1 );
				value = value.substring( i + 1 );
			}
			if ( attName.equals( name ) && 
					value.equals( oldValue ) )
				return prefix + newValue;
			return value;
		}

		public String getValue(int index) {
			return processValue( ref.getQName( index ), ref.getValue( index ) );
		}

		public String getValue(String uri, String localName) {
			return processValue( localName, ref.getValue( uri, localName ) );
		}

		public String getValue(String qName) {
			return processValue( qName, ref.getValue( qName ) );
		}

	}

}
