package com.japisoft.framework.xml.refactor2.elements.xslt;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.refactor2.AbstractRefactor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class VariableParamRefactor extends AbstractRefactor {

	public boolean requireNewValue() {
		return true;
	}

	private String extractVariableName( FPNode node ) {
		if ( node.matchContent( "variable" ) && 
				node.hasAttribute( "name" ) )
			return node.getAttribute( "name" );

		if ( node.matchContent( "param" ) && 
				node.hasAttribute( "name" ) )
			return node.getAttribute( "name" );

		if ( node.matchContent( "with-param" ) && 
				node.hasAttribute( "name" ) )
			return node.getAttribute( "name" );

		for ( int i = 0; i < node.getViewAttributeCount(); i++ ) {
			String att = node.getViewAttributeAt( i );
			String val = node.getAttribute( att );
			int j = val.indexOf( "$" );
			if ( j > -1 ) {
				StringBuffer sb = new StringBuffer();
				for ( int k = j + 1; k < val.length(); k++ ) {
					if ( Character.isLetterOrDigit( val.charAt( k ) )
							|| val.charAt( k ) == '_' )
						sb.append( val.charAt( k ) );
					else
						break;
				}
				if ( sb.length() > 0 ) {
					return sb.toString();
				}
			}
		}
		return null;
	}

	public String getTitle( FPNode node ) {
		String name = extractVariableName( node );
		if ( name != null ) {			
			return "The variable or parameter '" + name + "'";
		}
		return null;
	}

	public boolean process( FPNode node ) {
		return ( oldValue = extractVariableName( node ) ) != null;
	}

	/////////////////////////////////////////////////////////////

	public static final String NS = "http://www.w3.org/1999/XSL/Transform";

	public void startElement(
			String uri,
			String localName, 
			String qName, 
			Attributes atts ) throws SAXException {
		
		boolean found = false;
		
		if ( NS.equals( uri ) && 
				( "variable".equals( localName ) || 
						( "param".equals( localName ) ) ||
							( "with-param".equals( localName ) ) ) ) {
			if ( oldValue.equals( 
					atts.getValue( "name" ) ) ) {

				atts = new AttributesProxy( atts, true );
				found = true;
				
			}
		}

		if ( !found ) {
			
			// Search in the atts value
			String waited = "$" + oldValue;
			for ( int i = 0; i < atts.getLength(); i++ ) {

				if ( atts.getValue( i ).contains( waited ) ) {

					atts = new AttributesProxy( atts, false );
					
				}

			}

		}

		super.startElement(
				uri, 
				localName, 
				qName, 
				atts
		);
	}

	// -----------------------------------------------------------
	
	class AttributesProxy implements Attributes {

		private Attributes ref;
		private boolean variableOrParam = false;
		
		AttributesProxy( Attributes ref, boolean variable ) {
			this.ref = ref;
			this.variableOrParam = variable;
		}

		public int getIndex(String uri, String localName) {
			return ref.getIndex( uri, localName );
		}

		public int getIndex(String qName) {
			return ref.getIndex( qName );
		}

		public int getLength() {
			return ref.getLength();
		}

		public String getLocalName(int index) {
			return ref.getLocalName( index );
		}

		public String getQName(int index) {
			return ref.getQName( index );
		}

		public String getType(int index) {
			return ref.getType( index );
		}

		public String getType(String uri, String localName) {
			return ref.getType( uri, localName );
		}

		public String getType(String qName) {
			return ref.getType( qName );
		}

		public String getURI(int index) {
			return ref.getURI( index );
		}

		private String processValue( String value ) {
			if ( variableOrParam && 
					value.equals( oldValue ) )
				return newValue;

			if ( !variableOrParam ) {
				value = value.replaceAll( 
						"\\$" + oldValue, 
							"\\$" + newValue );
			}

			return value;
		}

		public String getValue(int index) {
			return processValue( ref.getValue( index ) );
		}

		public String getValue(String uri, String localName) {
			return processValue( ref.getValue( uri, localName ) );
		}

		public String getValue(String qName) {
			return processValue( ref.getValue( qName ) );
		}
		
	}

}
