package com.japisoft.framework.xml.refactors.elements.xslt;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.refactor.elements.AbstractRefactor;
import com.japisoft.framework.xml.refactor.elements.RefactorAction;
import com.japisoft.framework.xml.refactor.ui.RefactorTable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class VariableRefactor extends AbstractRefactor {
	private static final String RENAME_ACTION = "(V1) RENAME TO (V2)";
	private static final String DELETE_ACTION = "DELETE (V1)";

	public static String[] ACTIONS = new String[] {
		RENAME_ACTION,
		DELETE_ACTION,
	};

	public VariableRefactor( ) {
		super( Node.ELEMENT_NODE );
	}

	public String[] getActions() {
		return ACTIONS;
	}

	public String getName() {
		return "xslt Variable";
	}

	public boolean isDefault() {
		return false;
	}

	boolean variableFound = false;
	
	protected Node refactorIt( Node node, RefactorAction ra ) {
		if ( "variable".equals( node.getLocalName() ) ) {
			Element e = ( Element )node;
			if ( ra.matchOldValue( e.getAttribute( "name" ) ) ) {
				variableFound = true;
				if ( RENAME_ACTION.equals( ra.getAction() ) ) {
					if ( !ra.isNewValueEmpty() )
						e.setAttribute( "name", ra.getNewValue() );					
				} else 
				if ( DELETE_ACTION.equals( ra.getAction() ) ) {
					return null;
				}
			}
		} else {
			if ( variableFound ) {
				if ( "value-of".equals( node.getLocalName() ) || 
					"variable".equals( node.getLocalName() ) ) {
					Element e = ( Element )node;
					String value = e.getAttribute( "select" );
					if ( value != null ) {
						String newValue = ( "$" + ra.getNewValue() );
						if ( DELETE_ACTION.equals( ra.getAction() ) )
							newValue = "";
						value = value.replace( 
								"$" + ra.getOldValue(), 
								newValue );
						e.setAttribute( "select", value );
					}
				}
			}
		}
		return node;
	}

	public void stop() {
		variableFound = false;
	}

	public void initTable(RefactorTable table, FPNode context) {
		if ( context.matchContent( "variable" ) ) {
			if ( context.hasAttribute( "name" ) )
				table.init( 0, context.getAttribute( "name" ) );
		}
	}
		
}
