package com.japisoft.multipanes;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicTitledPane implements TitledPane {

	/** 	
	 * @param name Name of the titledPane, it can't be <code>null</code>
	 * @param title Title of the titledPane, it can't be <code>null</code>
	 * @param icon Major icon, it can be <code>null</code>
	 * @param tooltip help for the titledPane, it can be <code>null</code>
	 * @param view, main view, it can't be <code>null</code> */
	public BasicTitledPane( String name, String title, Icon icon, String tooltip, JComponent view ) {
		if ( name == null )
			throw new RuntimeException( "illegal null name " );
		if ( title == null )
			throw new RuntimeException( "illegal null title " );
		if ( view == null )
			throw new RuntimeException( "illegal null view" );
		this.name = name;
		this.title = title;
		this.icon = icon;
		this.tooltip = tooltip;
		this.view = view;
	}

	/** All parameters are required, so they can't be <code>null</code> */
	public BasicTitledPane( String name, String title, JComponent view ) {
		this( name, title, null, null, view );
	}
	
	public BasicTitledPane( String name, String title, String tooltip, JComponent view ) {
		this( name, title, null, tooltip, view );
	}
	
	private String name;
	private String title;
	private Icon icon;
	private String tooltip;
	private JComponent view;

	public String getTitle() {
		return title;
	}

	public void setTitle( String title ) {
		if ( this.title != title ) {
			propertyChanged( Action.NAME, this.title, title );
		}
	}

	public Icon getIcon() {
		return icon;
	}

	public void setIcon( Icon icon ) {
		if ( this.icon != icon )
			propertyChanged( Action.SMALL_ICON, this.icon, icon );
		this.icon = icon;
	}

	public String getToolTip() {
		return tooltip;
	}

	public void setToolTip( String tooltip ) {
		if ( this.tooltip != tooltip ) {
			propertyChanged( Action.SHORT_DESCRIPTION, this.tooltip, tooltip );
		}
		this.tooltip = tooltip;
	}

	public String getName() {
		return name;
	}

	public JComponent getView() {
		return view;
	}
	
	public void open() {
	}

	public void close() {
	}

	private boolean enabled = true;

	public void setEnabled( boolean enabled ) {
		if ( this.enabled != enabled )
			propertyChanged( 
				"enabled", 
				new Boolean( this.enabled ), 
				new Boolean( enabled ) );	
		this.enabled = enabled;
		view.setEnabled( enabled );
	}

	public boolean isEnabled() {
		return enabled;
	}

	public void dispose() {
		if ( listeners != null ) {
			listeners.removeAll( listeners );
			listeners = null;
		}
	}

	private ArrayList listeners = null;
	private Color background = null;
	private Color foreground = null;
	
	public void setBackground( Color background ) {
		this.background = background;
		propertyChanged( 
				"background",
				this.background,
				background );
	}
	
	public void setForeground( Color foreground ) {
		this.foreground = foreground;
		propertyChanged(
				"foreground",
				this.foreground,
				foreground );
	}

	public Color getBackground() { return background; }
	public Color getForeground() { return foreground; }
	
	public void addPropertyChangeListener( PropertyChangeListener listener ) {
		if ( listeners == null )
			listeners = new ArrayList();
		listeners.add( listener );		
	}

	private void propertyChanged( String name, Object oldValue, Object newValue ) {
		if ( listeners != null ) {
			PropertyChangeEvent pce = 
				new PropertyChangeEvent(
					this, name, oldValue, newValue );
			
			for ( int i = 0; i < listeners.size(); i++ ) {
				PropertyChangeListener listener = ( PropertyChangeListener )listeners.get( i );
				listener.propertyChange( pce );
			}
		}
	}

}
