package com.japisoft.stylededitor.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;

import javax.swing.text.Element;

import com.japisoft.stylededitor.EditorByCSS;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ViewPartSelection implements ViewPart {

	public void paint( EditorByCSS editor, Graphics gc ) {
		
		if ( editor.getSelectionEnd() != editor.getSelectionStart() ) {
			// Display selection
			
			int start = Math.min( editor.getSelectionStart(), editor.getSelectionEnd() );
			int end = Math.max( editor.getSelectionStart(), editor.getSelectionEnd() );

			int indexStartSelection = editor.getDocument().getDefaultRootElement().getElementIndex( start );
			int indexEndSelection = editor.getDocument().getDefaultRootElement().getElementIndex( end );

			Element startElement = editor.getDocument().getDefaultRootElement().getElement( indexStartSelection );
			Element endElement = editor.getDocument().getDefaultRootElement().getElement( indexEndSelection );

			if ( startElement == null ) return;
			if ( endElement == null ) return;

			int startOffset = start - startElement.getStartOffset();
			int endOffset = end - endElement.getStartOffset();			
			
			for ( int i = indexStartSelection; i<= indexEndSelection; i++ ) {

				Element e = editor.getDocument().getDefaultRootElement().getElement( i );
				if ( e == null )
					return;

				int startOffsetTmp = e.getStartOffset();
				int endOffsetTmp = e.getEndOffset();

				if ( i == indexStartSelection ) {
					startOffsetTmp += startOffset;
				}

				if ( i == indexEndSelection ) {
					endOffsetTmp = e.getStartOffset() + endOffset;
				}

				Rectangle r1 = editor.modelToView( 
						startOffsetTmp 
				);
				Rectangle r2 = editor.modelToView( 
						endOffsetTmp 
				);				

				if ( r1 == null ) {
					return;
				}

				if ( r2 == null ) {
					return;
				}

				int xi = r1.x + r1.width;
				int yi = r1.y;

				int xj = r2.x + r2.width;
				int yj = r2.y + r2.height;

				gc.setColor( Color.WHITE );
				gc.setXORMode( Color.BLUE );
				gc.fillRect( xi, yi, xj - xi, yj - yi );
				
			}

		}
	}

}
