package com.japisoft.stylededitor.ui.node;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;

import com.japisoft.stylededitor.EditorByCSS;
import com.japisoft.stylededitor.model.NodeElement;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NodeElementViewProxy implements ElementView {

	private ElementView source = null;
	
	public NodeElementViewProxy( ElementView source ) {
		this.source = source;
	}
	
	public Rectangle getBounds(EditorByCSS editor, NodeElement element,
			int offset, Graphics gc) {
		return source.getBounds(editor, element, offset, gc );
	}

	public int getOffset(EditorByCSS editor, NodeElement element, int x, int y,
			Graphics gc) {
		return source.getOffset(editor, element, x, y, gc);
	}

	private Dimension getTagDimension( NodeElement element, Graphics gc ) {
		String tagName = element.getNode().getNodeName();
		FontMetrics fm = gc.getFontMetrics();		
		int width = fm.stringWidth( tagName ) + 2;
		int height= fm.getHeight() + 2;		
		return new Dimension( width, height );		
	}
	
	public Dimension getSize(EditorByCSS editor, NodeElement element,
			Graphics gc) {
		Dimension dim = source.getSize(editor, element, gc);
		
		Dimension tagDim = getTagDimension( element, gc );

		if ( dim == null ) {
			element.setData( "size", tagDim );
			return tagDim;
		}

		return new Dimension( dim.width + tagDim.width, dim.height + tagDim.height );
	}

	public boolean isInside(EditorByCSS editor, NodeElement element, int x,
			int y, Graphics gc) {
		return source.isInside(editor, element, x, y, gc);
	}
	
	private Color TAG_COLOR = new Color( 255, 204, 102 );

	public void paint(EditorByCSS editor, NodeElement element, int x, int y, Graphics gc) {
		Dimension tagDim = getTagDimension( element, gc );
		
		source.paint(editor, element, x, y + tagDim.height + 1, gc);
		
		element.setData( "tag.x", x );
		element.setData( "tag.y", y );
		element.setData( "tag.width", tagDim.width );
		element.setData( "tag.height", tagDim.height );

		gc.setColor( TAG_COLOR );
		gc.fill3DRect( 0, 0, tagDim.width, tagDim.height, true );
		gc.setColor( Color.BLACK );
		( ( Graphics2D )gc ).setRenderingHint( 
				RenderingHints.KEY_TEXT_ANTIALIASING, 
				RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB 
		);

		gc.drawString( element.getNode().getNodeName(), 1, gc.getFontMetrics().getAscent() + 1 );
	}

}
