package com.japisoft.universalbrowser;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemException;
import org.apache.commons.vfs.FileSystemManager;
import org.apache.commons.vfs.VFS;
import org.apache.commons.vfs.provider.local.LocalFile;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JUniversalBrowserTree extends JTree 
		implements 
		TreeSelectionListener, 
		MouseListener, 
		MouseMotionListener {

	public JUniversalBrowserTree() {
		setCellRenderer( new TreeCellRendererImpl() );
	}

	public void addNotify() {
		super.addNotify();
		addTreeSelectionListener( this );
		addMouseListener( this );
		addMouseMotionListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		removeTreeSelectionListener( this );
		removeMouseListener( this );
		removeMouseMotionListener( this );
	}

	public void setFileView( FileView fv ) {
		( ( TreeCellRendererImpl )getCellRenderer() ).setFileView( fv );
	}

	public void setFileFilter( FileFilter ff ) {
		( ( TreeCellRendererImpl )getCellRenderer() ).setFileFilter( ff );
		repaint();
	}

	public FileObject getBrowsedFileObject() {
		if ( getModel().getRoot() instanceof FileObjectTreeNode )
			return ( ( FileObjectTreeNode )getModel().getRoot() ).getSource();
		return null;
	}

	public void refresh() throws Exception {
		if ( lastUri != null )
			browse( lastUri );
	}
	
	private boolean errorMode = false;

	public void browse( FileObject fo ) {

		errorMode = false;
		
		if ( fo instanceof LocalFile ) {
			try {
				
				lastUri = fo.getURL().toExternalForm();
				
				if ( !fo.exists() ) {
					errorMode = true;
					setModel( 
							new DefaultTreeModel( 
									new DefaultMutableTreeNode( 
											"Can't find the root path" ) ) );
				} else
					setModel( 
							new DefaultTreeModel( 
									new FileObjectTreeNode( fo ) ) );
			} catch (FileSystemException e) {
				setModel( 
						new DefaultTreeModel( 
								new DefaultMutableTreeNode( 
										"Can't find the root path" ) ) );
				errorMode = true;
			}
		} else
		
			setModel( 
					new DefaultTreeModel( 
							new FileObjectTreeNode( fo ) ) );
	}

	private String lastUri = null;

	public void browse( String uri ) throws FileSystemException {
		this.lastUri = uri;
		FileSystemManager fsManager = VFS.getManager();
		browse( fsManager.resolveFile( uri ) );
	}

	public void close() {
		try {
			if ( getModel().getRoot() instanceof FileObjectTreeNode ) {
				FileSystemManager fsManager = VFS.getManager();
				fsManager.closeFileSystem( 
						( (FileObjectTreeNode)getModel().getRoot() ).getSource().getFileSystem() 
				);
			}
		} catch (FileSystemException e) {
		}
	}
	
	private UniversalBrowserListener listener = null;
	
	public void setUnivervalBrowserListener( UniversalBrowserListener listener ) {
		this.listener = listener;
	}

	public void valueChanged(TreeSelectionEvent e) {
		if ( errorMode )
			return;
		
		FileObjectTreeNode fo = ( FileObjectTreeNode )e.getPath().getLastPathComponent();
		if ( !fo.isRoot() )
			if ( listener != null )
				listener.select( fo.getSource() );
	}	

	public FileObject getSelectedFileObject() {
		
		FileObjectTreeNode fotn = ( FileObjectTreeNode )getSelectionPath().getLastPathComponent();
		if ( fotn == null || fotn.isRoot() )
			return null;
		return fotn.getSource();
		
	}

	public void select( String uri ) {
		int i = uri.lastIndexOf( "!/" );
		if ( i > -1 ) {
			String path = uri.substring( i + 2 );
			StringTokenizer st = 
				new StringTokenizer( path, "/\\" );

			TreeNode currentParent = ( TreeNode )getModel().getRoot();
			ArrayList res = new ArrayList();
			res.add( currentParent );
			
			while ( st.hasMoreTokens() ) {

				currentParent = search( st.nextToken(), currentParent );
				if ( currentParent == null ) 
					break;
				res.add( currentParent );
				
			}
			
			setSelectionPath( new TreePath( res.toArray() ) );
		}
	}
	
	private TreeNode search( String name, TreeNode parent ) {
		for ( int i = ( parent.getChildCount() - 1 ); i >= 0 ;i-- ) {
			TreeNode tn = parent.getChildAt( i );
			if ( tn.toString().equals( name ) )
				return tn;
		}
		return null;
	}

	public void mouseDragged(MouseEvent e) {}

	public void mouseMoved(MouseEvent e) {
		if ( errorMode )
			return;

		TreePath tp = getPathForLocation( e.getX(), e.getY() );
		if ( tp != null ) {
			FileObjectTreeNode fon = ( FileObjectTreeNode )tp.getLastPathComponent();
			FileObject fo = fon.getSource();
			setToolTipText( fo.getName().getURI() );
		}
	}

	public void mouseClicked(MouseEvent e) {
		if ( errorMode )
			return;

		if ( e.getClickCount() >= 2 ) {
			if ( listener != null ) {
				if ( getSelectionPath() != null ) {
					FileObjectTreeNode fotn = ( FileObjectTreeNode )getSelectionPath().getLastPathComponent();				
					if ( fotn.isLeaf() )
						listener.doubleClick( getSelectedFileObject() );
				}
			}
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	/**
	 * @param args */
	public static void main(String[] args) throws Throwable {

		FileSystemManager fsManager = VFS.getManager();
		
		JUniversalBrowserTree ub = new JUniversalBrowserTree();
		//ub.browse(
		//		fsManager.resolveFile( "jar:file://c:/cdam-ab.zip" ) );

		ub.browse(
				fsManager.resolveFile( "file:///c:/titi/" ) );		

		JFrame f = new JFrame();
		f.add( new JScrollPane( ub ) );
		f.setSize( 300, 300 );
		f.setVisible( true );

	}

}
