package com.japisoft.universalbrowser;

import java.awt.Color;
import java.awt.Component;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.UIManager;

import com.japisoft.framework.ui.FastLabel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
class TreeCellRendererImpl implements javax.swing.tree.TreeCellRenderer {

	FastLabel fastlabel = new FastLabel(false);
	Icon drive = null;
	Icon document = null;
	Icon folder = null;
	Icon folder_closed = null;

	public TreeCellRendererImpl() {
		try {
			drive = new ImageIcon(getClass().getResource(
					"diskdrive.png"));
			document = new ImageIcon(getClass().getResource(
					"document.png"));
			folder = new ImageIcon(getClass().getResource(
					"folder.png"));
			folder_closed = new ImageIcon(getClass().getResource(
					"folder_closed.png"));
		} catch (Throwable th) {
			System.err.println("Can't init icons ? : " + th.getMessage());
		}
	}

	FileView fv = null;

	void setFileView(FileView fv) {
		this.fv = fv;
	}

	FileFilter filter = null;
	
	void setFileFilter( FileFilter filter ) {
		this.filter = filter;
	}
	
	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean selected, boolean expanded, boolean leaf, int row,
			boolean hasFocus) {

		if (!( value instanceof FileObjectTreeNode ) ) {
			fastlabel.setBackground( tree.getBackground() );
			fastlabel.setForeground( Color.RED );
			fastlabel.setText( value.toString() );
			return fastlabel;
		}

		FileObjectTreeNode node = (FileObjectTreeNode) value;
		
		Icon icon = null;

		if (node.isRoot())
			icon = drive;
		else {
			
			if ( node.isFolder() ) {
				icon = !expanded ? folder_closed : folder;
			} else
				icon = document;

		}

		if ( fv != null ) {
			if ( !node.isRoot() ) {
				Icon iconTmp = fv.getIcon( node.getSource() );
				if ( iconTmp != null )
					icon = iconTmp;
			}
		}

		fastlabel.setIcon( icon );
		
		fastlabel.setText(node.toString());

		if (selected) {
			fastlabel.setForeground(UIManager
					.getColor("List.selectionForeground"));
			fastlabel.setBackground(UIManager
					.getColor("List.selectionBackground"));
		} else {

			Color foreground = tree.getForeground();
			Color background = tree.getBackground();

			if ( fv != null ) {
				if ( !node.isRoot() ) {
					Color tmp = fv.getForeground( node.getSource() );
					if ( tmp != null )
						foreground = tmp;
					tmp = fv.getBackground( node.getSource() );
					if ( tmp != null )
						background = tmp;
				}
			}

			if ( filter != null && !node.isRoot() ) {
				if ( !filter.accept( node.getSource() ) )
					foreground = background.darker().darker();
			}

			fastlabel.setForeground( foreground );
			fastlabel.setBackground( background );

		}

		fastlabel.setToolTipText( tree.getToolTipText() );
		
		return fastlabel;
	}

}
