package com.japisoft.xflows.action.options.tasks.wizard;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

import javax.swing.DefaultListModel;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import com.japisoft.framework.ui.SimpleFileFilter;
import com.japisoft.framework.wizard.BasicStepView;
import com.japisoft.framework.wizard.BasicWizardStep;
import com.japisoft.framework.wizard.JWizard;
import com.japisoft.framework.wizard.StepView;
import com.japisoft.framework.wizard.WizardStepContext;
import com.japisoft.xflows.task.ui.XFlowsFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NewTaskWizard extends JWizard {

	public NewTaskWizard() {
		super();
		getWizardStepModel().addWizardStep(new FirstStep());
		getWizardStepModel().addWizardStep(new SecondStep());
		getWizardStepModel().addWizardStep(new ThirdStep());
		getWizardStepModel().addWizardStep(new FourthStep());
	}
	
	class FirstStep extends BasicWizardStep {

		JFileChooser chooser = null;

		public FirstStep() {
			super("archive");
			setShortTitle("Java archive");
			setLongTitle("Choose a java archive file containing your Task classes");
			chooser = new JFileChooser();
			chooser.setControlButtonsAreShown(false);

			chooser.addChoosableFileFilter(new SimpleFileFilter("Java Archive",
					"jar"));
			setView( new BasicStepView( chooser ) );
		}

		public void stop(WizardStepContext context) {
			context.setSharedData("file", chooser.getSelectedFile());
		}
	}

	class SecondStep extends BasicWizardStep {

		DefaultListModel model = new DefaultListModel();

		JList list;

		public SecondStep() {
			super("task");
			setShortTitle("Task class");
			setLongTitle("Choose your java Task class");
			setView( new BasicStepView( new JScrollPane(list = new JList( model ) ) ) );
		}

		public String storeFileKey = "running";

		public boolean canStart(WizardStepContext context) {
			File f = (File) context.getSharedData( "file" );
			if (f == null) {
				XFlowsFactory.buildAndShowErrorDialog( "Wrong archive file" );
				return false;
			}
			return true;
		}

		public boolean start(WizardStepContext context) {
			File f = (File) context.getSharedData( "file" );
			model.removeAllElements();

			try {
				JarInputStream input = new JarInputStream(
						new FileInputStream(f));
				try {
					JarEntry entry = null;
					do {
						entry = input.getNextJarEntry();
						if ( entry != null && entry.getName().endsWith( ".class" ) ) {

							String cl = entry.getName();
							cl = cl.replace('/', '.');
							cl = cl.replace('\\', '.');
							cl = cl.substring(0, entry.getName().lastIndexOf(
									"."));
							model.addElement(cl);
						}

					} while (entry != null);
				} finally {
					try {
						input.close();
					} catch (IOException exc) {
					}
				}
			} catch (FileNotFoundException e) {
				XFlowsFactory.buildAndShowErrorDialog(e.getMessage());
			} catch (IOException e) {
				XFlowsFactory.buildAndShowErrorDialog(e.getMessage());
			}

			return true;
		}

		public void stop(WizardStepContext context) {
			context.setSharedData(storeFileKey, list.getSelectedValue());
		}

	}

	class ThirdStep extends SecondStep {

		public ThirdStep() {
			storeFileKey = "ui";
			setShortTitle( "UI class" );
			setLongTitle( "Choose your java User Interface Task class" );
			setView( new BasicStepView( new JScrollPane(list = new JList( model ) ) ) );
		}
	}

	class FourthStep extends BasicWizardStep {

		private JTextField tfName;
		
		public FourthStep() {
			super("third");
			setShortTitle("Name");
			setLongTitle("Choose a short task name");

			JLabel lblName = new JLabel();
			tfName = new JTextField();

			lblName.setText("Name");

			JPanel panel = new JPanel();
			
			panel.setLayout(new GridBagLayout());
			tfName.setText("");
			panel.add(lblName, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(71, 7, 0, 0), 0, 0));
			panel.add(tfName, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
					new Insets(8, 7, 101, 7), 382, 0));
			
			setView( new BasicStepView( panel ) );
		}
		
		public void stop(WizardStepContext context) {
			context.setSharedData( "name", tfName.getText() );
		}

	}

}
