package com.japisoft.xflows.task;

import java.io.File;

import com.japisoft.framework.toolkit.Logger;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicTaskContext extends AbstractTaskContext {

	public static final int INFO = 0;
	public static final int WARNING = 1;
	public static final int ERROR = 2;

	private Task source = null;

	public BasicTaskContext( Task source ) {
		this.source = source;
	}

	public BasicTaskContext( TaskContext context ) {
		this( context.currentTask() );
	}

	public Task currentTask() {
		return source;
	}
	
	boolean quietInfo = false;

	/** Enabled/Disabled the information output */
	public void setQuietInfoMode( boolean quiet ) {
		this.quietInfo = quiet;
	}

	boolean quietWarning = false;
	
	/** Enabled/Disabled the warning output */
	public void setQuietWarningMode( boolean quiet ) {
		this.quietWarning = quiet;
	}

	boolean quietError = false;
	
	/** Enabled/Disabled the error output */
	public void setQuietErrorMode( boolean quiet ) {
		this.quietError = quiet;
	}

	/** Add a warning message in the log */
	public void addWarning( String message ) {
		if ( !quietWarning )
			Logger.addWarning( message );
	}
	
	/** Add an information in the log */
	public void addInfo( String message ) {
		if ( !quietInfo )
			Logger.addInfo( message );
	}

	private boolean errorFound = false;

	public boolean hasErrorFound() {
		return errorFound;
	}

	/** Add an error message in the log */
	public void addError( String message ) {
		this.errorFound = true;
		if ( !quietError )
			Logger.addError( message );
	}	

	/** @return a parameter value from the UI part */
	public String getParam( String name ) {
		String value = source.getParams().getParamValue( name );
		if ( "".equals( value ) )
			return null;
		return value;
	}

	/** 
	 * @return a parameter value from the UI part */
	public String getParam( String name, String def ) {
		String p = getParam( name );
		if ( p != null )
			return p;
		return def;
	}

	/** @return a parameter with a directory path */
	public String getParamForPath( String name ) {
		String param = getParam( name );
		if ( param != null && !param.endsWith( "/" ) )
			return param + "/";
		return param;
	}
		
	public boolean hasParam( String name ) {
		return source.getParams().hasParamValue( name ); 
	}
	
	public TaskParams getParams() {
		return source.getParams();
	}
	
	private boolean interrupted = false;

	/** Interrupt the task ? */
	public void interrupt() {
		interrupted = true;
	}
	
	public boolean isInterrupted() {
		return interrupted;
	}

	private File sourceFile = null;
	private File targetFile = null;
	
	/** Reset the current source file */
	public void setCurrentSourceFile( File f ) {
		this.sourceFile = f;
	}

	/** Reset the current target file */
	public void setCurrentTargetFile( File f ) {
		this.targetFile = f;
	}

	/** @return the current source file */
	public File getCurrentSourceFile() {
		return sourceFile;
	}
	
	/** @return the current target file */
	public File getCurrentTargetFile() {
		return targetFile;
	}	
	
}
