package com.japisoft.xflows.task;

import java.awt.Color;
import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.toolkit.Logger;
import com.japisoft.framework.toolkit.LoggerListener;
import com.japisoft.xflows.log.FileLogger;
import com.japisoft.xflows.task.ui.XFlowsFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TaskLogTable extends JTable 
	implements LoggerListener, ListSelectionListener {

	static final Integer INFO = new Integer( 0 );
	static final Integer WARNING = new Integer( 1 );
	static final Integer ERROR = new Integer( 2 );

	public TaskLogTable() {
		super();
		resetModel();
	}

	private void resetModel() {
		setModel( new DefaultTableModel(
				new Object[] { "T", "Message" }, 0 ) );
		getSelectionModel().setSelectionMode(
				ListSelectionModel.SINGLE_SELECTION );
		getColumnModel().getColumn( 0 ).setMaxWidth( 20 );
	}
	
	public void addNotify() {
		super.addNotify();
		Logger.addLoggerListener( this );
		getSelectionModel().addListSelectionListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		Logger.removeLoggerListener( this );
		getSelectionModel().removeListSelectionListener( this );
	}

	public void valueChanged(ListSelectionEvent e) {
		int row = getSelectedRow();
		if  ( row != -1 ) {
			setToolTipText(
					(String)getModel().getValueAt( row, 1 ) );
		}
		repaint();
	}	

    public boolean isCellEditable(int row, int column) {
    	return false;
    }	

    public void clean() {
    	resetModel();
    }
    
    // LOGGER
    
    private void updateLogsTitle() {
    }
    
	public void addInfo( String message ) {		
		((DefaultTableModel)getModel()).addRow(
				new Object[] {
						INFO,
						message
				} );
		updateLogsTitle();
	}	

	public void addWarning( String message ) {
		((DefaultTableModel)getModel()).addRow(
				new Object[] {
						WARNING,
						message
				} );
		updateLogsTitle();
	}

	public void addError( String message ) {
		((DefaultTableModel)getModel()).addRow(
				new Object[] {
						ERROR,
						message
				} );
		updateLogsTitle();
	}

	TableRenderer renderer;

	public TableCellRenderer getDefaultRenderer( Class columnClass ) {
		if ( renderer != null )
			return renderer;
		renderer = new TableRenderer();
		return renderer;
	}

	/////////////////////////////////////

	static ImageIcon info = XFlowsFactory.getImageIcon( "images/bug_green.png" );
	static ImageIcon warning = XFlowsFactory.getImageIcon( "images/bug_yellow.png" );
	static ImageIcon error = XFlowsFactory.getImageIcon( "images/bug_red.png" );

	class TableRenderer implements TableCellRenderer {
		
		Color first = Preferences.getPreference( "scenario", "logFirstColor", new Color( 220, 220, 240 ) );
		Color second = Preferences.getPreference( "scenario", "logSecondColor", new Color( 255, 255, 255 ) );
		JLabel lbl = new JLabel();

		public TableRenderer() {
			lbl.setOpaque( true );
		}

		public Component getTableCellRendererComponent(
				JTable table, 
				Object value,
				boolean isSelected, 
				boolean hasFocus, 
				int row, 
				int column ) {

			if ( row % 2 == 0 ) {
				lbl.setForeground( Color.black );
				lbl.setBackground( first );
			} else { 
				lbl.setForeground( Color.black );
				lbl.setBackground( second );
			}
			
			if ( isSelected ) {
				Color _tmp = lbl.getForeground();
				lbl.setForeground ( lbl.getBackground() );
				lbl.setBackground( _tmp );
			}

			if ( column == 1 ) {
			
				lbl.setIcon( null );
				if ( value != null ) 
					lbl.setText( value.toString() );
				else
					lbl.setText( null );
			
			} else {

				if ( value == INFO ) {
					lbl.setIcon( info );
				}
				else
				if ( value == WARNING ) {
					lbl.setIcon( warning );
				}
				else
				if ( value == ERROR ) {
					lbl.setIcon( error );
				}

				lbl.setText( null );
				
			}

			return lbl;
		}
	}

}
