package com.japisoft.xflows.task;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractCellEditor;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xflows.XFlowsApplicationModel;
import com.japisoft.xflows.task.ui.builder.TaskTypeListener;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TaskTable extends JTable implements ListSelectionListener {

	private TableCellEditor editor = null;
	private TableCellRenderer renderer = null;
	private TaskTypeListener listener = null;
	private boolean editable;
	
	public TaskTable( TaskTypeListener listener, boolean editable ) {
		super();
		this.editable = editable;
		this.listener = listener;
		setModel( new TableModel() );
		getColumnModel().getColumn( 0 ).setMaxWidth( 40 );
		getColumnModel().getColumn( 2 ).setMaxWidth( 90 );
		getColumnModel().getColumn( 2 ).setPreferredWidth( 90 );
		getSelectionModel().setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
	}

	public boolean isCellEditable(int row, int column) {
		if ( column == 0 )
			return false;
		return editable;
	}
	
	public void stopEditing() {
		if  ( editor != null )
			editor.stopCellEditing();
	}

	public void valueChanged(ListSelectionEvent e) {
		super.valueChanged( e );
		if  (getModel() instanceof TableModel ) {	
			if ( getSelectedRow() == -1 )
				return;
			Task t = getCurrentTask();
			if ( listener != null ) {				
				listener.showUI( t );
			}
		}
		repaint();
	}

	public Task getCurrentTask() {
		return ( Task )( (TableModel)getModel() ).innerModel.get(
					getSelectedRow() );
	}
	
	public TableCellEditor getDefaultEditor( Class columnClass ) {
		if ( editor != null )
			return editor;
		editor = new TableEditor();
		return editor;
	}
	
	public TableCellRenderer getDefaultRenderer( Class columnClass ) {
		if ( renderer != null )
			return renderer;
		renderer = new TableRenderer();
		return renderer;
	}

	public void setTasks( List<Task> tasks ) {
		( ( TableModel )getModel() ).resetModel( tasks );
		if ( tasks.size() > 0 )
			getSelectionModel().setSelectionInterval(0,0);
	}

	public List<Task> getTasks() {
		return ( ( TableModel )getModel() ).innerModel;
	}

	public void upTask( ) {
		if  ( editor != null )
			editor.stopCellEditing();
		
		TableModel model = ( TableModel )getModel();
		int row = getSelectedRow();
		if ( row > 0 ) {
			Task t = ( Task )model.innerModel.get( row );
			if ( t != null ) {
				model.innerModel.remove( t );
				model.innerModel.add( row - 1, t );
				model.fireTableDataChanged();
			}
		}
	}

	public void downTask() {
		if  ( editor != null )
			editor.stopCellEditing();
		
		TableModel model = ( TableModel)getModel();
		int row = getSelectedRow();
		if ( row < model.getRowCount() - 1 && row != -1 ) {
			Task t = ( Task )model.innerModel.get( row );
			if ( t != null ) {
				model.innerModel.remove( t );
				model.innerModel.add( row + 1, t );
				model.fireTableDataChanged();
			}
		}
	}

	public void deleteTask() {
		if  ( editor != null )
			editor.stopCellEditing();
		TableModel model = ( TableModel)getModel();
		int row = getSelectedRow();
		if ( row == -1 )
			return;
		model.innerModel.set( row, null );
		model.fireTableDataChanged();
	}

	public void runTask() {
		if ( editor != null )
			editor.stopCellEditing();	
		if ( getSelectedRow() != -1 ) {
			TaskManager.stopIt = false;
			TaskManager.run( ( Task )( ( TableModel )getModel() ).innerModel.get(
					getSelectedRow() ), TaskManager.BACKGROUND_DIALOG );
		}
	}

	public void selectTaskForName( String name ) {
		for ( int i = 0; i < getRowCount(); i++ ) {
			if ( name.equals( getValueAt( i, 1 ) ) ) {
				getSelectionModel().setSelectionInterval( i, i );
				break;
			}
		}
	}
	
	///////////////////////////////////

	Color first = Preferences.getPreference( "interface", "tableFirstColor", new Color( 220, 255, 220 ) );
	Color second = Preferences.getPreference( "interface", "tableSecondColor", new Color( 255, 255, 255 ) );
		
	class TableRenderer implements TableCellRenderer {

		JLabel lbl = new JLabel();
		JComboBox combo = new JComboBox( TaskElementFactory.getAvailableTypes() );
		
		public TableRenderer() {
			lbl.setOpaque( true );
			combo.insertItemAt( "No Action", 0 );
		}
		
		public Component getTableCellRendererComponent(
				JTable table, 
				Object value,
				boolean isSelected, 
				boolean hasFocus, 
				int row, 
				int column ) {

			Component c = lbl;
			if ( column == 2 )
				c = combo;

			if ( c != combo || !ApplicationModel.isMacOSXPlatform() ) {
			
				if ( row % 2 == 0 ) {
					c.setForeground( Color.black );
					c.setBackground( first );
				} else { 
					c.setForeground( Color.black );
					c.setBackground( second );
				}

				if ( isSelected ) {
					Color _tmp = c.getForeground();
					c.setForeground ( c.getBackground() );
					c.setBackground( _tmp );
				}
				
			}

			if ( c == lbl  ) {
			
				if ( value != null ) {
					lbl.setText( value.toString() );
				}
				else {
					lbl.setText( null );
				} 
		
				return lbl;

			} else {
			
				if ( value != null ) {
					
					combo.setSelectedItem( value.toString() );
					
				} else {
					
					combo.setSelectedItem( "No Action" );
					
				}
				
				return combo;
			}

		}
	}

	class TableEditor extends AbstractCellEditor 
			implements TableCellEditor, ItemListener {

		private JTextField name = new JTextField();
		private JComboBox type = new JComboBox( TaskElementFactory.getAvailableTypes() );
		private JComponent lastCompo = null;
		private int lastRow = -1;
		private int lastCol = -1;

		public TableEditor() {
			super();
			type.addItemListener( this );
		}
		
		public Component getTableCellEditorComponent(JTable table,
				Object value, boolean isSelected, int row, int column) {

			this.lastRow = row;
			this.lastCol = column;

			JComponent c = lastCompo;
			
			if ( column == 1 ) {
				if ( value != null )
					name.setText( value.toString() );
				else
					name.setText( null );
				c = name;
			} else
			if ( column == 2 ) {
				if ( value != null )
					type.setSelectedItem( value.toString() );
				else
					type.setSelectedIndex( -1 );	// No selection
				c = type;
			}

			if ( c != null && ( c!= type || !ApplicationModel.isMacOSXPlatform() ) ) {
			
				if ( row % 2 == 0 ) {
					c.setForeground( Color.black );
					c.setBackground( first );
				} else { 
					c.setForeground( Color.black );
					c.setBackground( second );
				}

			}
			
			lastCompo = c;
			
			return lastCompo;
		}

		public void actionPerformed(ActionEvent e) {
		}
		
		public void itemStateChanged(ItemEvent e) {
			if ( lastRow != -1 && 
					lastCol == 2 ) {
				Task t = 
					( Task )( ( TableModel )getModel() ).innerModel.get( lastRow );

				( ( TableModel )getModel() ).setValueAt( type.getSelectedItem(), lastRow, lastCol );
				t = ( Task )( ( TableModel )getModel() ).innerModel.get( lastRow );					
				
				listener.showUI( t );				
			}			
		}
		
		public Object getCellEditorValue() {
			if ( lastCompo == null )
				return null;
			if ( lastCompo instanceof JTextField ) {
				return ( ( JTextField )lastCompo ).getText();
			} else
			if ( lastCompo instanceof JComboBox ) {
				return ( ( JComboBox )lastCompo ).getSelectedItem();
			}
			return null;
		}

		protected void fireEditingStopped() {
			if ( lastRow != -1 && 
					lastCol == 2 ) {
				Task t = 
					( Task )( ( TableModel )getModel() ).innerModel.get( lastRow );
				if ( t != null ) { 
					listener.showUI( t );
				}
			}
			super.fireEditingStopped();
		}
	}

	class TableModel extends AbstractTableModel {
		
		List<Task> innerModel = null;

		public TableModel() {
			int rc = getRowCount();
			innerModel = new ArrayList<Task>( rc );
			for ( int i = 0; i < rc; i++ )
				innerModel.add( null );
		}

		void resetModel( List<Task> model ) {
			this.innerModel = model;
			if ( model.size() < getRowCount() ) {
				for ( int i = model.size(); i < getRowCount(); i++ )
					model.add( null );
			}
			getSelectionModel().removeListSelectionListener( TaskTable.this );
			fireTableDataChanged();
			getSelectionModel().addListSelectionListener( TaskTable.this );
		}

		public int getRowCount() {
			return Preferences.getPreference( "xflows", "maxtask", 30 );
		}
		
		public int getColumnCount() {
			return 3;
		}

		public String getColumnName(int column) {
			if  ( column == 0 )
				return "Order";
			else
			if ( column == 1 )
				return "Name";
			return "Action";
		}

		public Object getValueAt( int rowIndex, int columnIndex ) {
			if  ( columnIndex == 0 ) { 
				return new Integer( rowIndex + 1 );
			} else {
				Task t = ( Task )innerModel.get( rowIndex );
				if ( t != null ) {
					if ( columnIndex == 1 )
						return t.getName();
					if ( columnIndex == 2 )
						return t.getType();
				}
			}
			return null;
		}

		public void setValueAt( Object aValue, int rowIndex, int columnIndex ) {
			Task t = ( Task )innerModel.get( rowIndex );
			if ( t == null ) {
				t = new Task();
				innerModel.set( rowIndex, t );
			}
			if ( columnIndex == 1 ) {
				t.setName( ( String ) aValue );
			} else
			if ( columnIndex == 2 ) {
				t.setType( ( String ) aValue );
				if ( aValue != null && 
						( t.getName() == null || "".equals( t.getName() ) ) ) {
					// Reset a name
					int cpt = 1;
					for ( int i = 0; i < getRowCount(); i++ ) {
						if ( getValueAt( i, 1 ) != null ) {
							if ( aValue.equals( getValueAt( i, 2 ) ) ) {
								cpt++;
							}
						}
					}
					setValueAt( aValue.toString() + "_" + cpt, rowIndex, 1 );
				}
			}
			XFlowsApplicationModel.setModified();			
			fireTableCellUpdated( rowIndex, columnIndex );
		}

		public boolean isCellEditable(int rowIndex, int columnIndex) {
			if ( columnIndex == 0 )
				return false;
			return true;
		}		
	}

}
