package com.japisoft.xflows.task.concat;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import com.japisoft.xflows.task.FilesTaskRunner;
import com.japisoft.xflows.task.TaskContext;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ConcatRunner extends FilesTaskRunner {


	public ConcatRunner() {
		super( new FileConcatRunner(), true );
		fileTarget = true;
		defaultProcessingLog = true;
	}

	public boolean run(TaskContext context) {
		
		if ( !context.hasParam( ConcatUI.ROOTTAG ) ) {
			context.addError( "No root tag" );
			return ERROR;
		}

		String target = context.getParam( ConcatUI.TARGETPATH );
		if ( target == null || 
				"".equals( target ) ) {
			context.addError( "No target file" );
			return ERROR;
		}

		try {

			FileWriter fw = new FileWriter( target );
			( ( FileConcatRunner )monoTask ).setFileWriter( fw );
			
			fw.write( "<?xml version=\"1.0\"?>\n" );
			fw.write( "<");
			fw.write( context.getParam( ConcatUI.ROOTTAG ) );
			fw.write( ">\n" );

			boolean ok = super.run( context );			

			fw.write( "\n</");
			fw.write( context.getParam( ConcatUI.ROOTTAG ) );
			fw.write( ">" );

			try {
				fw.close();
			} catch( IOException exc ) {
				context.addError( "Can't close " + target );
				return ERROR;
			}
			if ( ok == ERROR ) {
				new File( target ).delete();
				return ERROR;
			}
			return OK;
		} catch( IOException exc ) {
			context.addError( "Error while writing on " + target + ":" + exc.getMessage() );
			return ERROR;
		}

	}	

}
