package com.japisoft.xflows.task.docbook;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.border.TitledBorder;

import com.japisoft.framework.ui.text.FileTextField;
import com.japisoft.xflows.task.AbstractTaskUI;
import com.japisoft.xflows.task.FilesTaskRunner;
import com.japisoft.xflows.task.ui.XFlowsFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DocBookUI extends AbstractTaskUI {
	public static final String SOURCEPATH = FilesTaskRunner.SOURCEPATH;
	public static final String SOURCEFILTER = FilesTaskRunner.SOURCEFILTER;
	public static final String OUTPUT_TYPE = "outputType";
	public static final String TARGETPATH = FilesTaskRunner.TARGETPATH;
	public static final String TARGETNAME = FilesTaskRunner.TARGETNAME;

	TitledBorder titledBorder1;
	JLabel lblSource = new JLabel();
	FileTextField tfSourcePath = new FileTextField( null, "xml" );
	JLabel lblSourceFilter = new JLabel();
	JComboBox cbSourceFilter = XFlowsFactory.getSourceFilter( "docbook" );
	JLabel lblXSLTPath = new JLabel();
	JLabel lblTarget = new JLabel();
	FileTextField tfTargetPath = new FileTextField( null, ( String )null );
	JLabel lblTargetName = new JLabel();
	JComboBox cbTargetName = XFlowsFactory.getTargetName( "docbook" );
	GridBagLayout gridBagLayout1 = new GridBagLayout();

	JComboBox cbOutputType = 
			new JComboBox(
				new String[] { 
						"HTML",
						"HTML Help",
						"XHTML",
						"Java Help",
						"FO",
						"PDF",
						"XML",
						"PCL",
						"PS",
						"TXT",
						"SVG"						
					 } );	

	public DocBookUI() {
		super( "docbook" );
		try {
			jbInit();

			tfSourcePath.setName(SOURCEPATH);
			cbSourceFilter.setName(SOURCEFILTER);
			cbOutputType.setName(OUTPUT_TYPE);
			tfTargetPath.setName(TARGETPATH);
			cbTargetName.setName(TARGETNAME);

			tfSourcePath.setDirectoryMode(true);
			tfTargetPath.setDirectoryMode(true);
			tfTargetPath.setOpenedMode( false );
			tfSourcePath.setPreferenceBinding( "xflows", "defaultSourcePath" );
			tfTargetPath.setPreferenceBinding( "xflows", "defaultTargetPath" );

		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}

	public void preSetParams() {
		if ( !params.hasParamValue( OUTPUT_TYPE ) ) {
			params.setParam( OUTPUT_TYPE, "PDF" );
		}
		if ( !params.hasParamValue( TARGETNAME ) ) {
			params.setParam( TARGETNAME, "$1.pdf" );
		}
	}

	void jbInit() throws Exception {
		titledBorder1 = new TitledBorder(
				BorderFactory.createLineBorder(
						new Color( 153, 153, 153 ), 2 ), "DocBook" );
		lblSource.setText("DocBook Source path");
		this.setBorder(titledBorder1);
		this.setLayout(gridBagLayout1);
		tfSourcePath.setText("");
		lblSourceFilter.setText("DocBook Source filter");
		cbSourceFilter.setEditable(true);
		lblXSLTPath.setText("Output type");
		lblTarget.setToolTipText("");
		lblTarget.setText("Target path");
		tfTargetPath.setText("");
		lblTargetName.setText("Target name");
		cbTargetName.setEditable( true );

		this.add( lblSource, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,
						17, 0, 0), 0, 0));
		this.add( tfSourcePath, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(2, 9, 0, 14), 434, 4));
		this.add( lblSourceFilter, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						20, 17, 0, 0), 0, 0));
		this.add( cbSourceFilter, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
				new Insets(16, 9, 0, 14), 406, -1));
		this.add( lblXSLTPath, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						20, 17, 0, 7), 0, 0));
		this.add( cbOutputType, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(16, 9, 0, 14), 434, 4));		
		this.add(lblTarget, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						18, 17, 0, 31), 0, 0));
		this.add(tfTargetPath, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(14, 9, 0, 14), 434, 4));
		this.add(lblTargetName, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						18, 17, 17, 25), 0, 0));
		this.add(cbTargetName, new GridBagConstraints(1, 4, 1, 1, 1.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
				new Insets(14, 9, 17, 14), 406, -1));
	}

}
