package com.japisoft.xflows.task.parsing;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import javax.xml.parsers.SAXParser;

import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;

import com.japisoft.framework.xml.XMLParser;
import com.japisoft.framework.xml.action.CustomEntityResolver;
import com.japisoft.xflows.XFlowsApplicationModel;
import com.japisoft.xflows.task.TaskContext;
import com.japisoft.xflows.task.TaskRunner;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FileParsingRunner implements TaskRunner, ErrorHandler {
	
	private TaskContext context;
	
	static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";

	public boolean run( TaskContext context ) {

		File source = context.getCurrentSourceFile();
		SAXParser parser = null;
		
		try {
			
			boolean schemaFound = false;
			boolean validate = "true".equals( context.getParam(
					ParsingUI.VALIDATING ) );
			
			if ( validate ) {
			
				// Check for schema W3C
				BufferedReader br = new BufferedReader( new FileReader( source ) );
				try {
					String line = null;
					while ( ( line = br.readLine() ) != null ) {
						if ( line.indexOf( "http://www.w3.org/2001/XMLSchema-instance" ) > -1 ) {
							schemaFound = true;
							break;
						} else
						if ( line.indexOf( "<!DOCTYPE " ) > -1 ) {
							schemaFound = false;
							break;
						}
					}
				} finally {
					br.close();
				}

			}

			parser = XMLParser.getSaxParser( validate );		
		
			if ( schemaFound )
				parser.setProperty( JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA );

		} catch( Exception exc ) {
			context.addError( "FATAL PARSING ERROR CAN'T USE THE DEFAULT PARSER" );
			XFlowsApplicationModel.debug( exc );
			return ERROR;
		}

		try {
			InputSource is = new InputSource( source.toString() );
			XMLReader reader = parser.getXMLReader();
			reader.setEntityResolver( CustomEntityResolver.getInstance() );
			reader.setErrorHandler( this );
			this.context = context;			
			reader.parse(is);
		} catch (SAXException exc) {
			return ERROR;
		} catch (IOException exc) {
			context.addError(exc.getMessage());
			return ERROR;
		}

		return state;
	}

	private boolean state = OK;

	public void warning(SAXParseException exception)
			throws org.xml.sax.SAXException {
		context.addWarning(exception.getMessage());
	}

	public void error(SAXParseException exception)
			throws org.xml.sax.SAXException {
		state = ERROR;
		context.addError(exception.getMessage());
	}

	public void fatalError(SAXParseException exception)
			throws org.xml.sax.SAXException {
		state = ERROR;
		context.addError(exception.getMessage());
	}

}
