package com.japisoft.xflows.task.ui.builder;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.japisoft.framework.dockable.InnerWindowProperties;
import com.japisoft.framework.dockable.JDock;
import com.japisoft.framework.dockable.action.ActionModel;
import com.japisoft.framework.ui.InitPanel;
import com.japisoft.xflows.ApplicationComponent;
import com.japisoft.xflows.XFlowsApplicationModel;
import com.japisoft.xflows.task.Task;
import com.japisoft.xflows.task.TaskElementFactory;
import com.japisoft.xflows.task.TaskParams;
import com.japisoft.xflows.task.TaskUI;
import com.japisoft.xflows.task.ui.XFlowsFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ScenarioBuilder extends JDock 
	implements 
		TaskTypeListener,
		ApplicationComponent {

	public ScenarioBuilder() {
		super();
		init();
	}
	
	private TaskUIPanel parameters;
	private TasksPanel panel;

	public List<Task> getTasks() {
		return panel.getTasks();
	}
	
	public void setTasks( List<Task> tasks ) {
		panel.setTasks( tasks );
	}
	
	private void init() {
		setLayout( new BorderLayout() );
		panel = new TasksPanel( this );
		panel.setPreferredSize( new Dimension( 100, 250 ) );

		addInnerWindow(
				new InnerWindowProperties(
				"tasks",
				"Tasks",
				panel ),
				BorderLayout.NORTH );

		addInnerWindow(
				new InnerWindowProperties(
				"parameters",
				"Parameters",
				parameters = new TaskUIPanel() ), 
				BorderLayout.CENTER );

		ActionModel model = getInnerWindowActionsForId( "tasks" );
		model.addAction( ActionModel.SEPARATOR );
		panel.fillActionModel( model );
		
		model = getInnerWindowActionsForId( "parameters" );
		model.addAction( ActionModel.SEPARATOR );
		model.addAction( new CopyAction() );
		model.addAction( paste = new PasteAction() );
		paste.setEnabled( false );		
	}

	private PasteAction paste = null;
	
	public void setApplicationModel( XFlowsApplicationModel model ) {
		panel.table.setTasks( model.getTasks() );
		panel.table.getSelectionModel().setSelectionInterval( 0, 0 );		
	}	

	public void updateCurrentParams() {
		if ( parameters.oldTask != null )
			parameters.oldTask.stop();
	}
	
	public void update() {
		updateCurrentParams();
	}
	
	public void stopEditing() {
		panel.table.stopEditing();
	}
	
	public void showUI( Task t ) {
		parameters.update( t );
	}

	public void print( Printable renderer ) throws PrinterException {
		renderer.print( parameters.getGraphics(), new PageFormat(), 0 );
	}
	
	class TaskUIPanel extends JPanel {
		private JComponent label = null;
		
		private TaskUIPanel() {
			setLayout( new BorderLayout() );

			add( label = 
					new JLabel( 
						"No parameters. Please select a task with an action" 
			) );
		}

		private TaskUI oldTask = null;

		public void update( Task t ) {
			if ( t == null || t.getType() == null ) {
				if ( !( getComponentCount() == 1 && 
						getComponent( 0 ) == label ) ) {
					removeAll();
					add( label );
					invalidate();
					validate();
					repaint();					
				} 
			} else
			if ( t.getType() != null ) {
				try {
					TaskUI ui = TaskElementFactory.getUIForType( t.getType() );
					if ( oldTask != null )
						oldTask.stop();
					removeAll();
					add( ui.getView() );
					invalidate();
					validate();
					repaint();
					ui.start();
					ui.setParams( t.getParams() );
					oldTask = ui;
				} catch( Throwable th ) {
					XFlowsApplicationModel.debug( th );
					XFlowsFactory.buildAndShowErrorDialog( "XFlows has met an error using this task " + th.getMessage() );
				}
			}
		}
	}

	private TaskParams currentCopy = null;

	/** Copy the parameters */
	class CopyAction extends AbstractAction {
		public CopyAction() {
			putValue( Action.SHORT_DESCRIPTION, "Copy the parameters" );
			putValue( Action.SMALL_ICON,  XFlowsFactory.getImageIcon( "images/copy.png" ) );
		}
		
		public void actionPerformed( ActionEvent e ) {
			Task t = panel.table.getCurrentTask();
			if ( t != null ) {
				currentCopy = t.getParams().cloneParams();
				paste.setEnabled( true );
			}
		}
	}

	/** Paste the parameters */
	class PasteAction extends AbstractAction {
		public PasteAction() {
			putValue( Action.SHORT_DESCRIPTION, "Paste the parameters" );
			putValue( Action.SMALL_ICON,  XFlowsFactory.getImageIcon( "images/paste.png" ) );			
		}

		public void actionPerformed( ActionEvent e ) {
			if ( currentCopy != null ) {
				Task t = panel.table.getCurrentTask();
				if ( t != null ) {
					t.setParams( currentCopy );
					panel.table.valueChanged( null );
				}
			}
		}
	}
}
