package com.japisoft.xflows.task.ui.builder;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.japisoft.framework.dockable.action.ActionModel;
import com.japisoft.xflows.task.Task;
import com.japisoft.xflows.task.TaskTable;
import com.japisoft.xflows.task.ui.XFlowsFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TasksPanel extends JPanel {

	public TasksPanel( TaskTypeListener listener ) {
		super();
		init( listener );
	}

	TaskTable table;
	private TaskTypeListener listener;
	
	private void init( TaskTypeListener listener ) {
		setLayout( new BorderLayout() );
		add( new JScrollPane( 
				table = new TaskTable( this.listener = listener, true ) ), 
				BorderLayout.CENTER );
		table.setRowHeight( table.getRowHeight() + 5 );
	}

	public List<Task> getTasks() {
		return table.getTasks();
	}
	
	public void setTasks( List<Task> tasks ) {
		table.setTasks( tasks );
	}

	void fillActionModel( ActionModel model ) {
		model.addAction( new RunAction() );
		model.addAction( ActionModel.SEPARATOR );
		model.addAction( new DeleteAction() );
		model.addAction( ActionModel.SEPARATOR );		
		model.addAction( new DownAction() );
		model.addAction( new UpAction() );
	}

	//////////////////////////////////////////////////////////
	
	class UpAction extends AbstractAction {
		public UpAction() {
			super();
			putValue( Action.SMALL_ICON, 
					new ImageIcon( ClassLoader.getSystemResource( "images/element_up.png" )  ) );
			putValue( Action.SHORT_DESCRIPTION, "Move the task up" );
		}
		public void actionPerformed( ActionEvent e ) {
			table.upTask();
		}
	}

	class DownAction extends AbstractAction {
		public DownAction() {
			super();
			putValue( Action.SMALL_ICON, 
					new ImageIcon( ClassLoader.getSystemResource( "images/element_down.png" )  ) );
			putValue( Action.SHORT_DESCRIPTION, "Move the task down" );
		}
		public void actionPerformed( ActionEvent e ) {
			table.downTask();
		}
	}

	class DeleteAction extends AbstractAction {
		public DeleteAction() {
			super();
			putValue( Action.SMALL_ICON, 
					new ImageIcon( ClassLoader.getSystemResource( "images/element_delete.png" )  ) );
			putValue( Action.SHORT_DESCRIPTION, "Delete the selected task" );
		}
		public void actionPerformed( ActionEvent e ) {
			table.deleteTask();
		}
	}

	class RunAction extends AbstractAction {
		public RunAction() {
			super();
			putValue( Action.SMALL_ICON, 
					XFlowsFactory.getImageIcon( "images/element_run.png" ) );
			putValue( Action.SHORT_DESCRIPTION, "Run the task" );
		}

		public void actionPerformed( ActionEvent e ) {
			listener.update();
			table.runTask();
		}
	}

}

