package com.japisoft.xflows.task.ui.runner;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JScrollPane;

import com.japisoft.framework.dockable.InnerWindowProperties;
import com.japisoft.framework.dockable.JDock;
import com.japisoft.framework.dockable.action.ActionModel;
import com.japisoft.xflows.ApplicationComponent;
import com.japisoft.xflows.XFlowsApplicationModel;

import com.japisoft.xflows.task.Task;
import com.japisoft.xflows.task.TaskLogTable;
import com.japisoft.xflows.task.TaskManager;
import com.japisoft.xflows.task.TaskRunnerListener;
import com.japisoft.xflows.task.ui.XFlowsFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ScenarioRunner extends JDock 
		implements 
			ApplicationComponent, 
			RunnerAction,
			TaskRunnerListener {

	private TasksPanel panel;
	private TaskLogTable log;
	
	public ScenarioRunner() {
		setLayout( new BorderLayout() );
		panel = new TasksPanel( this );
		panel.setPreferredSize( new Dimension( 100, 300 ) );
		
		addInnerWindow(
				new InnerWindowProperties(
				"tasks",
				"Scenario",
				panel ),
				BorderLayout.NORTH );

		addInnerWindow(
				new InnerWindowProperties(
				"logs",
				"Logs",
				new JScrollPane( log = new TaskLogTable() ) ), 
				BorderLayout.CENTER );
		
		ActionModel model = getInnerWindowActionsForId( "tasks" );
		model.addAction( ActionModel.SEPARATOR );
		panel.fillActionModel( model );

		model = getInnerWindowActionsForId( "logs" );
		model.addAction( ActionModel.SEPARATOR );
		model.addAction(
				new CleanLog() );
		
		TaskManager.setTaskRunnerListener( this );
	}

	public void run(Task t) {
		String name = t.getName();
		panel.table.selectTaskForName( name );
	}	
	
	public void setApplicationModel( XFlowsApplicationModel model ) {
		panel.table.setTasks( model.getTasks() );
	}

	public void stopEditing() {
	}	
	
	public void run() {
		log.clean();
		ArrayList taskList = XFlowsApplicationModel.ACCESSOR.getTasks();
		TaskManager.run( taskList, TaskManager.BACKGROUND );
	}	

	public void stop() {
		TaskManager.stopCurrentTask();
	}

	class CleanLog extends AbstractAction {
		public CleanLog() {
			putValue( Action.SHORT_DESCRIPTION, "Clean the logs" );
			putValue( Action.SMALL_ICON, XFlowsFactory.getImageIcon( "images/document_dirty.png" ) );
		}

		public void actionPerformed( ActionEvent e ) {
			log.clean();
		}
	}
	
}
