package com.japisoft.xflows.task.zip;

import java.io.File;
import java.util.List;

import com.japisoft.datasource.DataSource;
import com.japisoft.datasource.DataSourceContainer;
import com.japisoft.datasource.DataSourceContainerProxy;
import com.japisoft.datasource.DataSourceItem;
import com.japisoft.datasource.DataSourceManager;
import com.japisoft.framework.toolkit.FileToolkit;
import com.japisoft.xflows.task.TaskContext;
import com.japisoft.xflows.task.TaskRunner;
import com.japisoft.xflows.task.copy.CopyUI;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class UnzipRunner implements TaskRunner {

	@Override
	public boolean run(TaskContext context) {
		
		if ( !context.hasParam( UnzipUI.SOURCEPATH ) ) {
			context.addError("No source path found");
			return ERROR;
		}

		if ( !context.hasParam( UnzipUI.TARGETPATH ) ) {
			context.addError("No target path found");
			return ERROR;
		}
		
		String source = context.getParam( UnzipUI.SOURCEPATH );
		String target = context.getParam( UnzipUI.TARGETPATH );
		
		if ( source.equalsIgnoreCase( target ) ) {
			context.addError( "You target must be different from your source" );
			return ERROR;
		}
		
		DataSourceContainer dsc = DataSourceManager.getDataSourceContainer( source );
		
		if ( dsc == null )
			return ERROR;
		
		if ( context.hasParam( CopyUI.SOURCEFILTER ) ) {
			dsc = new DataSourceContainerProxy( dsc, context
					.getParam(CopyUI.SOURCEFILTER));
		}

		try {
			List<DataSource> l = dsc.list();
			
			for (int i = 0; i < l.size(); i++) {

				DataSource ds = l.get( i );

				if ( ds instanceof DataSourceItem ) {
					// Unzip it
					
					DataSourceItem item = ( DataSourceItem )ds;
					unzip( item.getPath(), target );
					
				}
			}
			
			
		} catch( Exception exc ) {
			context.addError( "Can't unzip " + exc.getMessage() );
		}
		
		
		return OK;
	}

	private void unzip( String file, String target ) throws Exception {
		
		File f = new File( file );
		File realTarget = new File( target, f.getName() );
		realTarget.mkdirs();
		FileToolkit.unzip( f, realTarget );
		
	}
	
}
