package com.japisoft.xflows.task.zip;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.ui.text.FileTextField;
import com.japisoft.xflows.task.AbstractTaskUI;
import com.japisoft.xflows.task.FilesTaskRunner;
import com.japisoft.xflows.task.ui.XFlowsFactory;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class UnzipUI extends AbstractTaskUI implements ActionListener {

	public static final String SOURCEPATH = FilesTaskRunner.SOURCEPATH;
	public static final String SOURCEFILTER = FilesTaskRunner.SOURCEFILTER;
	public static final String TARGETPATH = FilesTaskRunner.TARGETPATH;
	
	public UnzipUI() {
		super( "unzip" );
		initComponents();
		tfSourcePath.setName( SOURCEPATH );
		cbSourceFilter.setName( SOURCEFILTER );
		tfTargetPath.setName( TARGETPATH );

		tfSourcePath.setDirectoryMode( true );
		tfTargetPath.setDirectoryMode( true );
		tfSourcePath.setFileMode( false );
		tfTargetPath.setFileMode( false );
		tfTargetPath.setOpenedMode( false );
	}

	public void addNotify() {
		super.addNotify();
		tfSourcePath.addActionListener( this );
		tfTargetPath.addActionListener( this );
	}
	
	public void removeNotify() {
		super.removeNotify();
		tfSourcePath.removeActionListener( this );
		tfTargetPath.removeActionListener( this );
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		String source = tfSourcePath.getText();
		String target = tfTargetPath.getText();
		if ( source != null && source.equalsIgnoreCase( target ) ) {
			EditixFactory.buildAndShowErrorDialog( "Your source and target must be different !" );
		}
	}
	
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">                          
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        // tfSourcePath = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        // cbSourceFilter = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        // tfTargetPath = new javax.swing.JTextField();

        jLabel1.setText("Source path");

        jLabel2.setText("Source filter");

        jLabel3.setText("Target path");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addComponent(jLabel3)
                    .addComponent(jLabel1))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(cbSourceFilter, javax.swing.GroupLayout.DEFAULT_SIZE, 694, Short.MAX_VALUE)
                    .addComponent(tfTargetPath)
                    .addComponent(tfSourcePath, javax.swing.GroupLayout.Alignment.TRAILING))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(24, 24, 24)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(tfSourcePath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(cbSourceFilter, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel3)
                    .addComponent(tfTargetPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(31, Short.MAX_VALUE))
        );
    }// </editor-fold>                        


    // Variables declaration - do not modify                     
    private JComboBox cbSourceFilter = XFlowsFactory.getSourceFilter( "zip" );
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private FileTextField tfSourcePath = new FileTextField( ( String )null, ( String )null );
    private FileTextField tfTargetPath = new FileTextField( null, ( String )null );
    // End of variables declaration        
    
}
