package com.japisoft.xmlform.component;

import java.awt.Component;
import java.awt.Container;
import java.awt.Rectangle;

import javax.swing.JViewport;

import com.japisoft.xmlform.component.container.GridComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DynamicLayout {

	public static int BORDER_Y = 4;
	
	public static void remove( 
			Container parent,
			AbstractXMLFormComponent child ) {

		// Remove from DOM
		AbstractXMLFormComponent xmlParent = child.getXMLFormComponentParent();
		xmlParent.getDOM().removeChild( child.getDOM() );
		
		int limitY = child.getY();
		int deltaY = child.getHeight();

		// Remove physically
		parent.remove( child );
		
		unpushSiblingComponent( parent, limitY, deltaY );
		
		parent.invalidate();
		parent.validate();
	}
	
	private static void unpushSiblingComponent(
			Container parent,
			int limitY,
			int deltaY ) {

		for ( int i = 0; i < parent.getComponentCount(); i++ ) {
			Component c = 
				parent.getComponent( i );		
			if ( c.getY() > limitY ) {
				Rectangle r = c.getBounds();
				r.translate( 0, -deltaY );
				c.setBounds( r );
			}
		}			

		if ( parent instanceof GridComponent ) {
			parent = parent.getParent();
		}

		Rectangle r = parent.getBounds();
		r.height -= deltaY;
		parent.setBounds( r );

		if ( parent.getParent() instanceof XMLFormComponent ) {
			parent.invalidate();
			parent.validate();
			
			unpushSiblingComponent( parent.getParent(), r.y, deltaY );
		} else {
			
			// Force scrollPane resize
			if ( parent.getParent() instanceof JViewport ) {
				parent.setPreferredSize( parent.getSize() );
				parent.getParent().getParent().invalidate();
				parent.getParent().getParent().validate();
			}
			
			
		}
	}

	// ----------------------------------------------------------------------------------
	
	public static void add( 
			Container parent,
			AbstractXMLFormComponent child,
			AbstractXMLFormComponent siblingChild ) {

		// Insert it under the child
		
		Rectangle r = child.getBounds();		
		
		Rectangle r2 = siblingChild.getBounds();
		r2.y = r.y + r.height + BORDER_Y;
		
		parent.add( siblingChild );

		siblingChild.setBounds( r2 );

		pushSiblingComponent( 
				parent, 
				siblingChild, 
				siblingChild.getY(), 
				siblingChild.getHeight() + BORDER_Y );

		parent.invalidate();
		parent.validate();

	}

	private static void pushSiblingComponent(
			Container parent,
			Component newChild,
			int limitY,
			int deltaY ) {

		int maxY = 0;

		for ( int i = 0; i < parent.getComponentCount(); i++ ) {
			Component c = 
				parent.getComponent( i );
			if ( c != newChild ) {
				if ( c.getY() >= limitY - 2 * BORDER_Y ) {
					Rectangle r = c.getBounds();
					r.translate( 0, deltaY + BORDER_Y );
					c.setBounds( r );
				}
			}
			maxY = Math.max( 
					c.getY() + c.getHeight(), 
					maxY );
		}

		if ( maxY > parent.getHeight() ) {
			// Resize the parent too
			
			int delta = 
				( maxY - parent.getHeight() );

			if ( parent instanceof GridComponent ) {
				parent = parent.getParent();
			}

			Rectangle r = parent.getBounds();
			r.height += delta + BORDER_Y;
			parent.setBounds( r );

			if ( parent.getParent() instanceof XMLFormComponent ) {
				parent.invalidate();
				parent.validate();
				
				// Check this parent again
				pushSiblingComponent( 
						parent.getParent(), 
						parent, 
						parent.getY(), 
						delta + BORDER_Y );
			}

			// Force scrollPane resize
			if ( parent.getParent() instanceof JViewport ) {
				parent.setPreferredSize( 
						parent.getSize() );
				parent.getParent().getParent().invalidate();
				parent.getParent().getParent().validate();
			}

		}

	}

}
