package com.japisoft.xmlform.component;

import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.util.HashMap;

import javax.swing.plaf.ColorUIResource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.japisoft.xmlform.component.container.XMLFormContainer;
import com.japisoft.xmlform.designer.data.GrammarNodeTreeNode;
import com.japisoft.xmlform.editor.EditorModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLDeserizalizer {

	public static AbstractXMLFormComponent build( 
			Document doc, 
			boolean editing,
			GrammarNodeTreeNode parentNode,
			HashMap<String,AbstractXMLFormComponent> components,
			ComponentContext context ) throws Exception {
		Element root = 
			doc.getDocumentElement();
		
		if ( root.hasAttribute( "spell") ) {
			EditorModel.SPELL_CHECK = root.getAttribute( "spell" );
		} else
			EditorModel.SPELL_CHECK = null;

		NodeList nl = 
			root.getElementsByTagName( "fields" );
		if ( nl.getLength() == 0 )
			return null;
		Element fields = 
			( Element )nl.item( 0 );

		AbstractXMLFormComponent result = null;
		
		NodeList children = fields.getChildNodes();
		for ( int i = 0; i < children.getLength(); i++ ) {
			if ( children.item( i ) instanceof Element ) {
				Element e = 
					( Element )children.item( i );
				if ( "field".equals( 
						e.getNodeName() ) ) {
					result = build( 
							e, 
							editing, 
							parentNode, 
							components, 
							context, 
							true );
					break;
				}
			}
		}

		if ( result == null )
			throw new Exception( 
				"Invalid format, can't find fields" );

		if ( root.hasAttribute( "schema" ) ) {
			String schemaURI = root.getAttribute( "schema" );
			if ( result instanceof XMLFormContainer ) {
				( ( XMLFormContainer )result ).setSchemaURI( schemaURI );
			}
		}

		return result;

	}

	public static AbstractXMLFormComponent build( 
			Element field, 
			boolean editing,
			GrammarNodeTreeNode parentNode,
			HashMap<String,AbstractXMLFormComponent> components,
			ComponentContext context,
			boolean firstComponent ) throws Exception {

		String cl = field.getAttribute( "class" );
		// Read properties
		HashMap<String,Object> properties = 
			getProperties( field );
		AbstractXMLFormComponent component = 
			context.getComponentFactory().newComponent( 
					cl,
					properties,
					editing,
					context );

		component.setReferenceComponent( true );
		
		if ( components != null )
			components.put( 
					component.getId(), 
					component );

		// Resolve parentNode
		if ( ( parentNode != null ) && 
				( component.getXpath() != null ) ) {
			GrammarNodeTreeNode childNode = parentNode.resolveRelativeXPath(
					component.getXpath() );
			if ( childNode != null ) {
				component.setGrammarNode( childNode );
			}
			parentNode = childNode;
		}

		component.setFieldDescription( field );
		component.setTopComponent( firstComponent );
		// Check for inner children
		NodeList children = field.getChildNodes();
		for ( int i = 0; i < children.getLength(); i++ ) {
			Node n = children.item( i );
			if ( n instanceof Element ) {
				Element e = ( Element )n;
				if ( "field".equals( 
						e.getNodeName() ) ) {
					component.add(
							build( 
								e, 
								editing, 
								parentNode, 
								components,
								context, 
								false ) );
				}
			}
		}

		return component;
	}

	public static boolean isPropertyManaged( Object value ) {
		if ( value instanceof String )
			return true;
		if ( value instanceof Rectangle )
			return true;
		if ( value instanceof Color )
			return true;
		if ( value instanceof ColorUIResource ) 
			return true;
		if ( value instanceof Boolean )
			return true;
		if ( value instanceof Integer )
			return true;
		if ( value instanceof Font )
			return true;
		if ( value instanceof String[] )
			return true;
		if ( value instanceof HashMap )
			return true;
		return false;
	}

	private static HashMap<String,Object> getProperties( Element field ) throws Exception {
		Element properties = null;
		NodeList children = field.getChildNodes();
		for ( int i = 0; i < children.getLength(); i++ ) {
			if ( children.item( i ) instanceof Element ) {
				Element e = ( Element )children.item( i );
				if ( "properties".equals( e.getNodeName() ) ) {
					properties = e;
					break;
				}
			}
		}
		if ( properties == null )
			throw new Exception( "Invalid format, can't find properties" );
		children = 
			properties.getElementsByTagName( "property" );
		HashMap<String,Object> res = new HashMap<String,Object>();
		for ( int i = 0; i < children.getLength(); i++ ) {
			Element p = ( Element )children.item( i );
			String cl = p.getAttribute( "class" );
			String v = p.getAttribute( "value" );
			String n = p.getAttribute( "name" );
			
			if ( "java.lang.String".equals( cl ) ) {
				res.put( n, v );
			} else
			if ( "java.lang.Boolean".equals( cl ) ) {
				res.put( n, new Boolean( v ) );
			} else
			if ( "java.lang.Integer".equals( cl ) ) {
				res.put( n, new Integer( v ) );
			}
			if ( "java.awt.Rectangle".equals( cl ) ) {
				String[] r = v.split( "," );
				res.put( n, new Rectangle( 
						Integer.parseInt( r[ 0 ] ),
						Integer.parseInt( r[ 1 ] ),
						Integer.parseInt( r[ 2 ] ),
						Integer.parseInt( r[ 3 ] ) ) );
			} else
			if ( "java.lang.Boolean".equals( cl ) ) {
				res.put( n, "true".equals( v ) );
			} else
			if ( "java.awt.Color".equals( cl ) || 
					"javax.swing.plaf.ColorUIResource".equals( cl ) ) {
				String[] r = v.split( "," );
				res.put( n, new Color( 
						Integer.parseInt( r[ 0 ] ),
						Integer.parseInt( r[ 1 ] ),
						Integer.parseInt( r[ 2 ] ) ) );
			} else
			if ( "java.awt.Font".equals( cl ) ) {
				String[] r = v.split( "," );
				res.put( n, new Font( 
						r[ 0 ], 
						Integer.parseInt( r[ 1 ] ), 
						Integer.parseInt( r[ 2 ] ) ) );
			} else
			if( "[Ljava.lang.String;".equals( cl ) ) {
				String[] r = v.split( "~~" );
				res.put( n, r );
			} else
			if ( "java.util.HashMap".equals( cl ) ){
				String[] r = v.split( "~~" );
				HashMap<String,String> map = new HashMap<String, String>();
				for ( int j = 0; j < r.length; j += 2 ) {
					map.put( r[ j ], r[ j + 1 ] );
				}
				res.put( n, map );
			}
		}
		return res;
	}
	
}
