package com.japisoft.xmlform.component.editable;

import java.awt.Color;
import java.util.ArrayList;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlform.component.AbstractXMLFormComponent;
import com.japisoft.xmlform.component.ComponentContext;
import com.japisoft.xmlform.designer.properties.PropertyDescriptor;
import com.japisoft.xmlform.designer.properties.PropertyDescriptorImpl;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLEditableComponent extends AbstractXMLFormComponent {

	public XMLEditableComponent( 
			boolean designMode, 
			ComponentContext context ) {
		super( designMode, context );
	}	
	
	protected String tooltip = null;
	
	public void setTooltip( String tooltip ) {
		firePropertyChange( "tooltip", this.tooltip, tooltip );
		this.tooltip = tooltip;
	}

	public String getTooltip() {
		return tooltip;
	}

	@Override
	public void setMinOccurs( int minOccurs ) {
		super.setMinOccurs(minOccurs);
// Musn't force if the user doesn't want it
//		if ( minOccurs > 0 )
//			setRequired( true );
	}

	@Override
	protected void prepareProperties( 
			ArrayList<PropertyDescriptor> l ) throws Exception {
		super.prepareProperties(l);
		l.add(
			new PropertyDescriptorImpl(
					"tooltip",
					String.class,
					this ) );
	}

	public Color getColorForRequiredField() {
		return Preferences.getPreference( 
				"editor", 
				"requiredField", 
				new Color( 246,191,191 ) );
	}
	
	protected Node domText = null;

	private Text getFirstTextNode( Node parentNode ) {
		if ( parentNode.hasChildNodes() ) {
			if ( parentNode.getFirstChild() instanceof Text ) {
				return ( Text )parentNode.getFirstChild();
			} else
				return getFirstTextNode( parentNode.getFirstChild() );
		} else
			return null;
	}
	
	protected Node getDOMText() {
		if ( domText == null ) {
			Node n = getDOM();
			
			if ( !( n instanceof Text ) ) {
			
				if ( n instanceof Attr ) {
					this.domText = n;
				} else {
					domText = getFirstTextNode( n );
					if ( domText == null ) {
						domText = context.getDocument().createTextNode( "" );
						domText.setUserData( "ui", this, null );
						n.appendChild( domText );
					}
				}

			} else {
				
				domText = n;
				
			}
		}
		
		checkNodeIsBound( getDOM(), this );
		return domText;
	}

	protected void deleteDOMText() {
		if ( domText != null ) {			
			Node parent = domText.getParentNode();
			
			if ( domText instanceof Attr ) 
				parent = ( ( Attr )domText ).getOwnerElement();

			if ( parent != null ) {
				if ( domText instanceof Attr ) {
					( ( Element )parent ).removeAttributeNode( ( Attr )domText );
				} else
					parent.removeChild( domText );
			}
			domText = null;
		}
	}
	
}
