package com.japisoft.xmlform.designer.data;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;

import javax.swing.tree.TreeNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AbstractTreeNode implements TreeNode {

	private ArrayList<TreeNode> children = null;
	
	public Enumeration children() {
		if ( children != null )
			return Collections.enumeration( children );
		return null;
	}
	
	public void addChild( TreeNode node ) {
		if ( children == null )
			children = new ArrayList<TreeNode>();
		children.add( node );
	}
	
	public TreeNode getNextSibling() {
		if ( parent == null )
			return null;
		int index = parent.getIndex( this );
		if ( index == -1 )	// ?
			return null;
		if ( index < parent.getChildCount() - 1 ) {
			return parent.getChildAt( index + 1 );
		} else
			return null;	// The last one
	}

	public boolean getAllowsChildren() {
		return true;
	}

	public TreeNode getChildAt(int childIndex) {
		if ( children == null )
			return null;
		return children.get( childIndex );
	}

	public int getChildCount() {
		if ( children == null )
			return 0;
		return children.size();
	}

	public int getIndex(TreeNode node) {
		if ( children == null )
			return -1;
		return children.indexOf( node );
	}

	protected TreeNode parent = null;
	
	public TreeNode getParent() {
		return parent;
	}

	public boolean isLeaf() {
		return ( getChildCount() == 0 );
	}

	private Object userObj = null;
	
	public void setUserObject( Object obj ) {
		userObj = obj;
	}
	
	public Object getUserObject() {
		return userObj;
	}

}
