package com.japisoft.xmlform.designer.data;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTree;

import com.japisoft.framework.xml.grammar.GrammarAttribute;
import com.japisoft.framework.xml.grammar.GrammarContainer;
import com.japisoft.framework.xml.grammar.GrammarElement;
import com.japisoft.framework.xml.grammar.GrammarNode;
import com.japisoft.framework.xml.grammar.GrammarText;
import com.japisoft.framework.xml.grammar.GrammarType;
import com.japisoft.framework.xml.grammar.xsd.XSDGrammar;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class GrammarNodeTreeNode extends AbstractTreeNode {

	private GrammarNode node = null;

	public GrammarNodeTreeNode( 
		ArrayList<GrammarNode> processed, 
		GrammarNode node ) {
		this.node = node;		
		processed.add( node );
		GrammarType type = node.getType();
		List<GrammarNode> nodes = type.getValues();
		if ( nodes != null && 
				!( node instanceof GrammarText ) && 
					!(node instanceof GrammarAttribute ) ) {
			for ( GrammarNode gn : nodes ) {
				if ( processed.contains( gn ) )
					continue;
				GrammarNodeTreeNode gntn = new GrammarNodeTreeNode( processed, gn );
				gntn.parent = this;
				addChild( gntn );
			}
		}	
	}

	@Override
	public String toString() {
		return node.getName();
	}
	
	public GrammarNode getSource() {
		return node;
	}

	public GrammarNodeTreeNode resolveRelativeXPath( String xpath ) {
		if ( xpath.startsWith( "/" ) ) {
			xpath = xpath.substring( 1 );
			// Don't resolve the root element
			return this;
		}

		if ( xpath.startsWith( "@" ) ) {
			String attributeName = xpath.substring( 1 );
			for ( int i = 0; i < getChildCount(); i++ ) {
				GrammarNodeTreeNode child = 
					( GrammarNodeTreeNode )getChildAt( i );
				if ( child.node instanceof GrammarAttribute ) {
					if ( attributeName.equals( child.node.getName() ) )
						return child;
				}
			}
		} else {
			int i = xpath.lastIndexOf( "/" );
			if ( i > -1 )
				xpath = xpath.substring( 0, i );
			
			for ( i = 0; i < getChildCount(); i++ ) {

				GrammarNodeTreeNode child = 
					( GrammarNodeTreeNode )getChildAt( i );
				
				if ( child.node instanceof GrammarContainer ) {
					// Explore the container
					GrammarNodeTreeNode result = 
						child.resolveRelativeXPath( xpath );
					if ( result != null )
						return result;
				} else
				if ( child.node instanceof GrammarElement ) {
					if ( xpath.equals( 
							child.node.getName() ) )
						return child;
				} else
				if ( child.node instanceof GrammarText ) {
					if ( "text()".equals( xpath ) )
							return child;
				}
				
			}

		}

		return null;
	}

	public String getId() { 
		return toXPath();
	}

	public String toXPath() {

		if ( node instanceof GrammarContainer ) {

			return ( ( GrammarNodeTreeNode )getParent() ).toXPath();

		} else {

			StringBuffer sb = new StringBuffer();
			if ( node instanceof GrammarAttribute ) {
				sb.append( "@" ).append( toString() );
			} else {

				if ( node.getType().getValues() == null ) {	// Simple type

					if ( "#text".equals( toString() ) ) {

					} else {
						sb.append( toString() );
						sb.append( "/" );
					}

					sb.append( "text()" );
				} else
					sb.append( toString() );

			}

			sb.insert( 0, "/" );

			if ( parent instanceof GrammarNodeTreeNode ) {
				sb.insert(
						0,
						( ( GrammarNodeTreeNode )parent ).toXPath() );
			}

			return sb.toString();

		}
	}
	
	public static void main( String[] args ) throws Exception {
		
		String p = "c:/travail/soft/japisoft-editix-2014/distrib/install-content/samples/xmlform/purchaseOrder.xsd";
		XSDGrammar g = new XSDGrammar( p );
		GrammarNodeTreeNode gntn = new GrammarNodeTreeNode( new ArrayList<GrammarNode>(), g.getGlobalElement( "purchaseOrder" ) );
		JFrame f = new JFrame();
		JTree t = new JTree( gntn );
		f.add( new JScrollPane( t ) );
		f.setVisible( true );
		
	}

}
