package com.japisoft.xmlform.editor.actions.xml;

import java.awt.event.ActionEvent;
import java.util.List;
import javax.swing.AbstractAction;

import org.w3c.dom.Node;

import com.japisoft.framework.internationalization.Traductor;
import com.japisoft.xmlform.UIToolkit;
import com.japisoft.xmlform.component.AbstractXMLFormComponent;
import com.japisoft.xmlform.editor.EditorFrame;
import com.japisoft.xmlform.editor.EditorModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CheckAction extends AbstractAction {

	public void actionPerformed( ActionEvent e ) {
		
		EditorFrame frame = EditorModel.getEditorFrame();
		
		if ( frame.validateDocument() ) {

			List<Node> warning = 
				frame.checkEmptyFields();
			
			if ( warning.size() > 0 ) {
				
				Node first = warning.get( 0 );
				AbstractXMLFormComponent afc = 
					getFirstComponent( first );
				if ( afc != null ) {
					frame.setFocusTo( afc );
					UIToolkit.warn( 
							Traductor.traduce( 
								"empty",
								"This field is empty or contains whitespace characters" ) );
				} else 
					UIToolkit.info( 
							Traductor.traduce( 
									"dok",
									"Your document is correct" ) );

			} else {

				UIToolkit.info( 
						Traductor.traduce(
								"dok",
								"Your document is correct" ) );

			}

		}

	}

	private AbstractXMLFormComponent getFirstComponent( Node n ) {
		
		if ( n == null )
			return null;
		
		AbstractXMLFormComponent c = 
			( AbstractXMLFormComponent )n.getUserData( "ui" );
	
		if ( c != null )
			return c;
		
		return getFirstComponent( n.getParentNode() );

	}

}
