package com.japisoft.xmlpad;

import javax.swing.JPopupMenu;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.JTree;

import com.japisoft.xmlpad.editor.XMLEditor;
import com.japisoft.xmlpad.elementview.ElementView;
import com.japisoft.xmlpad.error.ErrorView;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface UIAccessibility {

	/** @return the current XMLEditor. Note that depending the splitting state of the <code>XMLContainer</code> this is not always the same instance */
	public XMLEditor getEditor();

	/** Show or hide the location tree. This method has no effect
	 * if no tree is available => <code>XMLPadProperties.setProperty("tree", "false" )</code>
	 * @param treeVisible Show or hide the current location tree */
	public void setTreeVisible(boolean treeVisible);
	
	/** @return <code>true</code> if there's a current location tree and the splitpane bar is not closed */
 	public boolean isTreeVisible();

	/** Decide to have a left tree with the XML document content or not. By default <code>true</code> */
	public void setTreeAvailable(boolean treeAvailable);

	/** @return <code>true</code> if the tree is available */
	public boolean isTreeAvailable(); 

	/** Decide to have a toolbar at the top of the tree or not. By default <code>false</code> */
	public void setTreeToolBarAvailable( boolean treeToolBarAvailable );

	/** @return <code>true</code> if the tree is available */
	public boolean isTreeToolBarAvailable(); 
	
	/** @return the current XML tree view */	
	public JTree getTree(); 

	/** This is a way to share a tree which is outside the XMLContainer. This is only useful if the
	 * <code>setTreeAvailable( false )</code> method is called */	
	public void setTreeDelegate(JTree treeDelegate);
		
	/** @return the current tree menu popup */
	public JPopupMenu getCurrentTreePopup();

	/** @return the main toolbar */
	public JToolBar getToolBar();

	/** @return the tree toolBar or <code>null</code> */
	public JToolBar getTreeToolBar();
	
	/** Show or hide the default toolbar. This method has no effect
	 * if the default toolbar is not available. Note that this method
	 * is only for dynamic usage, it has no effect before the visibility
	 * of the <code>XMLContainer</code>. If you wish no toolbar, you must
	 * call <code>setToolBarAvailable( false )</code> */
	public void setToolBarVisible(boolean toolbarVisible);

	/** @return the toolbar state */
	public boolean isToolBarVisible();

	/** Create a default toolbar. By default <code>true</code>. If user has
	 * an external toolbar, this property must be set to <code>false</code>. It is
	 * possible to control the visibility by calling <code>setToolBarVisible( ... )</code> */
	public void setToolBarAvailable(boolean toolBarAvailable);

	/** @return <code>true</code> if a default toolbar is available */
	public boolean isToolBarAvailable();

	/** Create a default popup. By default <code>true</code> */
	public void setPopupAvailable(boolean popupAvailable);

	/** @return <code>true</code> if a default popup is available */
	public boolean isPopupAvailable();

	/** Reset the tree popup. By default to <code>true</code>. This code has no effect if no tree
	 * is used.
	 */
	public void setTreePopupAvailable(boolean treePopupAvailable);

	/** @return <code>true</code> if a tree exists and if the tree popup is available */
	public boolean isTreePopupAvailable();

	/** @return the main splitpane dividing the tree and the source. It may be <code>null</code> depending on the moment you call it */
	public JSplitPane getSplitPane();

	/** @return the secondary splitpane dividing the tree and the element view. It may be <code>null</code> depending on the moment you call it */
	public JSplitPane getSecondarySplitPane();

	/** Reset the initial divider location between tree and text. This value is between 0 to 1
	 * @param location Location between 0 and 1 / 0.5 means at the middle
	 * */
	public void setInitialTreeTextDividerLocation(double location);

	/** Reset the initial divider location between the tree and the element view. 
	 * This value is between 0 to 1
	 * @param location Location between 0 and 1 / 0.5 means at the middle
	 */
	public void setInitialElementViewDividerLocation(double location);

	/** Define a new error view panel. It must be called before showing the first error */
	public void setErrorView( ErrorView view );

	/** @return the current error view panel */
	public ErrorView getErrorView();

	/**
	 * Set a view fo visualizing an XML element under the tree. if view is
	 * <code>null</code> no view will be available */	
	public void setElementView( ElementView view );

	/** @return the current view for visualizing an element */
	public ElementView getElementView();	

	/** This is only for inner usage, it mustn't be called by the user */
	void dispose();

	/** Particular cas for DragNDrop from the tree */
	public void setEnableDragNDropForRoot( boolean enabled );
	
	/** Particular cas for DragNDrop from the tree */
	public boolean isEnabledDragNDropForRoot();

}
