package com.japisoft.xmlpad.action.edit;

import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;

import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.XMLAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CopyNodeAction extends XMLAction {

	public static final String ID = CopyNodeAction.class.getName();

	protected boolean autoRequestFocus() { return false; }

	protected Point getNodeOffset() {
		FPNode n = ( FPNode )container.getCurrentNode();
		return getNodeOffset( n );
	}

	public static Point getNodeOffset( FPNode n ) {
		if ( n == null )
			return null;
		int start = n.getStartingOffset();
		int stop = n.getStoppingOffset();
		if ( n.isTag() )
			stop++;
		else
			if ( n.isText() )
				stop--;
		return new Point( start, stop );		
	}

	public static boolean copyAction( XMLContainer container, FPNode node ) {
		Point n = getNodeOffset( node );
		if (n == null) {
			JOptionPane.showMessageDialog( container.getView(), "No Node Found", "Error", JOptionPane.ERROR_MESSAGE );
		}

		container.getEditor().requestFocus();

		try {
			String content = container.getDocument().getText( n.x, n.y - n.x + 1 );
			
			Clipboard systemClipboard =
				Toolkit
					.getDefaultToolkit()
					.getSystemClipboard();
			Transferable transferableText =
				new StringSelection( content );
			systemClipboard.setContents(
				transferableText,
				null );
			
		} catch( Exception e ) {
			return INVALID_ACTION;
		}
		
		return VALID_ACTION;
	}
	
	public boolean notifyAction() {
		
		if ( container.getTreeListeners() == null )
			return INVALID_ACTION;
		if ( container.getTree() == null )
			return INVALID_ACTION;
		
		return copyAction( 
			container, 
			container.getCurrentNode() 
		);
		
	}

}
