package com.japisoft.xmlpad.action.file;

import javax.swing.*;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.xmlpad.Debug;
import com.japisoft.xmlpad.action.ActionModel;
import com.japisoft.xmlpad.action.XMLAction;
import com.japisoft.xmlpad.toolkit.XMLToolkit;

import java.io.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SaveAction extends XMLAction {

	public static final String ID = SaveAction.class.getName();
	
	public static String ENCODING_PROPERTY = "encoding";
	
	public SaveAction() {
		super();
	}

	String currentEncoding = null;

	public void setProperty(String propertyName, Object value) {
		if ( ENCODING_PROPERTY.equals( propertyName ) ) {
			currentEncoding = value.toString();
		}
	}

	/** Save the current document at the location */
	protected boolean saveDocument(String location) {
		if (container.getDocumentIntegrity().isParseBeforeSaving()) {
			// Error detected
			if (!ActionModel.activeActionByName(ActionModel.PARSE_ACTION))
				return false;
		}

		String txt = editor.getText();
		
		try {
			Reader r = null;
			r = new StringReader( txt );

			BufferedReader br = new BufferedReader( r );
			try {
				Writer fw = null;
				if ( "DEFAULT".equals( currentEncoding ) ) 
					fw = new FileWriter( location );
				else {
					if ( "AUTOMATIC".equals( currentEncoding ) || 
							currentEncoding == null ) {						
						currentEncoding = XMLToolkit.getXMLEncoding( txt, currentEncoding );
						if ( currentEncoding == null || "AUTOMATIC".equals( currentEncoding ) )
							currentEncoding = "UTF-8";
						container.getDocumentInfo().setEncoding( currentEncoding );
					} else
						currentEncoding = container.getDocumentInfo().getEncoding();
					
					// Search for processing instruction for skiping XML encoding
					int i = txt.indexOf( "<?encoding " );
					if ( i > 0 ) {
						int j = txt.indexOf( "?>", i );
						if ( j > 0 ) {
							currentEncoding = txt.substring( i + 11, j ).trim();
						}
					}
					
					if ( currentEncoding != null ) {
						try {
							fw = new OutputStreamWriter(
									new FileOutputStream( location ), currentEncoding );
						} catch( UnsupportedEncodingException e ) {
							fw = new FileWriter( location );
						}
					} else
						fw = new FileWriter( location );
				}

				Debug.debug( "Write with " + currentEncoding );

				BufferedWriter writer =
					new BufferedWriter( fw );
				try {
					for (String line = null; ( line = br.readLine() ) != null; ) {
						writer.write( line );
						writer.newLine();
					}
				} finally {
					writer.close();
				}
			} finally {
				br.close();
			}
			
			container.getDocumentInfo().setCurrentDocumentModifiedDate(
					new File( location ).lastModified() );			
			container.setModifiedState( false );
			
		} catch (Throwable th) {
			JOptionPane.showMessageDialog(
				editor,
				"Can't save to " + container.getCurrentDocumentLocation());
			return INVALID_ACTION;
		}
		return VALID_ACTION;
	}

	public boolean notifyAction() {
		if (container.getCurrentDocumentLocation() == null) {
			com.japisoft.framework.application.descriptor.ActionModel.activeActionById( "saveAs", null );
			return false;
		} else {
			return saveDocument(container.getCurrentDocumentLocation());
		}
	}
}
