package com.japisoft.xmlpad.bookmark;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Shape;

import javax.swing.Icon;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import javax.swing.text.Highlighter.HighlightPainter;

import com.japisoft.framework.toolkit.FileToolkit;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DefaultBookmarkContext implements BookmarkContext {

	private Icon icon;
	private Color lineColor;

	/**
	 * @param icon
	 *            A small icon for each bookmark
	 * @param Color
	 *            lineColor color for each bookmarked line
	 */
	public DefaultBookmarkContext(Icon icon, Color lineColor) {
		this.icon = icon;
		this.lineColor = lineColor;
	}

	public Icon getIcon() {
		return icon;
	}

	public Color getColor() {
		return lineColor;
	}
	
	BookmarkModel model = null;

	public BookmarkModel getModel() {
		if (model == null)
			model = new DefaultBookmarkModel();
		return model;
	}

	/** @return <code>true</code> if this line (starting from 0) match one bookmark */
	public boolean matchLine( 
			String uri, 
			int line, 
			XMLContainer container ) {
		if ( model == null )
			return false;
		for ( int i = 0; i < model.getBookmarkCount(); i++ ) {
			BookmarkPosition pos = model.getBookmarkPositionAt( i );
			int offset = pos.getOffset();
			XMLContainer container2 = container;
			if ( pos.getSource() instanceof XMLContainer ) {
				container2 = ( XMLContainer )pos.getSource();
				if ( !FileToolkit.sameFileName( 
						container2.getCurrentDocumentLocation(), uri ) )
					continue;
			}
			XMLPadDocument document = 
				container2.getXMLDocument();
			int line2 = 
				document.getDefaultRootElement().getElementIndex( offset );
			if ( line == line2 )
				return true;
		}
		return false;
	}
	
	private DefaultHighlighter defaultHighlight;
	
	public HighlightPainter getHighlightPainter() {
		if ( defaultHighlight == null )
			defaultHighlight = new DefaultHighlighter();
		return defaultHighlight;
	}
	
	class DefaultHighlighter implements HighlightPainter {

		public void paint(Graphics g, int p0, int p1, Shape bounds,
				JTextComponent textComponent) {

			FontMetrics metrics = g.getFontMetrics();
			Document doc = textComponent.getDocument();
			int lineNo = doc.getDefaultRootElement().getElementIndex(p0);

			Rectangle rect = (Rectangle) bounds;
			int height = metrics.getHeight();
			int x = rect.x;
			int y = rect.y + height * lineNo;
			int width = textComponent.getWidth();

			g.setColor(lineColor);
			g.fillRect(x, y, width, height);

		}

	}

}
