package com.japisoft.xmlpad.dialog;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;

import javax.swing.Icon;
import javax.swing.JComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DialogManager {

	public static final int OK = 0;
	public static final int CANCEL = 1;

	static DialogManagerDelegate DELEGATE = null;
	
	public static void setDialogManagerDelegate( DialogManagerDelegate manager ) {
		DELEGATE = manager;
	}

	/**
	 * @param owner dialog or frame parent
	 * @param dialogTitle The title of the dialog
	 * @param title The main title
	 * @param comment The user comment
	 * @param icon An icon
	 * @param pane The user UI element
	 * @return an Action Id like OKAction.ID
	 */
	public static int showDialog( 
			Window owner, 
			String dialogTitle, 
			String title, 
			String comment, 
			Icon icon, 
			JComponent pane ) {

		if ( DELEGATE != null )
			return DELEGATE.showDialog(
					owner,
					dialogTitle,
					title,
					comment,
					icon, 
					pane );
		
		OkCancelDialog dialog = null;
		
		if ( owner instanceof Frame ) {

			dialog = new OkCancelDialog(
					(Frame)owner,
					dialogTitle,
					title,
					comment );
			
		} else
		if ( owner instanceof Dialog ) {

			dialog = new OkCancelDialog(
					(Dialog)owner,
					dialogTitle,
					title,
					comment );

		} else {

			dialog = new OkCancelDialog(
					(Frame)null,
					dialogTitle,
					title,
					comment );
						
		}

		dialog.getContentPane().add( pane );
		dialog.pack();
		dialog.setVisible( true );
		if ( dialog.isOk() )
			return OK;
		return CANCEL;
		
	}	
	
}
 
