package com.japisoft.xmlpad.editor;

import javax.swing.text.*;
import com.japisoft.xmlpad.SharedProperties;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLEditorKit extends DefaultEditorKit implements ViewFactory {
	private ViewPainterListener listener;

	public XMLEditorKit(ViewPainterListener listener ) {
		this.listener = listener;
	}

	/**
	 * View factory for text element */
	public ViewFactory getViewFactory() {
		return this;
	}

	private boolean syntaxColor = true;

	/** Reset it to support XML syntax color. By default <code>true</code> */
	public void setSyntaxColor( boolean syntaxColor ) {
		this.syntaxColor = syntaxColor;
		if ( !syntaxColor && lastView != null )
			lastView.setSyntaxColor( false );
	}
	
	private boolean dtdMode = false;
	
	public void setDTDMode( boolean dtdMode ) {
		this.dtdMode = dtdMode;
		if ( dtdMode && lastView != null )
			lastView.setDTDMode( true );
	}

	public boolean wrappedMode = false;

	/** Wrapped the line automatically if <code>true</code> */
	public void setWrappedMode( boolean wrappedMode ) {
		this.wrappedMode = wrappedMode;
	}

	XMLViewable lastView;
	
	private boolean displaySpace = false;
	
	public void setDisplaySpace( boolean displaySpace ) {
		this.displaySpace = displaySpace;
		if ( lastView != null && lastView instanceof XMLTextView ) {
			( ( XMLTextView )lastView ).setDisplaySpace( displaySpace );
		}
	}

	public boolean isDisplaySpace() {
		return displaySpace;
	}

	/**
	 * @param elem element to draw. Call only once ! */
	public View create(Element elem) {		
		if ( syntaxColor ) {			
			if ( SharedProperties.WRAPPED_LINE ) {
				lastView = new WrappedXMLView( elem ); 
			} else {
				if ( SharedProperties.FULL_TEXT_VIEW ) {
					lastView = new XMLTextView( elem, displaySpace );
					( ( XMLTextView )lastView ).setViewPainterListener( listener );
				} else {
					lastView = new XMLView( elem );
					( ( XMLView )lastView ).setViewPainterListener( listener );
				}
			}

			return ( View )lastView;
		}
		else
			return ( View )( lastView = new XMLView( elem ) );
	}
	

	/**
	 * @return the default document */
	public Document createDefaultDocument() {
		return new XMLPadDocument( null );
	}

	public String getContentType() {
		return "text/plain";
	}

}
