package com.japisoft.xmlpad.helper.handler.relaxng;

import java.util.Enumeration;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.walker.TreeWalker;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.schema.AbstractTagHandler;
import com.japisoft.xmlpad.helper.model.SchemaNodable;
import com.japisoft.xmlpad.helper.model.SchemaNode;
import com.japisoft.xmlpad.helper.model.TagDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RelaxNGTagHelperHandler extends AbstractTagHandler implements
		SchemaNodable {

	private FPNode root;

	public RelaxNGTagHelperHandler( FPNode root ) {
		this.root = root;
	}

	public void dispose() {
		super.dispose();
		this.root = null;
	}	

	public TagDescriptor getTag( FPNode node ) {
		FPNode xmlnode = getXMLElementByName( node.getContent() );
		if ( xmlnode != null ) {
			
			RelaxNGToSchemaNode rng = new RelaxNGToSchemaNode();			
			TagDescriptor td = new TagDescriptor( node.getContent(), false );
			rng.processContentAndAttributes( td, xmlnode );
			
			return td;
			
		}
		return null;
	}

	protected void notifyLocation() {
		schemaNode = null;
		
		if ( currentDocumentNode == null ) {
			schemaNode = ( new RelaxNGToSchemaNode() ).getSchemaNode( root );
		} else {
			FPNode element = getXMLElementByName( currentDocumentNode.getContent() );		
			if ( element != null ) {
				schemaNode = ( new RelaxNGToSchemaNode() ).getSchemaNode( element );
			}
		}
	}

	private FPNode getXMLElementByName( String name ) {
		TreeWalker walker = new TreeWalker( root );
		Enumeration enume = walker.getTagNodeByName( "element", true );
		FPNode node;
		
		while ( enume.hasMoreElements() ) {
			String name2 = RelaxNGToSchemaNode.getName( node = ( FPNode )enume.nextElement() );
			if ( name.equals( name2 ) )
				return node;
		}

		return null;
	}

	private SchemaNode schemaNode = null;

	public SchemaNode getSchemaNode() {
		return schemaNode;
	}

	public SchemaNode getSecondarySchemaNode() {
		return null;
	}	

}
