package com.japisoft.xmlpad.helper.handler.schema;

import javax.swing.text.BadLocationException;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.SchemaHelperManager;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.AttDescriptor;
import com.japisoft.xmlpad.helper.model.AttValueDescriptor;
import com.japisoft.xmlpad.helper.model.TagDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AttributeValueHandler extends AbstractHelperHandler {
	
	private AbstractTagHandler tagHandler;
	
	public AttributeValueHandler( AbstractTagHandler handler ) {
		this.tagHandler = handler;
	}
	
	public void dispose() {
		super.dispose();
		this.tagHandler = null;
	}	

	public String getTitle() {
		return tagHandler.getTitle();
	}

	public int getPriority() {
		return 1;
	}	
	
	protected String getActivatorSequence() {
		return "\"";
	}	

	public String getName() {
		return SchemaHelperManager.SCHEMA_ATTRIBUTE_VALUES;
	}	

	public boolean haveDescriptors(
			FPNode currentNode, 
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset, 
			String activatorString ) {
		
		if ( ( "\"".equals( activatorString ) || 
				( "'".equals( activatorString ) )
				) && 
					document.isInsideTag( 
							offset,
							true,
							true )
		) {
			return true;
		} else
			if ( activatorString == null ) {
				if ( document.isInsideAttributeValue( 
						offset ) )
					return true;
			}
		return false;
	}

	protected void installDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			int offset, 
			String activatorString ) {
/*		SimpleNode currentNode = 
			document.getContainer().getCurrentElementNode(); */
		if ( currentNode == null )
			return;	// Wrong state?
		TagDescriptor td = 
			tagHandler.getTag( currentNode );
		if ( td == null )
			return;
		AttDescriptor[] atts = td.getAtts();
		if ( atts != null ) {
			String currentAttribute = document.getForwardAttributeName( 
					document, offset );
			if ( currentAttribute == null )
				return;
			for ( int i = 0; i < atts.length; i++ ) {
				AttDescriptor ad = atts[ i ];
				if ( ad.getName().equals( currentAttribute ) ) {
					String[] enums = ad.getEnumValues();
					if ( enums != null ) {
						for ( int j = 0; j < enums.length; j++ ) {
							AttValueDescriptor 
								avd = new AttValueDescriptor( enums[ j ], 
										activatorString != null ? 
												activatorString.charAt( 0 ) : '"' );
							avd.setAddedPart( activatorString );
							avd.setSource( this );
							addDescriptor( avd );
						}
					}
					break;
				}
			}
		}
	}

	public boolean hasDelegateForInsertingResult() {
		return true;
	}	
	
	public void insertResult(
			XMLPadDocument document, 
			int offset, 
			String result ) {
		int[] attDel = document.getAttributeValueLocation( offset );
		if ( attDel == null )
			document.insertStringWithoutHelper( offset, result, null );
		else {
			try {
				document.replace(
						attDel[ 0 ],
						attDel[ 1 ] - attDel[ 0 ] + 1,
						result,
						null );
			} catch (BadLocationException e) {
			}
		}
	}

}
