package com.japisoft.xmlpad.helper.model;

import java.util.Vector;

import javax.swing.DefaultListModel;

import com.japisoft.framework.xml.parser.node.FPNode;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class AbstractEntityHelper extends AbstractHelper implements EntityHelper {

	private Vector entities = null;
		
	public AbstractEntityHelper() {
		initDefaultEntities();
	}
	
	public String getTitle() {
		return "Entities";
	}
	
	public boolean hasTitle() {
		return true;
	}
	
	protected boolean hasElements() {
		return ( entities != null ) && ( entities.size() > 0 );
	}

	protected void fillList( FPNode node, DefaultListModel model ) {
		if ( entities == null )
			return;
		for ( int i = 0; i < entities.size(); i++ )
			model.addElement( entities.get( i ) );
	}

	/** Reset gt,lt,quot,apos entities */
	protected void initDefaultEntities() {
		addEntity( "gt", ">" );
		addEntity( "lt", "<" );
		addEntity( "quot", "\"" );
		addEntity( "amp", "&" );
		addEntity( "apos", "\'" );		
		addEntity( "#10", "Line break" );
	}

	/** Add a new entry */
	public void addEntityDescriptor( EntityDescriptor entity ) {
		if ( entities == null )
			entities = new Vector();
		entities.add( entity );
	}

	public void addEntity( String name, String value ) {
		addEntityDescriptor( new EntityDescriptor( name, value ) );
		cache = null;
	}

	private EntityDescriptor[] cache = null;

	/* (non-Javadoc)
	 * @see com.japisoft.xmlpad.helper.EntityHelper#getEntities()
	 */
	public EntityDescriptor[] getEntities() {
		if ( entities == null )
			return null;
		if ( cache != null )
			return cache;
		EntityDescriptor[] d = new EntityDescriptor[ entities.size() ];
		entities.copyInto( d );
		cache = d;
		return d;
	}
	

	protected String getLostCharacter() {
		return "&";
	}	
}
