package com.japisoft.xmlpad.helper.model;

import java.util.ArrayList;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AttDescriptor extends AbstractDescriptor {
	private String name;
	private String defValue;
	private boolean required = true;
	private ArrayList enumValues;

	public AttDescriptor( String name, String defValue ) {
		this.name = name;
		this.defValue = defValue;
	}

	public AttDescriptor( String name, String defValue, boolean required ) {
		this( name, defValue );
		this.required = required;
	}

	public boolean isRequired() {
		return required;
	}

	public void setRequired( boolean required ) {
		this.required = required;
	}

	private boolean enabled = true;

	public void setEnabled( boolean enabled ) {
		this.enabled = enabled;
	}

	public boolean isEnabled() {
		return enabled;
	}

	public String getName() {
		return name;
	}

	public String getNameForHelper() {
		return getName();
	}

	public String getDefaultValue() {
		if ( defValue == null )
			return "";
		return defValue;
	}

	public void setDefaultValue( String def ) {
		this.defValue = def;
	}
	
	public void addEnumValue( String enumItem ) {
		if ( enumItem != null ) {
			if (enumValues == null)
				enumValues = new ArrayList();
			enumValues.add( enumItem );
		}
	}

	public String[] getEnumValues() {
		if (enumValues == null)
			return null;
		String[] res = new String[enumValues.size()];
		for (int i = 0; i < enumValues.size(); i++)
			res[i] = (String) enumValues.get(i);
		return res;
	}

	public boolean hasEnumValues() {
		return enumValues != null;
	}

	public boolean hasAutomaticNextHelper() {
		return hasEnumValues() || super.hasAutomaticNextHelper();
	}	
	
	public boolean isRaw() {
		return false;
	}

	public String toString() {
		return name + (required ? " (required) " : "");
	}

	public String toExternalForm() {
		if ( addedPart == null )
			addedPart = "";
		return addedPart + 
			getName() + "=\"" + 
				getDefaultValue() + "\"";
	}

}
