package com.japisoft.xmlpad.helper.model;

import javax.swing.DefaultListModel;
import javax.swing.text.BadLocationException;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AttributeValuesHelper extends AbstractHelper {

	private AttDescriptor descriptor;
	private char delimiter; 
	
	public AttributeValuesHelper( AttDescriptor descriptor, char delimiter ) {
		this.descriptor = descriptor;
		this.delimiter = delimiter;
	}

	public String getTitle() {
		return descriptor.getName();
	}

	protected boolean hasElements() {
		return ( descriptor != null && descriptor.hasEnumValues() );
	}
	
	protected void prepareDocumentBeforeInserting( XMLPadDocument document, int offset ) {
		try {
			// Search for another delimiter without a '='
			boolean firstDelimiter = false;
			boolean secondDelimiter = false;
			int location = 0;
			for ( int i = ( offset ); i < document.getLength(); i++ ) {
				char ch = document.getText( i, 1 ).charAt( 0 );
				if ( ch == delimiter ) {
					if ( !firstDelimiter ) {
						firstDelimiter = true;
						location = i;
					} else {
						secondDelimiter = true;
						location = i;
						break;
					}
				}
				if ( ch == '>' )
					break;
				if ( ch == '\n' )
					break;
				if ( ch == '=' ) {
					break;
				}
			}
			
			if ( firstDelimiter || 
					secondDelimiter ) {
				
				document.remove( 
						offset, 
						location - offset + 1 );
			}

		} catch (BadLocationException e) {
		}
	}
	
	protected void fillList( FPNode node, DefaultListModel model ) {
		String[] values = descriptor.getEnumValues();
		if ( values != null ) {
			for ( int i = 0; i < values.length; i++ ) {
				model.addElement( new AttValueDescriptor( values[ i ], delimiter ) );
			}
		}
	}
	
}
