package com.japisoft.xmlpad.helper.model;

import java.awt.Component;
import java.awt.Font;

import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.UIManager;

import com.japisoft.framework.app.toolkit.Toolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CommonDescriptorRenderer extends DefaultListCellRenderer {

	private Icon defIcon;

	private CommonDescriptorRenderer() {
		super();
		defIcon = UIManager.getIcon("xmlpad.helper.icon");
		if ( defIcon == null )
			defIcon = Toolkit.getImageIcon( "images/element.png" );
	}

	private static CommonDescriptorRenderer RENDERER = null;

	public static CommonDescriptorRenderer getRenderer() {
		if ( RENDERER == null )
			RENDERER = new CommonDescriptorRenderer();
		return RENDERER;
	}

	Font f = null;

	public Component getListCellRendererComponent(
		JList list,
		Object value,
		int index,
		boolean isSelected,
		boolean cellHasFocus) {
		JLabel c =
			( JLabel )super.getListCellRendererComponent(
				list,
				value,
				index,
				isSelected,
				cellHasFocus );

		Icon icon = null;		
		
		if ( ( value != null ) && ( value instanceof Descriptor ) ) {
			Descriptor d = ( Descriptor )value;
			c.setEnabled( d.isEnabled() );
			c.setText( d.getNameForHelper() );
			if ( d.getColor() != null )
				c.setForeground( 
						d.getColor() );
			else {
				if ( isSelected )
					c.setForeground( 
							list.getSelectionForeground() );
				else
					c.setForeground( 
							list.getForeground() );
			}
			icon = d.getIcon();
			if ( icon == null )
				icon = defIcon;
		}

		if (f == null)
			c.setFont(
				f = new Font( 
					getFont().getName(), 
					0, 
					getFont().getSize() ) );
		else
			c.setFont( f );

		if ( icon != null )
			c.setIcon( icon );

		return c;
	}

}
