package com.japisoft.xmlpad.nodeeditor;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.dialog.DialogManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DefaultEditor implements Editor {

	/** Accept all text node : by checking <code>isText</code> */
	public boolean accept(FPNode node) {
		return ( node.isText() );
	}

	/*
	 * Edit a text node by showing a Text dialog */
	public void edit(EditorContext context) {
		XMLContainer container = context.getXMLContainer();

		TextPane pane = new TextPane();
		pane.setText( context.getEditedText() );
		pane.setPreferredSize( new Dimension( 300, 300 ) );

		if ( DialogManager.showDialog(
				SwingUtilities.getWindowAncestor( container.getView() ),
				"Edit", 
				"Update", 
				"Edit the current selected text node", 
				null,
				pane ) == DialogManager.OK ) {
			context.setResult( pane.getText() );
		}
	}

	//////////////////////////////////////////

	class TextPane extends JPanel {
		private JTextArea text;
		
		public TextPane() {
			init();
		}
		
		void init() {
			setLayout( new BorderLayout() );
			add( new JScrollPane( text = new JTextArea() ) );
		}

		public void addNotify() {
			super.addNotify();
			text.requestFocus();
		}

		public void setText( String content ) {
			// Replace '&amp;', '&lt;', '&gt;', '&apos;'
			content = content.replaceAll( "&lt;", "<" );
			content = content.replaceAll( "&gt;", ">" );
			text.setText( content );
		}

		public String getText() {
			String content = text.getText();
			content = content.replaceAll( "<", "&lt;" );
			content = content.replaceAll( ">", "&gt;" );
			return content;
		}
	}

}
