package com.japisoft.xmlpad.tree;

import java.util.ArrayList;

import javax.swing.JTree;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.tools.XMLToolkit;
import com.japisoft.xmlpad.Debug;
import com.japisoft.xmlpad.error.ErrorListener;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
class TreeErrorListener implements ErrorListener {

	private JTree tree;
	
	TreeErrorListener( JTree tree ) {
		if ( tree == null ) {
			throw new RuntimeException( "No tree found. Illegal usage for class TreeErrorListener" );
		}
		this.tree = tree;
	}

	public void initErrorProcessing() {}

	public void stopErrorProcessing() {
		if ( tree.getModel() == null ||	
				tree.getModel().getRoot() == null ) {	// Skip wrong tree state
			savedErrors = null;
			return;	
		}
		if ( savedErrors != null ) {
			for ( int i = 0; i < savedErrors.size(); i+= 2 ) {
				int offset = ( ( Integer )savedErrors.get( i ) ).intValue();
				String message = ( String )savedErrors.get( i + 1 );
				addErrorOnTheTree( offset, message );
			}
			savedErrors = null;
			tree.repaint();
		}
	}

	private ArrayList savedErrors = null;

	public void notifyError(Object context,boolean localError, String sourceLocation,
			int line, int col, int offset, String message, boolean onTheFly) {
		if ( localError ) {
			if ( savedErrors == null )
				savedErrors = new ArrayList();
			savedErrors.add( new Integer( offset ) );
			savedErrors.add( message );
		}
	}

	public void notifyNoError(boolean onTheFly) {
		savedErrors = null;
	}

	private void addErrorOnTheTree(
			int offset,
			String message ) {
		// Get the XML document
		FPNode node = ( FPNode )tree.getModel().getRoot();
		if ( node != null ) {
			try {
				FPNode targetNode = ( FPNode )XMLToolkit.getNodeForOffset( node.getDocument(), offset );
				while ( targetNode != null ) {
					if ( targetNode.errorMessage == null )
						targetNode.errorMessage = message;
					else
						targetNode.errorMessage = "\n" + message;
					targetNode = ( FPNode )targetNode.getFPParent();
				}
			} catch (RuntimeException e) {
				Debug.debug( e );
			}
		}
	}

}
