package com.japisoft.xmlpad.tree.action;

import javax.swing.tree.TreePath;

import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.TreeAction;
import com.japisoft.xmlpad.action.XMLAction;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class AbstractTreeAction extends XMLAction implements TreeAction {

	public AbstractTreeAction() {
		setPopable(false);
		setToolbarable(false);
	}

	public AbstractTreeAction(String referenceAction) {
		super(referenceAction);
		setPopable(false);
		setToolbarable(false);
	}

	/** This override controls if the container has a tree */
	public void setXMLContainer(XMLContainer container) {
		this.container = container;
		if (container != null && container.getTree() != null)
			notifyXMLContainer();
	}

	public boolean notifyAction() {
		if ( container.getTree() == null )
			return XMLAction.INVALID_ACTION;
		if ( container.getTree().getSelectionPath() == null )
			return XMLAction.INVALID_ACTION;
		if ( container.getTree().getSelectionPath() != null )
			actionForSelection( container.getTree().getSelectionPath() );
		else
			return XMLAction.INVALID_ACTION;
		return XMLAction.VALID_ACTION;
	}
	
	protected void actionForSelection( TreePath tp ) {}
	
	protected boolean treeToolBar = true;

	/** <code>true</code> if this action could appear inside the tree toolbar. By default <code>true</code> */
	public void setTreeToolBarable( boolean treeToolBar ) {
		this.treeToolBar = treeToolBar;
	}

	/** @return <code>true</code> if this action could be shown in the tree toolBar */
	public boolean isTreeToolBarable() {
		return treeToolBar;
	}

	protected boolean treePopable = true;

	/** <code>true</code> if this action is available in the tree popup. By default <code>true</code> */
	public void setTreePopable( boolean treePopable ) {
		this.treePopable = treePopable;
	}

	/** @return <code>true</code> if this action is visible in the tree popup */
	public boolean isTreePopable() {
		return treePopable;
	}

}
