package com.japisoft.xmlpad.xml.validator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLPadSAXParserFactory {

	static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
	static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
	
	static XMLPadSAXParserFactory DELEGATE = new XMLPadSAXParserFactory();

	/** This is a way to override this factory by creating a subclass */
	public static void setDelegate( XMLPadSAXParserFactory factory ) {
		DELEGATE = factory;
	}

	/** @return a new SAX Parser. It uses JAXP as default */
	public static SAXParser getNewSAXParser( boolean validating )
			throws ParserConfigurationException, SAXException {
		//System.out.println( DELEGATE );
		return DELEGATE.getNonStaticNewSAXParser( validating );
	}

	/** @return a new DocumentBuilder. This is used for formatting the document */
	public static DocumentBuilder getNewDocumentBuilder( boolean validating, boolean schemaAware )
			throws ParserConfigurationException {
		return DELEGATE.getNonStaticNewDocumentBuilder( validating, schemaAware );
	}
	
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	
	protected DocumentBuilder getNonStaticNewDocumentBuilder( boolean validating, boolean schemaAware ) 
			throws ParserConfigurationException {
		DocumentBuilderFactory factory = getDocumentBuilderFactory(); 
		factory.setValidating( validating );
		factory.setNamespaceAware( true );
		factory.setIgnoringElementContentWhitespace( true );
		if ( schemaAware ) {
			factory.setAttribute(
				JAXP_SCHEMA_LANGUAGE,
				W3C_XML_SCHEMA 
			);
		}
		return factory.newDocumentBuilder();
	}

	protected DocumentBuilderFactory getDocumentBuilderFactory() {
		return DocumentBuilderFactory.newInstance();
	}

	protected SAXParser getNonStaticNewSAXParser( boolean validating )
			throws ParserConfigurationException, SAXException {
		SAXParserFactory parserFactory = SAXParserFactory.newInstance();
		parserFactory.setValidating( validating );
		parserFactory.setNamespaceAware(true);
		
		if ( "true".equals( System.getProperty( "xmlpad.debug" ) ) )
			System.out.println( "Parsing with " + parserFactory );
		
		return parserFactory.newSAXParser();
	}
}
