package com.japisoft.xpath;

import java.util.Enumeration;
import java.util.Hashtable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class AbstractKit implements XPathKit {

	/** Compute the node ID value. Only for compatibility with the 1.1
	 * This method will throw a RuntimeException with version > 1.1
	 *  @deprecated */
	public String getId(Object node) {
		throw new RuntimeException("Supported only with 1.1 version");
	}

	private Hashtable htFeatures = null;

	/** Used by the descendant for specifying a supported feature */
	protected void addFeature(String feature, boolean enable) {
		if (htFeatures == null)
			htFeatures = new Hashtable();
		htFeatures.put(feature.toLowerCase(), new Boolean(enable));
	}

	/** Set a feature support for the current kit. A RuntimeException
	 * should be thrown by the kit that doesn't support such feature */
	public void setFeature(String feature, boolean enable) {
		if (htFeatures == null)
			throw new RuntimeException("Unknown feature " + feature);
		if (!htFeatures.containsKey(feature.toLowerCase()))
			throw new RuntimeException("Unknown feature " + feature);
		htFeatures.put(feature.toLowerCase(), new Boolean(enable));
	}

	/** @return true if the feature is supported by the kit */
	public boolean isFeatureSupported(String feature) {
		if (htFeatures == null)
			return false;
		if (!htFeatures.containsKey(feature.toLowerCase()))
			return false;
		return ((Boolean) htFeatures.get(feature.toLowerCase())).booleanValue();
	}

	/** Return a list of supported features */
	public String[] getSupportedFeatures() {
		if (htFeatures == null)
			return new String[] {
		};
		String[] s = new String[htFeatures.size()];
		int i = 0;
		for (Enumeration enume = htFeatures.keys(); enume.hasMoreElements();) {
			s[i++] = (String) enume.nextElement();
		}
		return s;
	}

	/** Particular case where the provided node is not valid for Xpath. This is often then case for
	 * Document objet rather than Node object */
	public Object getBetterReferenceNode( Object refNode ) {
		return refNode;
	}
}
