// (c) ALEXANDRE BRILLANT : http://www.japisoft.com
// All this work is confidential, you have rights to
// change and evolve it for your products but you
// have no rights to sell it, propose concurrent works.
// Morever any changes to bugs or evolutions should
// be send to JAPISOFT that needs to maintain a
// valid version and has all rights on the product.

package com.japisoft.xpath.kit;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xpath.function.basic.FunctionLib;
import com.japisoft.xpath.function.Lib;
import com.japisoft.xpath.navigator.FastParserNavigator;
import com.japisoft.xpath.AbstractKit;
import com.japisoft.xpath.NodeSet;
import com.japisoft.xpath.Navigator;


/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FastParserKit
    extends AbstractKit {

  public FastParserKit() {
    super();
  }

  /** @return the library resolver. If <code>null</code> is returned then the standard library is used */
  public Lib getLibrary() {
    return new FunctionLib();
  }

  /** @return the tree navigator toolkit */
  public Navigator getNavigator() {
    return new FastParserNavigator();
  }

  // Particular method for the string-value on node
  private void findSubTextNode(FPNode n, StringBuffer res) {
    for (int i = 0; i < n.childCount(); i++) {
      FPNode child = n.childAt(i);
      if (child.isText()) {
        res.append(child.getContent());
      }
    }
    for (int i = 0; i < n.childCount(); i++) {
      FPNode child = n.childAt(i);
      if (child.isTag() && !child.isLeaf()) {
        findSubTextNode(child, res);
      }
    }
  }

  /** Compute the string-value for this node */
  public String getStringValue(Object node) {
    if (node instanceof FPNode) {
      FPNode n = (FPNode) node;
      // Cas of the attribute
      if (n.getType() == (FPNode.COMMENT_NODE + 1)) {
        FPNode p = n.getFPParent();
        return p.getAttribute(n.getContent());
      }
      else
      if ( (n.getType() == FPNode.TEXT_NODE) ||
          (n.getType() == FPNode.COMMENT_NODE)) {
        return n.getContent();
      }
      // Particular cas for node : get all sub text node
      StringBuffer sb = new StringBuffer();
      findSubTextNode(n, sb);
      return sb.toString();
    }
    else
    if (node instanceof NodeSet) {
      if ( ( (NodeSet) node).size() > 0) {
        return getStringValue( ( (NodeSet) node).elementAt(0));
      }
    }
    return node.toString();
  }

  private void buildNodeSet(FPNode node, NodeSet ns) {
    ns.addNode(node);
    for (int i = 0; i < node.childCount(); i++) {
      FPNode n1 = node.childAt(i);
      if ( (n1.childCount() > 0) &&
          (n1.isTag())) {
        buildNodeSet(n1, ns);
      }
    }
  }

  /** Compute the local name of the node */
  public String getLocalName(Object node) {
    FPNode n = (FPNode) node;
    return n.getContent();
  }

  /** Compute the namespace URI for this node */
  public String getNamespaceURI(Object node) {
    FPNode n = (FPNode) node;
    return n.getNameSpaceURI();
  }

  /** Compute the qualified name for this node */
  public String getName(Object node) {
    FPNode n = (FPNode) node;
    if (n.getNameSpacePrefix() != null) {
      return n.getNameSpacePrefix() + ":" + n.getContent();
    }
    return getLocalName(node);
  }

  /** Compute the language for this node */
  public String getLang(Object node) {
    FPNode n = (FPNode) node;
    if ( n == null )
    	return null;
    String s = n.getAttribute( "xml:lang" );
    if ( "".equals( s ) ) {
      if ( n.getFPParent() != null )
        return getLang( n.getFPParent() );
      else
        return null;
    } else
      return s;
  }

  /**
   * This method will always return <code>null</code> as the
   * FastParser SimpleNode for v1.6 doesn't includes support for ID
   * @param refNode a reference document element
   * @param id ID value to match
   * @return the node with the unique ID. The ID scope is theorically
   * limited to attribute defined as ID in the DTD
   */
  public Object getNodeForId(Object refNode, String id) {
    return null;
  }

}
